#ifndef NO_SCCS_ID
static char SccsId[ ] = "@(#) sm_free.cpp 1.23 3/25/98 17:25:20"; 
#endif

//////////////////////////////////////////////////////////////////////////
// sm_free.cpp
//
// This CTI Library implements DSA, 3DES, and DH using crypto++
// It will inherit SHA1 from the sm_common CTI
//
//////////////////////////////////////////////////////////////////////////

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <setjmp.h>
#include "sm_free.h"
#include "sm_free_asn.h"

//////////////////////////////////////////////////////////////////////////
SM_RET_VAL CSM_Free::SMTI_Login(void)
{
   // do we need to do anything to login to DH?
   // maybe check key file?  Check if matching certs exist?
   // other stuff as necessary...
   // get certificates and store them in the session?
   return SM_NO_ERROR;
}

//////////////////////////////////////////////////////////////////////////
SM_RET_VAL CSM_Free::SMTI_Sign(
            CSM_Buffer *pData, // input, data to be signed
            CSM_Buffer *pEncryptedDigest, // signature
            CSM_Buffer *pDigest) // digest
{
   SME_SETUP("CSM_Free::SMTI_Sign");
   CSM_Buffer bufferDigest;
   CSM_Buffer *pTempDigest = &bufferDigest;

   if ((pData == NULL) || (pEncryptedDigest == NULL))
      SME_THROW(SM_MISSING_PARAM, NULL, NULL)

   // if pDigest was passed in, use it, otherwise, use local temp
   if (pDigest != NULL)
      pTempDigest = pDigest;

   // digest the incoming data
   // check the preferred digest encryption alg
   // BTIGetPreferredCSInstAlgs(NULL, poidSignature, NULL, NULL);
   //if (poidSignature != id_dsa)
   //   SME_THROW(SM_FREE_UNSUPPORTED_ALG, NULL, NULL);

   SME(SMTI_DigestData(pData, pTempDigest));

   // prep the crypto++ DSA private key
   // m_DSAP, m_DSAQ, m_DSAG all come from the cert
   // m_DSAY comes from the cert, is not really necessary for this operation
   // m_DSAX comes from the EncryptedPrivateKeyInfo
   DSAPrivateKey dsaX(m_DSAP, m_DSAQ, m_DSAG, m_DSAY, m_DSAX);

   // sign
   pEncryptedDigest->Open(SM_FOPEN_WRITE);
   char *pch = pEncryptedDigest->Alloc(SM_FREE_DSA_SIG_LEN);
   dsaX.Sign(*m_pRng, (const byte *)pTempDigest->AccessAll(),
         (unsigned int)pDigest->Length(), (unsigned char *)pch);
   // now, pch has the signature, flush and close
   pEncryptedDigest->Flush();
   pEncryptedDigest->Close();
   
   SME_FINISH
   SME_CATCH_SETUP
      // catch/cleanup logic as necessary
   SME_CATCH_FINISH

   return SM_NO_ERROR;
}

//////////////////////////////////////////////////////////////////////////
SM_RET_VAL CSM_Free::SMTI_Verify(
            CSM_Buffer *pSignerKey, // input
            CSM_Alg    *pDigestAlg, // input
            CSM_Alg    *pSignatureAlg, // input
            CSM_Buffer *pData, // input
            CSM_Buffer *pSignature) // input
{
   CSM_Buffer bufferDigest;
   SM_RET_VAL lRet = SM_NO_ERROR;
   Integer *pP, *pQ, *pG;
   CSM_Buffer *pParams;
   byte *pbyte;

   SME_SETUP("CSM_Free::SMTI_Verify");

   if ((pData == NULL) || (pSignerKey == NULL) || (pSignature == NULL) ||
         (pDigestAlg == NULL) || (pSignatureAlg == NULL))
      SME_THROW(SM_MISSING_PARAM, NULL, NULL)

   // check the signature alg
   if (*pSignatureAlg != id_dsa)
      SME_THROW(SM_FREE_UNSUPPORTED_ALG, NULL, NULL);

   // are there params in pSignatureAlg, if so, use them...
   if ((pParams = pSignatureAlg->GetParams()) != NULL)
   {
      // ASN.1 decode them
      FREE_DSAParameters snaccDSAParameters;
      DECODE_BUF((&snaccDSAParameters), pParams);

      // load parameters
      // TBD, this may not be correct...be carefull...
      pP = new Integer((const unsigned char *)((char*)snaccDSAParameters.p), 
               snaccDSAParameters.p.Len());
      pQ = new Integer((const unsigned char *)((char*)snaccDSAParameters.q), 
               snaccDSAParameters.q.Len());
      pG = new Integer((const unsigned char *)((char*)snaccDSAParameters.g), 
               snaccDSAParameters.g.Len());
   }
   else
   {
      // p, q, and g come from this which was originally from a
      // decoded cert
      pP = new Integer(m_DSAP);
      pQ = new Integer(m_DSAQ);
      pG = new Integer(m_DSAG);
   }

   // prep the crypto++ DSA public key
   // Y comes from pSignerKey
   // convert CSM_Buffers into Integers for use by crypto++
   Integer y;
   pbyte = (byte *)pSignerKey->AccessAll();
   y.BERDecode(pbyte);

   DSAPublicKey dsaY(*pP, *pQ, *pG, y);

   // digest incoming data
   SME(SMTI_DigestData(pData, &bufferDigest));

   // verify the signature
   bool bVer = dsaY.Verify((const byte *)bufferDigest.AccessAll(),
         (unsigned int)bufferDigest.Length(),
         (unsigned char *)pSignature->AccessAll());
   if (!bVer)
      lRet = SM_FREE_VERIFY_FAILED;

   // cleanup pP, pQ, and pG?

   SME_FINISH
   SME_CATCH_SETUP
      // catch/cleanup logic as necessary
      // cleanup pP, pQ, and pG?
   SME_CATCH_FINISH

   return lRet;
}

//////////////////////////////////////////////////////////////////////////
SM_RET_VAL CSM_Free::SMTI_Encrypt(
            CSM_Buffer *pData, // input (data to be encrypted)
            CSM_Buffer *pEncryptedData, // output
            CSM_Buffer *pParameters, // output
            CSM_Buffer *pMEK) // output
{
   char achIn[SM_FREE_3DES_BLOCKSIZE + 1];
   char achOut[SM_FREE_3DES_BLOCKSIZE + 1];
   CSM_Buffer bufferIV;
   SM_RET_VAL lBlockLen;
   // achBuf is used for padding at end of encryption
   char achBuf[SM_FREE_3DES_BLOCKSIZE * 2];

   memset(&achBuf, 0, sizeof(achBuf)); // clear padded buffer

   SME_SETUP("CSM_Free::SMTI_Encrypt");

   // check incoming parameters
   if ((pData == NULL) || (pEncryptedData == NULL) ||
       (pParameters == NULL) || (pMEK == NULL))
      SME_THROW(SM_FREE_MISSING_PARAM, NULL, NULL);

   // generate the MEK
   SME(SMTI_Random(NULL, pMEK, SM_FREE_3DES_KEYLEN));

   // generate a random IV
   SME(SMTI_Random(NULL, &bufferIV, SM_FREE_3DES_IVLEN));

   // create our cipher
   TripleDES_Encryption encryption((const unsigned char*)(*pMEK));
   // create cbc object
   CBCEncryption cbc_encryption(encryption, (const unsigned char*)bufferIV);

   // open input for reading
   SME(pData->Open(SM_FOPEN_READ));

   // open output for writing
   SME(pEncryptedData->Open(SM_FOPEN_WRITE));

   // read input from pData, cbc encrypt it, and write encrypted
   // result to pEncryptedData WHILE we have full blocks
   while ((lBlockLen = pData->cRead(&achIn[0], SM_FREE_3DES_BLOCKSIZE))
         == SM_FREE_3DES_BLOCKSIZE)
   {
      // do cbc process
      cbc_encryption.ProcessBlock((unsigned char *)(&achIn[0]), 
            (unsigned char *)(&achOut[0]));
      // now, write the block to pEncryptedData
      SME(pEncryptedData->Write(&achOut[0], SM_FREE_3DES_BLOCKSIZE));
   }
   // if lBlockLen is 0 then the size of input in pData was on
   // a block boundary, add a full padded block
   if (lBlockLen == 0)
      // create/add padd block...Necessary???
      memset(&achBuf[0], SM_FREE_3DES_BLOCKSIZE, SM_FREE_3DES_BLOCKSIZE);
   // otherwise, we had a smaller block that needs to be padded
   // appropriately
   else
   {
      // copy current buffer into padded buffer
      memcpy(&achBuf[0], &achIn[0], lBlockLen);
      // do padding
      SM_FREE_Pad((&achBuf[0]), lBlockLen);
   }
   // do cbc process
   cbc_encryption.ProcessBlock((unsigned char *)(&achBuf[0]), 
         (unsigned char *)(&achOut[0]));
   // write padded block to pEncryptedData
   SME(pEncryptedData->Write(&achOut[0], SM_FREE_3DES_BLOCKSIZE));

   pData->Close();
   pEncryptedData->Close();

   // ASN.1 encode the parameters
   TDESParameters snaccParams((char*)bufferIV, bufferIV.Length());
   AsnBuf outputBuf;
   char chBuffer[4096];
   outputBuf.Init(&chBuffer[0], 4096);
   outputBuf.ResetInWriteRvsMode();
   snaccParams.BEnc(outputBuf);
   outputBuf.ResetInReadMode();
   CSM_Buffer *pTemp = NULL;
   SM_ReadFromAsnBuf(pTemp, outputBuf, outputBuf.DataLen(), NULL);
   pParameters->ReSet(*pTemp);

   SME_FINISH
   SME_CATCH_SETUP
      // catch/cleanup logic
      // cleanup pMEK, IV, in, out (as necessary)
      // close these if open
      pData->Close();
      pEncryptedData->Close();
   SME_CATCH_FINISH

   return SM_NO_ERROR;
}

//////////////////////////////////////////////////////////////////////////
SM_RET_VAL CSM_Free::SMTI_GenerateEMEK(
            CSM_Buffer *pRecipient, // input, Y of recip
            CSM_Buffer *pParameters, // output
            CSM_Buffer *pMEK, // input, MEK or special phrase
            CSM_Buffer *pEMEK, // output, encrypted MEK
            CSM_Buffer *pUKM) // input/output
{
   unsigned char *pX;
   long lXLen = 0;
   CSM_Buffer bufferIV;
   char achBuf[SM_FREE_3DES_BLOCKSIZE * 2];
   char achIn[SM_FREE_3DES_BLOCKSIZE + 1];
   char achOut[SM_FREE_3DES_BLOCKSIZE + 1];
   CSM_Buffer tempBuffer;
   CSM_Buffer k1, k2, TEK;
   char achCounter1[] = {0x00, 0x00, 0x00, 0x01};
   char achCounter2[] = {0x00, 0x00, 0x00, 0x02};

   memset(&achBuf, 0, sizeof(achBuf)); // clear padded buffer

   SME_SETUP("CSM_Free::SMTI_GenerateEMEK");

   // check incoming parameters
   if ((pRecipient == NULL) || (pEMEK == NULL) ||
         (pParameters == NULL) || (pUKM == NULL) || (pMEK == NULL))
      SME_THROW(SM_MISSING_PARAM, NULL, NULL);

   // check for valid MEK
   if (strncmp(pMEK->AccessAll(), SM_FREE_FORTENC, 
         strlen(SM_FREE_FORTENC)) == 0)
      SME_THROW(SM_FREE_UNSUPPORTED_ALG, 
            "Cannot Protect skipjack MEK", NULL);

   // load current parameters
   Integer p((const unsigned char*)m_ParamP, m_ParamP.Length());
   Integer g((const unsigned char*)m_ParamG, m_ParamG.Length());

   // load current private key X
   pX = ExtractX(lXLen);
   Integer x(pX, lXLen);

   // create dh class and set it's p, g, and x.
   // note, being able to set the x like this requires modification
   // of the DH class in dh.h to make x public.
   DH dhKeyAgreement(p, g);
   dhKeyAgreement.x = x; // this can't be done with the default DH class,
   // TBD, need to find a way to load X into DH without modifying DH's 
   // declaration

   // Create storage for the resulting agreed upon key and clear it
   SecByteBlock ZZ(dhKeyAgreement.AgreedKeyLength());
   memset(ZZ.ptr, 0x00, ZZ.size);

   // generate the agreed upon key from this class (this X and
   // provided parameters) and the recipient's public key Y.
   // put the result in AgreedUponKey
   const unsigned char *pAccessY = 
         (const unsigned char*)(pRecipient->AccessAll());
   dhKeyAgreement.Agree(pAccessY, ZZ);

   // time to generate the TEK
   // generate a random Ra, store in pUKM if we don't already have it
   if (pUKM->AccessAll() == NULL)
      SME(SMTI_Random(NULL, pUKM, SM_FREE_RA_SIZE));
   // concatentate ZZ, encoded oid, counter1, Ra
   SME(tempBuffer.Open(SM_FOPEN_WRITE));
   SME(tempBuffer.Write((char *)ZZ.ptr, ZZ.size));
   SME(tempBuffer.Write((char *)(dES_EDE3_CBC.Str()), dES_EDE3_CBC.Len()));
   SME(tempBuffer.Write(achCounter1, 4));
   SME(tempBuffer.Write((pUKM->AccessAll()), pUKM->Length()));
   tempBuffer.Close();
   // now hash
   SMTI_DigestData(&tempBuffer, &k1);
   // concatentate ZZ, encoded oid, counter2, Ra
   SME(tempBuffer.Open(SM_FOPEN_WRITE));
   SME(tempBuffer.Write((char *)ZZ.ptr, ZZ.size));
   SME(tempBuffer.Write((char *)(dES_EDE3_CBC.Str()), dES_EDE3_CBC.Len()));
   SME(tempBuffer.Write(achCounter2, 4));
   SME(tempBuffer.Write((pUKM->AccessAll()), pUKM->Length()));
   tempBuffer.Close();
   // now hash
   SMTI_DigestData(&tempBuffer, &k2);

   // concatenate k1 and k2 to form 40 byte value, use first 24 bytes as TEK
   SME(TEK.Open(SM_FOPEN_WRITE));
   SME(TEK.Write((char*)k1, k1.Length()));
   SME(TEK.Write((char*)k2, k2.Length()));
   
   // now we need to encrypt the MEK with the TEK
   // generate a random IV
   SME(SMTI_Random(NULL, &bufferIV, SM_FREE_3DES_IVLEN));

   // create our cipher
   // this should only use the first 24 bytes in TEK
   TripleDES_Encryption encryption((const unsigned char*)TEK);
   // create cbc object
   CBCEncryption cbc_encryption(encryption, (const unsigned char*)bufferIV);

   // open input for reading
   SME(pMEK->Open(SM_FOPEN_READ));

   // open output for writing
   SME(pEMEK->Open(SM_FOPEN_WRITE));

   // read input from MEK, cbc encrypt it, and write encrypted
   // result to pEMEK WHILE we have full blocks
   long lBlockLen;
   while ((lBlockLen = pMEK->cRead(&achIn[0], SM_FREE_3DES_BLOCKSIZE))
         == SM_FREE_3DES_BLOCKSIZE)
   {
      // do cbc process
      cbc_encryption.ProcessBlock((unsigned char *)(&achIn[0]), 
            (unsigned char *)(&achOut[0]));
      // now, write the block to pEncryptedData
      SME(pEMEK->Write(&achOut[0], SM_FREE_3DES_BLOCKSIZE));
   }
   // if lBlockLen is 0 then the size of input in m_MEK was on
   // a block boundary, add a full padded block
   if (lBlockLen == 0)
      // create/add padd block...Necessary???
      memset(&achBuf[0], SM_FREE_3DES_BLOCKSIZE, SM_FREE_3DES_BLOCKSIZE);
   // otherwise, we had a smaller block that needs to be padded
   // appropriately
   else
   {
      // copy current buffer into padded buffer
      memcpy(&achBuf[0], &achIn[0], lBlockLen);
      // do padding
      SM_FREE_Pad((&achBuf[0]), lBlockLen);
   }
   // do cbc process
   cbc_encryption.ProcessBlock((unsigned char *)(&achBuf[0]), 
         (unsigned char *)(&achOut[0]));
   // write padded block to pEncryptedData
   SME(pEMEK->Write(&achOut[0], SM_FREE_3DES_BLOCKSIZE));

   pMEK->Close();
   pEMEK->Close();

   // TBD, ASN.1 encode the Ra in pUKM???

   // ASN.1 encode the Parameters
   DHParameters snaccDHParameters;
   snaccDHParameters.p.Set((char*)m_ParamP, m_ParamP.Length());
   snaccDHParameters.g.Set((char*)m_ParamG, m_ParamG.Length());
   snaccDHParameters.iv.Set((char*)bufferIV, bufferIV.Length());
   ENCODE_BUF_NO_ALLOC((&snaccDHParameters), pParameters);

   SME_FINISH
   SME_CATCH_SETUP
      // catch/cleanup logic
      // figure out what we need to clean up and do it...
      // cleanup bufferIV?
      // close pMEK and pEMEK if necessary
   SME_CATCH_FINISH

   return SM_NO_ERROR;
}

//////////////////////////////////////////////////////////////////////////
SM_RET_VAL CSM_Free::SMTI_Decrypt(
            CSM_Buffer *pParameters, // input (initialization vector)
            CSM_Buffer *pEncryptedData, // input (data to be decrypted)
            CSM_Buffer *pMEK, // input (MEK or special phrase)
            CSM_Buffer *pData) // output (decrypted data)
{
   char *pIV;
   char achIn[SM_FREE_3DES_BLOCKSIZE + 1];
   char achInNext[SM_FREE_3DES_BLOCKSIZE + 1];
   char achOut[SM_FREE_3DES_BLOCKSIZE + 1];
   SM_RET_VAL lBlockLen;
   SM_RET_VAL lNextBlockLen = 0;

   SME_SETUP("CSM_Free::SMTI_Decrypt");

   // check incoming parameters
   if ((pData == NULL) || (pEncryptedData == NULL) || (pParameters == NULL)
         || (pMEK == NULL))
      SME_THROW(SM_FREE_MISSING_PARAM, NULL, NULL);

   // check for valid MEK
   if (strncmp(pMEK->AccessAll(), SM_FREE_FORTENC, 
         strlen(SM_FREE_FORTENC)) == 0)
      SME_THROW(SM_FREE_UNSUPPORTED_ALG, 
            "Cannot use skipjack MEK", NULL);

   // create our cipher
   TripleDES_Decryption decryption((const unsigned char*)(*pMEK));

   // ASN.1 decode the parameters that contain the IV
   AsnBuf snaccBuffer;
   snaccBuffer.InstallData(pParameters->CopyAll(), pParameters->Length());
   TDESParameters snaccParams;
   AsnLen bytesDecoded;
   jmp_buf mark;
   if (setjmp(mark) == 0)
      snaccParams.BDec(snaccBuffer, bytesDecoded, mark);
   else
      SME_THROW(SM_FREE_PARAM_DEC_ERROR, NULL, NULL);

   pIV = (char*)snaccParams;
   
   // create cbc object
   CBCDecryption cbc_decryption(decryption, (const unsigned char *)pIV);

   // open input for reading
   SME(pEncryptedData->Open(SM_FOPEN_READ));

   // open output for writing
   SME(pData->Open(SM_FOPEN_WRITE));

   // read input from pEncryptedData, cbc decrypt it, and write decrypted
   // result to pData WHILE we have full blocks
   // ...read first block then try to read the next block
   if ((lBlockLen = pEncryptedData->cRead(&achIn[0], SM_FREE_3DES_BLOCKSIZE))
            == SM_FREE_3DES_BLOCKSIZE)
      lNextBlockLen = pEncryptedData->cRead(&achInNext[0], 
            SM_FREE_3DES_BLOCKSIZE);
   while (lBlockLen == SM_FREE_3DES_BLOCKSIZE)
   {
      // do cbc process
      cbc_decryption.ProcessBlock((unsigned char *)(&achIn[0]), 
            (unsigned char *)(&achOut[0]));
      // do we currently have the last block?
      if (lNextBlockLen != SM_FREE_3DES_BLOCKSIZE)
      {
         // yes, what is in achIn is the final block and includes padding
         // that needs to be removed before we write
         if ((achOut[SM_FREE_3DES_BLOCKSIZE -1] < 0x01) || 
               (achOut[SM_FREE_3DES_BLOCKSIZE -1] > 0x08))
            SME_THROW(SM_FREE_DECRYPT_PAD_ERROR, "Invalid pad block data", 
                  NULL);
         if (achOut[SM_FREE_3DES_BLOCKSIZE - 1] != 0x08)
            SME(pData->Write(&achOut[0], 
                  (SM_FREE_3DES_BLOCKSIZE - achOut[SM_FREE_3DES_BLOCKSIZE - 1])));
         lBlockLen = lNextBlockLen;
      }
      else
      {
         // no, what is in achIn is not the final block, write it all
         // and move the next block into achIn
         SME(pData->Write(&achOut[0], SM_FREE_3DES_BLOCKSIZE));
         memcpy(&achIn[0], &achInNext[0], SM_FREE_3DES_BLOCKSIZE);
         // read the next block
         SME(lNextBlockLen = pEncryptedData->cRead(&achInNext[0],
               SM_FREE_3DES_BLOCKSIZE));
      }
   }

   pData->Close();
   pEncryptedData->Close();

   SME_FINISH
   SME_CATCH_SETUP
      // catch/cleanup logic
      // cleanup pMEK, in, out
      // close these if open
      pData->Close();
      pEncryptedData->Close();
   SME_CATCH_FINISH

   return SM_NO_ERROR;
}

//////////////////////////////////////////////////////////////////////////
SM_RET_VAL CSM_Free::SMTI_ExtractMEK(
            CSM_Buffer *pOriginator, // input, Y of originator
            CSM_Buffer *pParameters, // input
            CSM_Buffer *pEMEK, // input, encrypted MEK
            CSM_Buffer *pUKM, // input
            CSM_Buffer *pMEK) // output (MEK or special phrase)
{
   unsigned char *pX;
   long lXLen = 0;
   char achIn[SM_FREE_3DES_BLOCKSIZE + 1];
   char achOut[SM_FREE_3DES_BLOCKSIZE + 1];
   CSM_Buffer tempBuffer;
   CSM_Buffer k1, k2, TEK;
   char achCounter1[] = {0x00, 0x00, 0x00, 0x01};
   char achCounter2[] = {0x00, 0x00, 0x00, 0x02};
   long lBlockLen = 0;

   SME_SETUP("CSM_Free::SMTI_ExtractMEK");

   // check incoming parameters
   if ((pOriginator == NULL) || (pEMEK == NULL) ||
         (pParameters == NULL) || (pUKM == NULL) || (pMEK == NULL))
      SME_THROW(SM_MISSING_PARAM, NULL, NULL);

   // ASN.1 decode the parameters
   DHParameters snaccDHParameters;
   DECODE_BUF((&snaccDHParameters), pParameters);

   // load parameters
   Integer p((const unsigned char *)((char*)snaccDHParameters.p), 
            snaccDHParameters.p.Len());
   Integer g((const unsigned char *)((char*)snaccDHParameters.g), 
            snaccDHParameters.g.Len());

   // load current private key X
   pX = ExtractX(lXLen);
   Integer x(pX, lXLen);

   // create dh class and set it's p, g, and x.
   // note, being able to set the x like this requires modification
   // of the DH class in dh.h to make x public.
   DH dhKeyAgreement(p, g);
   dhKeyAgreement.x = x; // this can't be done with the default DH class,
   // TBD, need to find a way to load X into DH without modifying DH's 
   // declaration

   // Create storage for the resulting agreed upon key and clear it
   SecByteBlock ZZ(dhKeyAgreement.AgreedKeyLength());
   memset(ZZ.ptr, 0x00, ZZ.size);

   // generate the agreed upon key from this class (this X and
   // provided parameters) and the recipient's public key Y.
   // put the result in AgreedUponKey
   const unsigned char *pAccessY = 
         (const unsigned char*)(pOriginator->AccessAll());
   dhKeyAgreement.Agree(pAccessY, ZZ);

   // time to generate the TEK
   // UKM is stored in the incoming pUKM
   // TBD, do we need to ASN.1 decode it???

   // concatentate ZZ, encoded oid, counter1, Ra
   SME(tempBuffer.Open(SM_FOPEN_WRITE));
   SME(tempBuffer.Write((char *)ZZ.ptr, ZZ.size));
   SME(tempBuffer.Write((char *)(dES_EDE3_CBC.Str()), dES_EDE3_CBC.Len()));
   SME(tempBuffer.Write(achCounter1, 4));
   SME(tempBuffer.Write((pUKM->AccessAll()), pUKM->Length()));
   tempBuffer.Close();
   // now hash
   SMTI_DigestData(&tempBuffer, &k1);
   // concatentate ZZ, encoded oid, counter2, Ra
   SME(tempBuffer.Open(SM_FOPEN_WRITE));
   SME(tempBuffer.Write((char *)ZZ.ptr, ZZ.size));
   SME(tempBuffer.Write((char *)(dES_EDE3_CBC.Str()), dES_EDE3_CBC.Len()));
   SME(tempBuffer.Write(achCounter2, 4));
   SME(tempBuffer.Write((pUKM->AccessAll()), pUKM->Length()));
   tempBuffer.Close();
   // now hash
   SMTI_DigestData(&tempBuffer, &k2);

   // concatenate k1 and k2 to form 40 byte value, use first 24 bytes as TEK
   SME(TEK.Open(SM_FOPEN_WRITE));
   SME(TEK.Write((char*)k1, k1.Length()));
   SME(TEK.Write((char*)k2, k2.Length()));
   
   // now we need to decrypt the EMEK with the TEK
   // The IV comes from the parameters: snaccDHParameters.iv
   // create our cipher
   TripleDES_Decryption decryption((const unsigned char*)TEK);
   // create cbc object
   CBCDecryption cbc_decryption(decryption, 
         (const unsigned char *)((char*)snaccDHParameters.iv));

   // open input (encrypted MEK) for reading
   SME(pEMEK->Open(SM_FOPEN_READ));

   // open output (MEK) for writing
   SME(pMEK->Open(SM_FOPEN_WRITE));

   // read input from pEMEK, cbc decrypt it, and write decrypted
   // result to pMEK WHILE we have full blocks
   while ((lBlockLen = pEMEK->cRead(&achIn[0], 
         SM_FREE_3DES_BLOCKSIZE)) == SM_FREE_3DES_BLOCKSIZE)
   {
      // do cbc process
      cbc_decryption.ProcessBlock((unsigned char *)(&achIn[0]), 
            (unsigned char *)(&achOut[0]));
      // now, write the block to pData
      SME(pMEK->Write(&achOut[0], SM_FREE_3DES_BLOCKSIZE));
   }
   // TBD, we should have padding in pData now, remove it...

   pMEK->Close();
   pEMEK->Close();

   SME_FINISH
   SME_CATCH_SETUP
      // catch/cleanup logic
      // figure out what we need to clean up and do it...
      // close pMEK and pEMEK if necessary
   SME_CATCH_FINISH
   return SM_NO_ERROR;
}

//////////////////////////////////////////////////////////////////////////
// dave, this func modified to use common 
SM_RET_VAL CSM_Free::SMTI_DigestData(
            CSM_Buffer *pData, // input
            CSM_Buffer *pDigest) // output
{
   CSM_OID *poidDigest;
   HashModule *pHash;
   char *pchData;
   long lBytesRead;
   long lHashDataSize;

   SME_SETUP("CSM_Free::SMTI_DigestData");

   // find out what the preferred digest alg is
   if ((poidDigest = GetPrefDigest()) == NULL)
      SME_THROW(SM_FREE_NO_DIGEST_ALG, NULL, NULL);

   if (*poidDigest == id_sha1)
   {
      // do SHA1 digest using common CTI
      SME(CSM_Common::SMTI_DigestData(pData, pDigest));
   }
   else if (*poidDigest == id_md5)
   {
      // do MD5 digest
      pHash = new MD5;
      lHashDataSize = MD5::DATASIZE;

      // now use pHash to do the actual hash/digest
      // open the input and block through it
      SME(pData->Open(SM_FOPEN_READ));
      while (true)
      {
         SME(pchData = pData->nRead(lHashDataSize, (SM_SIZE_T&)lBytesRead));
         if ((lBytesRead != lHashDataSize) || (pchData == NULL))
            break;
         pHash->Update((const unsigned char *)pchData, lBytesRead);
      }
      // TBD, do padding and pHash->Update of last block

      pData->Close(); // close the incoming data

      // create storage for the digest
      SecByteBlock digest(pHash->DigestSize());

      pHash->Final(digest); // finish the digest
      SME(pDigest->Open(SM_FOPEN_WRITE)); // open the digest buffer
      SME(pDigest->Write((char *)digest.ptr, digest.size)); // write the digest
      pDigest->Close(); // close the digest buffer
   }
   else
      SME_THROW(SM_FREE_UNSUPPORTED_ALG, NULL, NULL);

   SME_FINISH
   SME_CATCH_SETUP
      // catch/cleanup logic
      // close pData and pDigest as necessary
   SME_CATCH_FINISH

   return SM_NO_ERROR;
}

//////////////////////////////////////////////////////////////////////////
SM_RET_VAL CSM_Free::SMTI_Random(
            CSM_Buffer *pSeed,   // input
            CSM_Buffer *pRandom, // input/output
            SM_SIZE_T lLength)   // input
{
   char *p = NULL;

   SME_SETUP("CSM_Free::SMTI_Random");

   // TBD:  Use pSeed

   if (pRandom == NULL)
      SME_THROW(SM_FREE_MISSING_PARAM, NULL, NULL);

   // open the buffer
   SME(pRandom->Open(SM_FOPEN_WRITE));
   // allocate memory for use in the buffer
   SME(p = pRandom->Alloc(lLength));

   // create lLength random bytes of data
   m_pRng->GetBlock((unsigned char *)p, lLength);
#ifdef FREE_NON_RANDOM // TBD, used for interop testing
   long lCount;
   for (lCount = 0; lCount < lLength; lCount++)
      p[lCount] = 0x33;
#endif

   // flush and close
   pRandom->Flush();
   pRandom->Close();

   SME_FINISH
   SME_CATCH_SETUP
      // cleanup/catch logic goes here...
      // TBD, p may need to be cleaned up???
   SME_CATCH_FINISH

   return SM_NO_ERROR;
}

//////////////////////////////////////////////////////////////////////////
char* CSM_Free::GetPassword()
{
   return (strdup(m_pszPassword));
}

//////////////////////////////////////////////////////////////////////////
void CSM_Free::SetPassword(char *pszPassword)
{
   m_pszPassword = strdup(pszPassword);
}

//////////////////////////////////////////////////////////////////////////
MAB_AB_def* CSM_Free::GetAB()
{
   return m_pAB;
}

//////////////////////////////////////////////////////////////////////////
void CSM_Free::SetAB(MAB_AB_def *pAB)
{
   m_pAB = pAB;
}

//////////////////////////////////////////////////////////////////////////
void CSM_Free::FreeAB()
{
   if (m_pAB != NULL)
      delete m_pAB;
}

//////////////////////////////////////////////////////////////////////////
// This function scans all the instances in pCSMIME to find out if there
// is only one reference to this Address Book, if so, it deletes it, other
// wise, it assumes other instances are using it and so it does not delete
// it
void CSM_Free::FreeAB(CSMIME *pCSMIME)
{
   bool bFound = false;
   CSM_Free *pCurrentFree;

   SME_SETUP("CSM_Free::FreeAB");

   if (pCSMIME == NULL)
      SME_THROW(SM_MISSING_PARAM, NULL, NULL);

   if (pCSMIME->m_pCSInsts == NULL)
      FreeAB();
   else
   {
      CSM_CSInstLst *pTmp = pCSMIME->m_pCSInsts->First();
      do
      {
         // make sure we're not in the current instance
         if (this != pTmp->Get()->GetTokenInterface())
         {
            // try to determine if the current instance is the same
            // type as this instance based on ID preffix
            if (strncmp(pTmp->Get()->GetID(), m_pszPreffix, 
                  strlen(m_pszPreffix)) == 0)
            {
               // same ID preffix, go for the gusto, use it as a Free CTI
               // potential problem, if this isn't a free CTI, bad news...
               pCurrentFree = (CSM_Free *)(pTmp->Get()->GetTokenInterface());
               // if the current AB equals this AB
               if (pCurrentFree->GetAB() == GetAB())
               {
                  // then there are two instances referencing
                  // this AB, we can't delete it.
                  bFound = true;
               }
            }
         }
      } while ((pTmp = pTmp->Next()) != NULL);

      if (!bFound)
      {
         delete (GetAB());
         SetAB(NULL);
      }
   }

   SME_FINISH
   SME_CATCH_SETUP
      // TBD catch/cleanup logic as necessary
   SME_CATCH_FINISH
}

//////////////////////////////////////////////////////////////////////////
CSM_Free::~CSM_Free()
{
   if (m_pszPassword != NULL)
      free (m_pszPassword);
   // DO NOT delete m_pAB, must manually do it with FreeAB because more
   // than one instance can reference the same AB
   if (m_pRandomCipher != NULL)
      delete m_pRandomCipher;
   //if (m_pRng != NULL)
   //   delete m_pRng;  // TBD, this crashes???
   if (m_pszPreffix != NULL)
      free(m_pszPreffix);
}

//////////////////////////////////////////////////////////////////////////
CSM_Free::CSM_Free()
{
   time_t t;
   time(&t); // use time to seed rng
   // store it in the seed member
   char *p = (char *)calloc(1, 128);
   m_seed.Set(p, (SM_SIZE_T) 4);
   memcpy((char*)m_seed, &t, sizeof(time_t));
   // use this key for the random cipher...TBD???
   byte rngKey[] = {0xab, 0xbc, 0xcd, 0xde, 0xef, 0x12, 0x23, 0x77};

   // clear other members
   m_pszPassword = NULL;
   m_pszPreffix = NULL;
   m_pAB = NULL;

   // construct random components
   m_pRandomCipher = new DES(&rngKey[0], ENCRYPTION);;
   m_pRng = new X917RNG(m_pRandomCipher, (unsigned char *)m_seed);

   // set up algs
   SetDefaultOIDs();
}

//////////////////////////////////////////////////////////////////////////
#ifdef SEED_CODE_IN_PLACE
CSM_Free::CSM_Free(char *pszSeed)
{
   // store provided seed in the seed member
   SM_SIZE_T l = strlen(pszSeed);
   char *p = (char *)calloc(1, l);
   m_seed.Set(p, l);
   strcpy((char*)m_seed, pszSeed);
   // use this key for the random cipher...TBD???
   byte rngKey[] = {0xab, 0xbc, 0xcd, 0xde, 0xef, 0x12, 0x23, 0x77};

   // clear other members
   m_pszPassword = NULL;
   m_pszPreffix = NULL;
   m_pAB = NULL;

   // construct random components
   m_pRandomCipher = new DES(&rngKey[0], ENCRYPTION);;
   m_pRng = new X917RNG(m_pRandomCipher, (unsigned char *)m_seed);

   // set up algs
   SetDefaultOIDs();
}
#endif

//////////////////////////////////////////////////////////////////////////
// pszSeed can be NULL
SM_RET_VAL SMFreeInit(CSMIME *pCSMIME, char *pszPassword,
                     char *pszAddressBook, char *pszPreffix)
{
   CSM_CSInst *pNewInstance = NULL;
   CSM_List<MAB_Entrydef> *pEntry = NULL;
   CSM_Free *pFree = NULL;
   CSM_CertificateChoice *pCertificateChoice;
   CSM_Buffer *pBuffer;
   long lCounter = 0;
   char szID[128];
   CSM_AlgLst *pDigestAlgs=NULL;
   CSM_AlgLst *pKeyEncryption=NULL;
   CSM_AlgLst *pDigestEncryption=NULL;
   CSM_AlgLst *pContentEncryption=NULL;
   AlgorithmIdentifier *pAlgID=NULL;


   SME_SETUP("SMFreeInit");

   if ((pCSMIME == NULL) || (pszPassword == NULL) || 
         (pszAddressBook == NULL) || (pszPreffix == NULL))
      SME_THROW(SM_MISSING_PARAM, NULL, NULL);

   // initialize an address book from 
   MAB_AB_def *pAB = new MAB_AB_def(pszAddressBook);
   if ((pAB == NULL) || (pAB->m_pEntries == NULL))
      SME_THROW(SM_MAB_ERROR, "address book init failed", NULL);

   // now, go through the address book and build an instance for each
   // applicable entry
   pEntry = pAB->m_pEntries->First();
   do
   {
      if ((pEntry->Get())->m_pPrivateOID != NULL) 
      {
         if ((*(pEntry->Get())->m_pPrivateOID == id_dh_public_number) ||
               (*(pEntry->Get())->m_pPrivateOID == id_dsa))
         {
            // dh or dsa entry
            // create an instance
            if ((pNewInstance = new CSM_CSInst) == NULL)
               SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
            // put it in the instance list in pCSMIME
            if (pCSMIME->m_pCSInsts == NULL)
            {
               if ((pCSMIME->m_pCSInsts = new CSM_CSInstLst(pNewInstance)) 
                        == NULL)
                  SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
            }
            else
               pCSMIME->m_pCSInsts->Append(pNewInstance);
            // generate a new FREE CTI class
            if ((pFree = new CSM_Free) == NULL)
               SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
            // CSM_Free constructor set's Alg IDs

            // store other information in the Free CTI
            pFree->SetPassword(pszPassword);
            pFree->SetAB(pAB);
            // store the private key info as either DH or DSA private key
            if (*(pEntry->Get())->m_pPrivateOID == id_dh_public_number)
               pFree->m_X = *((pEntry->Get())->m_pPrivateInfo);
            else
            {
               byte *pbyte = (byte *)((pEntry->Get())->m_pPrivateInfo)->AccessAll();
               pFree->m_DSAX.BERDecode(pbyte);
            }
            // store parameters and Y in the preferred Alg for this instance
            pAlgID = NULL;
            SME(pFree->GetParamsAndY(pEntry->Get(), pAB, pAlgID));

            // now, fill in what we can in the instance
            // store token interface pointer
            pNewInstance->SetTokenInterface((CSM_TokenInterface *)pFree);
            // set an id
            sprintf(szID, "%s%ld", pszPreffix, lCounter);
            ++lCounter; // increment counter
            // TBD, scan all instances in pCSMIME to make sure this is a
            // unique ID
            pNewInstance->SetID(&szID[0]);
            // store the preffix
            pFree->m_pszPreffix = strdup(pszPreffix);
            // copy the CSMIME error buf
            pNewInstance->m_pErrorBuf = pCSMIME->GetErrorBuf();
            // store certificate
            if ((pBuffer = new CSM_Buffer(*(pEntry->Get()->m_pCertFile))) 
                     == NULL)
               SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
            if ((pCertificateChoice = new CSM_CertificateChoice(pBuffer)) 
                     == NULL)
               SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
            // delete buffer since it was copied into pCertificateChoice
            delete (pBuffer); 
            pNewInstance->SetCertificates(new 
                  CSM_CertificateChoiceLst(pCertificateChoice));
            // store issuer and serial number
            pNewInstance->SetIssuerAndSerialNumber(
                     pEntry->Get()->GetIssuer());
            // store crls??? TBD


            // RWC; Set custom parameters from cert algorithm if necessary.
            // pAlgID was set by GetParamsAndY.  We store the parameters in
            // the instance so they may be used as necessary later on
            pFree->BTIGetAlgIDs(&pDigestAlgs, &pDigestEncryption, 
                &pKeyEncryption, &pContentEncryption);   // DO NOT FREE!!!!
            if (pAlgID) // only do this if there is something in pAlgID
            {
               CSM_AlgLst *tmpAlgs;
               if (pAlgID->algorithm == id_dsa) // do DSA side
               {
                  // loop through digest encryption algs until we find DSA
                  for (tmpAlgs=pDigestEncryption->First(); 
                        tmpAlgs && *tmpAlgs->Get()->m_pId != id_dsa; 
                        tmpAlgs=tmpAlgs->Next());
                  if (tmpAlgs != NULL)
                  {
                     // if we found one, store the algorithm OID from pAlgID
                     *tmpAlgs->Get()->m_pId = pAlgID->algorithm; 
                     // and then extract and store the parameters from pAlgID
                     SM_EXTRACT_ANYBUF(tmpAlgs->Get()->m_pParams, 
                           pAlgID->parameters);
                     // since this instance will never do key encryption,
                     // we can clear that part
                     pFree->ClearKeyEncryptionCapability();
                     // reset digest encryption (which has parameters added),
                     pFree->BTISetAlgIDs(NULL, pDigestEncryption, 
                           NULL, NULL);
                  }
               }
            } // ENDIF pAlgID
         }
      }
   } while ((pEntry = pEntry->Next()) != NULL);

   if (pAB)
      delete pAB;

   SME_FINISH
   SME_CATCH_SETUP
      // catch/cleanup logic as necessary
   SME_CATCH_FINISH

   return SM_NO_ERROR;
}

//////////////////////////////////////////////////////////////////////////
// This function restores or sets the default OIDs int he BTI
void CSM_Free::SetDefaultOIDs()
{
   // TBD, these are probably not right
   // put the AsnOids in CSM_OIDs
   CSM_OID oidSha1(id_sha1);
   CSM_OID oidMd5(id_md5);
   CSM_OID oidDh(id_dhStatic);
   CSM_OID oidDhPublicNumber(id_dh_public_number);
   CSM_OID oidDsa(id_dsa);
   CSM_OID oidTdes(dES_EDE3_CBC);

   // put the CSM_OIDs in CSM_Algs
   // TBD, add parameters?
   CSM_Alg *palgDigestSha = new CSM_Alg(oidSha1);
   CSM_Alg *palgDigestMd5 = new CSM_Alg(oidMd5);
   CSM_Alg *palgKeyEncryption = new CSM_Alg(oidDh); // key agreement?
   CSM_Alg *palgDHPublicNumber = new CSM_Alg(oidDhPublicNumber);
   CSM_Alg *palgDigestEncryption = new CSM_Alg(oidDsa);
   CSM_Alg *palgContentEncryption = new CSM_Alg(oidTdes);

   // put the CSM_Algs in CSM_AlgLsts
   CSM_AlgLst *pDigestAlgs = new CSM_AlgLst(palgDigestSha);
   pDigestAlgs->Append(palgDigestMd5);
   CSM_AlgLst *pKeyEncryption = new CSM_AlgLst(palgKeyEncryption);
   pKeyEncryption->Append(palgDHPublicNumber);
   CSM_AlgLst *pDigestEncryption = new CSM_AlgLst(palgDigestEncryption);
   CSM_AlgLst *pContentEncryption = new CSM_AlgLst(palgContentEncryption);

   // put the CSM_AlgLsts in the base token interface
   BTISetAlgIDs(pDigestAlgs, pDigestEncryption, pKeyEncryption, 
         pContentEncryption);
   // make sha1, dsa, dh, and tdes the preferred algs
   BTISetPreferredCSInstAlgs(&oidSha1, &oidDsa, &oidDh, &oidTdes);
}

//////////////////////////////////////////////////////////////////////////
// Extract X returns a ptr to the current private key
// and sets the length in lXLen
unsigned char * CSM_Free::ExtractX(long &lXLen)
{
   // TBD, currently just pulls from m_X
   lXLen = m_X.Length();
   return ((unsigned char *)m_X.AccessAll());
}

//////////////////////////////////////////////////////////////////////////
// ExtractParams ASN.1 decodes the provide any and sets the values into
// this object as appropriate
SM_RET_VAL CSM_Free::ExtractParams(AlgorithmIdentifier *pAlgID)
{
   char *pch;

   // TBD, fully implement error handling

   SME_SETUP("CSM_Free::ExtractParams");

   if ((pAlgID == NULL) || (pAlgID->parameters == NULL))
      SME_THROW(SM_MISSING_PARAM, NULL, NULL);

   if (pAlgID->algorithm == id_dh_public_number)
   {
      // ASN.1 decode pParams as DH parameters
      // store them in m_ParamP and m_ParamG
      DHParameters snaccDHParams;
      DECODE_ANY((&snaccDHParams), (pAlgID->parameters));
      // extract P
      m_ParamP.Open(SM_FOPEN_WRITE);
      pch = m_ParamP.Alloc(snaccDHParams.p.Len());
      memcpy(pch, (char*)snaccDHParams.p, snaccDHParams.p.Len());
      m_ParamP.Flush();
      m_ParamP.Close();
      // extract G
      m_ParamG.Open(SM_FOPEN_WRITE);
      pch = m_ParamG.Alloc(snaccDHParams.g.Len());
      memcpy(pch, (char*)snaccDHParams.g, snaccDHParams.g.Len());
      m_ParamG.Flush();
      m_ParamG.Close();
   }
   else if (pAlgID->algorithm == id_dsa)
   {
      byte *pbyte;
      // ASN.1 decode pParams as DSA parameters
      // store them in m_DSAP, m_DSAQ, and m_DSAG
      FREE_DSAParameters snaccDSAParams;
      DECODE_ANY((&snaccDSAParams), (pAlgID->parameters));
      // extract P
      pbyte = (byte *)((char*)snaccDSAParams.p);
      m_DSAP.Decode(pbyte, snaccDSAParams.p.Len());
      // extract Q
      pbyte = (byte *)((char*)snaccDSAParams.q);
      m_DSAQ.Decode(pbyte, snaccDSAParams.q.Len());
      // extract G
      pbyte = (byte *)((char*)snaccDSAParams.g);
      m_DSAG.Decode(pbyte, snaccDSAParams.g.Len());
   }
   else
      SME_THROW(SM_FREE_UNSUPPORTED_ALG, NULL, NULL);

   SME_FINISH
   SME_CATCH_SETUP
      // catch/cleanup logic as necessary
   SME_CATCH_FINISH

   return SM_NO_ERROR;
}

//////////////////////////////////////////////////////////////////////////
// DecodeCertificate accepts a pointer to a Certificate that will
// receive the decoded certificate.  It needs the buffer containing
// the encoded certificate.  It returns a pointer to the issuer,
// a pointer to the subject key info alg id, and a pointer to the subject
// public key.
SM_RET_VAL CSM_Free::DecodeCertificate(CSM_Buffer *pEncodedCert,
      Certificate *pSnaccCertificate, Name **ppIssuer,
      AlgorithmIdentifier **ppAlgID, AsnBits **ppY)
{
   SME_SETUP("CSM_Free::DecodeCertificate");

   if ((pEncodedCert == NULL) || (pSnaccCertificate == NULL)
         || (ppIssuer == NULL) || (ppAlgID == NULL) ||
         (ppY == NULL))
      SME_THROW(SM_MISSING_PARAM, NULL, NULL);

   // decode the certificate
   DECODE_BUF(pSnaccCertificate, pEncodedCert);

   if (pSnaccCertificate->certificateToSign == NULL)
      SME_THROW(SM_CERT_DEC_ERROR, "Certificate missing CTS", NULL);
   if (pSnaccCertificate->certificateToSign->subjectPublicKeyInfo == NULL)
      SME_THROW(SM_CERT_DEC_ERROR, "Certificate missing subj pub key info",
            NULL);
   *ppAlgID = new AlgorithmIdentifier;  // re-used elsewhere.
   **ppAlgID = *pSnaccCertificate->certificateToSign->subjectPublicKeyInfo->
            algorithm;
   if (*ppAlgID == NULL)
      SME_THROW(SM_CERT_DEC_ERROR, "Certificate missing subj pub key alg",
            NULL);
   *ppIssuer = pSnaccCertificate->certificateToSign->issuer;
   if (*ppIssuer == NULL)
      SME_THROW(SM_CERT_DEC_ERROR, "Certificate missing issuer", NULL);
   *ppY = &(pSnaccCertificate->certificateToSign->subjectPublicKeyInfo->
         subjectPublicKey);

   SME_FINISH
   SME_CATCH_SETUP
      // catch/cleanup logic as necessary
   SME_CATCH_FINISH

   return SM_NO_ERROR;
}

//////////////////////////////////////////////////////////////////////////
// GetParamsAndY accepts an address book entry and the address book as 
// input parameters.  It decodes the certificate referenced by the
// address book entry and compares the OID in the parameters with the
// OIDs in this instance of CSM_Free.  If this cert is a DH cert, then
// it extracts the parameters and puts them into the preferred key
// encryption CSM_Alg.  If this cert is a DSA cert, then it gets
// the parameters by going through the DSA cert chain (as necessary,
// searching the address book for issuers) and then puts the
// parameters into the preferred digest encryption CSM_Alg.
// It also extracts and stores the Y value from the cert
SM_RET_VAL CSM_Free::GetParamsAndY(MAB_Entrydef *pEntry, MAB_AB_def *pAB,
    AlgorithmIdentifier *&pAlgID)
{
   SME_SETUP("CSM_Free::GetParamsAndY");
   bool bUserMustHaveParams;
   Name *pIssuer;
   AsnBits *pY;
   Certificate snaccCertificate;
   Certificate snaccIssuer;
   CSM_Buffer bufferCert;

   // check incoming parameters
   if ((pEntry == NULL) || (pAB == NULL))
      SME_THROW(SM_MISSING_PARAM, NULL, NULL);

   // make a copy of the cert buffer and convert it to memory if necessary
   bufferCert = *(pEntry->m_pCertFile);
   bufferCert.ConvertFileToMemory();

   // decode the certificate
   SME(DecodeCertificate(&bufferCert, &snaccCertificate, &pIssuer,
         &pAlgID, &pY));

   // look at the subject public key OID to see if it is DH, if so, then
   // the parameters must be here, if not, then they may be here or they
   // may be in any one of the issuers...also, store the Y value
   if (pAlgID->algorithm == id_dh_public_number)
   {
      bUserMustHaveParams = true;
      // we never need our own Y value for DH
   }
   else if (pAlgID->algorithm == id_dsa)
   {
      bUserMustHaveParams = false;
      // Put the Y value (pY) in bufferTemp
      CSM_Buffer bufferTemp;
      if (SM_AsnBits2Buffer(pY, &bufferTemp) != SM_NO_ERROR)
         SME_THROW(SM_FREE_PUT_Y_ERROR, "couldn't convert Y", NULL);
      SME(SM_BufferReverseBits(&bufferTemp));   // Reverse bits for proper alignment. 
      byte *pbyte = (byte *)bufferTemp.AccessAll();
      m_DSAY.BERDecode(pbyte);
   }
   else
      SME_THROW(SM_FREE_UNSUPPORTED_ALG, "Cert is not DSA or DH", NULL);

   // extract (ASN.1 decode) the parameters and store them in this object
   if (!bUserMustHaveParams)
   {
      // find the parameters and store them
      while (pAlgID->parameters == NULL)
      {
         // find the parameters by climbing the chain
         // find the issuer of this cert in the address book
         // TBD, there is a potential problem here in that this code
         // finds the issuer in the local address book based only on the 
         // issuer DN...use serial number necessary???
         CSM_DN dnIssuer(*pIssuer);
         CSM_Buffer bufferIssuerCert;
         if (pAB->FindCertDN(dnIssuer.GetDNString(), 
               &bufferIssuerCert) != SM_NO_ERROR)
            SME_THROW(SM_FREE_MAB_NO_ISSUER, "couldn't find issuer in mab",
                  NULL);
         // decode the issuer
         SME(DecodeCertificate(&bufferIssuerCert, &snaccIssuer,
               &pIssuer, &pAlgID, &pY));
         // issuer must be DSA
         if (pAlgID->algorithm != id_dsa)
            SME_THROW(SM_FREE_ISSUER_NOT_DSA, NULL, NULL);
      }
   }
   SME(ExtractParams(pAlgID));

   SME_FINISH
   SME_CATCH_SETUP
      // catch/cleanup logic as necessary
   SME_CATCH_FINISH

   return SM_NO_ERROR;
}

//////////////////////////////////////////////////////////////////////////
// This CTI will override BTIFindAlgIds because in CEA and KEA's
// case, we only want to compare OIDs and are not interested in 
// comparing the parameters part of the AlgId
bool CSM_Free::BTIFindAlgIds(CSM_Alg *pdigestAlgID, 
            CSM_Alg *pdigestEncryptionAlgID,
            CSM_Alg *pkeyEncryptionAlgID,
            CSM_Alg *pcontentEncryptionAlgID)
{
   CSM_Alg *ptmpCEAlgID = NULL;
   CSM_Alg *ptmpKEAlgID = NULL;
   bool bRet = false;

   SME_SETUP("CSM_Free:BTIFindAlgIds");

   // if we have a content encryption AlgId, create a temporary AlgID with
   // only the OID from the one we got from the caller
   if (pcontentEncryptionAlgID)
      if ((ptmpCEAlgID = new CSM_Alg(*(pcontentEncryptionAlgID->m_pId))) 
            == NULL)
         SME_THROW(SM_MEMORY_ERROR, "couldn't duplicate CEA OID", NULL);

   // if we have a key encryption AlgId, create a temporary AlgID with
   // only the OID from the one we got from the caller
   if (pkeyEncryptionAlgID)
      if ((ptmpKEAlgID = new CSM_Alg(*(pkeyEncryptionAlgID->m_pId))) 
            == NULL)
         SME_THROW(SM_MEMORY_ERROR, "couldn't duplicate KEA OID", NULL);

   // call the lolevel BTIFindAlgIds with our modified CEA and/or KEA AlgId
   SME(bRet = CSM_BaseTokenInterface::BTIFindAlgIds(pdigestAlgID, 
         pdigestEncryptionAlgID, ptmpKEAlgID, ptmpCEAlgID));

   // delete the modified CE and/or KE Alg ID if it exists (cleanup)
   if (ptmpCEAlgID)
      delete ptmpCEAlgID;
   if (ptmpKEAlgID)
      delete ptmpKEAlgID;

   SME_FINISH_CATCH

   return bRet;
}

// EOF sm_free.cpp

