/* @(#) sm_common.h 1.10 3/27/98 18:40:57 */

// sm_common.h

// includes the Common class derived from CSM_TokenInterface.
// This crypto token library header is usually the only include that the C++
// application needs.  Directly included by the C++ app and the fortezza
// crypto token library.

#ifndef _SM_COMMON_H_
#define _SM_COMMON_H_

// SHA1 constants

#define SHA1_SIZE 20
#define MASK32    0xffffffff
#define BLOCK     64
#define K1        0x5a827999
#define K2        0x6ed9eba1
#define K3        0x8f1bbcdc
#define K4        0xca62c1d6

// End of SHA1 constants

// SHA1 macros

extern void SM_DoEndian(unsigned long *); /* byte swapper for LITTLE ENDIAN */

#define Ls5(num)  (((num)<<5)|((num)>>27))
#define Ls30(num) (((num)<<30)|((num)>>2))

#ifdef NO_ESHA
#define Ls1(num)  (num)
#else
#define Ls1(num)  (((num)<<1) | ((num)>>31)&1)
#endif

#define F0(x,y,z) ((z)^(x)&((y)^(z)))
#define F1(x,y,z) ((x)&((y)^(z))^((z)&(y)))
#define F2(x,y,z) ((x)^(y)^(z))

// End of SHA1 macros

#ifndef _SM_API_H_
#include "sm_api.h"
#endif

class CSM_Common : public CSM_BaseTokenInterface
{
public:
   CSM_Common(); // default constructor

   // member access functions

   // should add a oid parameter to select between mutiple hash
   // algorithms
   //
   SM_RET_VAL SMTI_DigestData(CSM_Buffer *pData, CSM_Buffer *pHashValue);
   SM_RET_VAL SMTI_Login(void);
   SM_RET_VAL SMTI_Sign(
            CSM_Buffer *pData, // input, data to be signed
            CSM_Buffer *pEncryptedDigest, // signature
            CSM_Buffer *pDigest); // digest
   SM_RET_VAL SMTI_Verify(
            CSM_Buffer *pSignerPublicKey, // input
            CSM_Alg    *pDigestAlg, // input
            CSM_Alg    *pSignatureAlg, // input
            CSM_Buffer *pData, // input
            CSM_Buffer *pSignature);
   SM_RET_VAL SMTI_Encrypt(
            CSM_Buffer *pData, // input (data to be encrypted)
            CSM_Buffer *pParameters, // input, parameters for alg.
            CSM_Buffer *pEncryptedData, // output
            CSM_Buffer *pMEK);
   SM_RET_VAL SMTI_GenerateEMEK(
            CSM_Buffer *pRecipient, // input, Y of recip
            CSM_Buffer *pParameters, // input, parameters for alg.
            CSM_Buffer *pMEK, // input, MEK or special phrase
            CSM_Buffer *pEMEK, // output, encrypted MEK
            CSM_Buffer *pUKM); // output, ukm, if applicable
   SM_RET_VAL SMTI_ExtractMEK(
            CSM_Buffer *pOriginator, // input, Y of originator
            CSM_Buffer *pParameters, // input, parameters for alg.
            CSM_Buffer *pEMEK, // input, encrypted MEK
            CSM_Buffer *pUKM, // input
            CSM_Buffer *pMEK); // output
   SM_RET_VAL SMTI_GenerateEMEK(
            CSM_Buffer *pRecipient, // input, Y of recip
            CSM_Buffer *pParameters, // input, parameters for alg.
            CSM_Buffer *pEMEK, // output, encrypted MEK
            CSM_Buffer *pUKM); // output, ukm, if applicable
   SM_RET_VAL SMTI_Decrypt(
            CSM_Buffer *pParameters, // input, parameters for alg.
            CSM_Buffer *pEncryptedData, // input (data to be decrypted)
            CSM_Buffer *pMEK, // input (MEK or special phrase)
            CSM_Buffer *pData);
    SM_RET_VAL SMTI_Random(
            CSM_Buffer *pSeed, // input
            CSM_Buffer *pRandom, // input/output
            SM_SIZE_T lLength); // input

   SM_RET_VAL SMTI_Lock(){ return 0; }
   SM_RET_VAL SMTI_Unlock(){ return 0; }

   virtual bool SMTI_MoreCSInstances() { return 1; };

private:

    typedef struct {
       unsigned long hashval[5];
       int           initflag;
       unsigned long total;
    } Hash_struct;

    Hash_struct hash;

        // private member functions
    SM_RET_VAL SHA1_GetHash(unsigned long  ,
                            unsigned char *,
                            unsigned char *);

    void SHA1_InitializeHash(void);
    void SHA1_SohHash(unsigned long *, unsigned long *);
};

SM_RET_VAL SMCommonInit(
    CSMIME *pCsmime,        // IN, structure to be updated with CSInst.
    CSM_Buffer *pCertFileBuf, // IN, file associated with single cert.
    char *pszSessionID);    // IN, SESSION ID for COMMON SMTI tokens.


#endif // _SM_FORT_H_

// EOF sm_fort.h
