#ifndef NO_SCCS_ID
static char SccsId[ ] = "@(#) sm_Decrypt.cpp 1.7 3/27/98 18:39:53"; 
#endif

//////////////////////////////////////////////////////////////////////////
// sm_Decrypt.cpp
// Implementation of the CSM_DecryptMsg and CSM_DecryptData classes.
// CSM_DecryptMsg is for high level use.  The app developer should
// not have to directly access the snacc generated classes.
// CSM_DecryptData is for low level use.  The app may have to
// directly access the exposed snacc generated class.  Both
// classes have the purpose of decrypting a CMS EnvelopedData
// based on the provided input (primarily the marked sessions
// in the CSMIME).
//////////////////////////////////////////////////////////////////////////

#include "sm_api.h"

//////////////////////////////////////////////////////////////////////////
// CSM_DecryptMsg Constructor
//////////////////////////////////////////////////////////////////////////
CSM_DecryptMsg::CSM_DecryptMsg(CSM_Buffer *pBlob, CSMIME *pCSMIME)
{
   SME_SETUP("CSM_DecryptMsg::CSM_DecryptMsg(csmime)");

   Clear();
   if (pBlob)
   {
      SetEncodedBlob(pBlob);
      if (pCSMIME)
         SME(PreProc(pBlob, pCSMIME));
   }
   SME_FINISH_CATCH;
}

//////////////////////////////////////////////////////////////////////////
// CSM_DecryptMsg::Decrypt
//////////////////////////////////////////////////////////////////////////
void CSM_DecryptMsg::Decrypt(CSMIME *pCSMIME)
{
   CSM_CertificateChoiceLst *pOrigCerts = NULL;
   CSM_Buffer bufPlainText;

   SME_SETUP("CSM_DecryptMsg::Decrypt");

   // load m_pOriginatorInfo into pOrigCerts
   if ((m_pOriginatorInfo) && (m_pOriginatorInfo->m_pMsgCertCrls))
      pOrigCerts = m_pOriginatorInfo->m_pMsgCertCrls->GetCertificates();

   // call low level decrypt
   SME(CSM_DecryptData::Decrypt(pCSMIME, pOrigCerts, &bufPlainText));

   // create content from the decrypted plain text
   CSM_Content content(&bufPlainText);
   // store the content here
   SetEncapContent(content);

   SME_FINISH
   SME_CATCH_SETUP
      // catch/cleanup logic as necessary
   SME_CATCH_FINISH
}

//////////////////////////////////////////////////////////////////////////
// CSM_DecryptMsg::PreProc-s
//////////////////////////////////////////////////////////////////////////
void CSM_DecryptMsg::PreProc(CSMIME *pCSMIME)
{
   SME_SETUP("CSM_DecryptMsg::PreProc csmime");
   // call lower level PreProc that will process with pCSMIME
   SME(CSM_DecryptData::PreProc(pCSMIME));
   // generate the m_pOriginatorInfo from the SNACC originatorInfo
   // for the application's convenience
   SME(m_pOriginatorInfo = new CSM_OriginatorInfo(m_SnaccEnvelopedData.
         originatorInfo));
   // generate the m_pRecipients from the SNACC recipientInfos
   // for the application's convenience
   if ((m_pRecipients = new CSM_RecipientLst) == NULL)
      SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
   SME(AddSNACCRecipients(&(m_SnaccEnvelopedData.recipientInfos)));
   SME_FINISH_CATCH
}

void CSM_DecryptMsg::PreProc(CSM_Buffer *pBlob, CSMIME *pCSMIME)
{
   SME_SETUP("CSM_DecryptMsg::PreProc buf csmime");
   // Decode the provided blob
   CSM_DecryptData::Decode(pBlob);
   // call other high level PreProc that will generate m_pRecipients
   // and m_pOriginatorInfo for application's use
   SME(PreProc(pCSMIME));
   SME_FINISH_CATCH
}

//////////////////////////////////////////////////////////////////////////
void CSM_DecryptMsg::AddSNACCRecipients(RecipientInfos *pRILst)
{
   RecipientInfo *pRI;
   CSM_Recipient *pNewRecipient;

   SME_SETUP("CSM_DecryptMsg::AddSNACCRecipients");

   if (pRILst == NULL)
      SME_THROW(SM_MISSING_PARAM, NULL, NULL);

   pRILst->SetCurrToFirst();
   pRI = pRILst->First();
   while (pRI != NULL)
   {
      if ((pNewRecipient = new CSM_Recipient) == NULL)
         SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
      SME(pNewRecipient->SetFromSNACCRecipientInfo(pRI));
      if (m_pRecipients == NULL)
      {
         if ((m_pRecipients = new CSM_RecipientLst(pNewRecipient)) == NULL)
            SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
      }
      else
         SME(m_pRecipients->Append(pNewRecipient));
      pRI = pRILst->GoNext();
   }

   SME_FINISH_CATCH
}

//////////////////////////////////////////////////////////////////////////
// CSM_DecryptData:: Constructors
//////////////////////////////////////////////////////////////////////////
CSM_DecryptData::CSM_DecryptData(CSM_Buffer *pbufEnvelopedData)
{
   SME_SETUP("CSM_DecryptData::CSM_DecryptData buf");

   if (pbufEnvelopedData == NULL)
      SME_THROW(SM_MISSING_PARAM, NULL, NULL);

   // ASN.1 decode the provided pbufEnvelopedData
   DECODE_BUF((&m_SnaccEnvelopedData), pbufEnvelopedData);

   SME_FINISH_CATCH
}

CSM_DecryptData::CSM_DecryptData(CSM_Buffer *pbufEnvelopedData,
                                 CSMIME *pCSMIME)
{
   SME_SETUP("CSM_DecryptData::CSM_DecryptData buf csmime");

   if ((pbufEnvelopedData == NULL) || (pCSMIME == NULL))
      SME_THROW(SM_MISSING_PARAM, NULL, NULL);

   // ASN.1 decode the provided pbufEnvelopedData
   DECODE_BUF((&m_SnaccEnvelopedData), pbufEnvelopedData);

   SME(PreProc(pCSMIME));

   SME_FINISH_CATCH
}

//////////////////////////////////////////////////////////////////////////
// CSM_DecryptData::PreProc
//////////////////////////////////////////////////////////////////////////
void CSM_DecryptData::PreProc(CSMIME *pCSMIME)
{
   CSM_CSInstLst *pTmpInstLst;

   SME_SETUP("CSM_DecryptData::PreProc csmime");

   if (pCSMIME == NULL)
      SME_THROW(SM_MISSING_PARAM, NULL, NULL);

   // using the provided instances, try to find applicable ones...
   if (pCSMIME->m_pCSInsts != NULL)
   {
      // what was used to encrypt the content?
      if ((m_SnaccEnvelopedData.encryptedContentInfo == NULL) ||
          (m_SnaccEnvelopedData.encryptedContentInfo->
            contentEncryptionAlgorithm == NULL))
         SME_THROW(SM_ENV_DATA_DEC_ERROR, 
               "contentEncryptionAlgorithm missing", NULL);
      // construct a temporary alg to hold the content encryption alg
      CSM_Alg tmpCEAlg(*(m_SnaccEnvelopedData.encryptedContentInfo->
            contentEncryptionAlgorithm));
      pTmpInstLst = pCSMIME->m_pCSInsts->First();
      do
      {
         // this instance will only be applicable if it supports the
         // content encryption algorithm used in this enveloped data
         if (pTmpInstLst->Get()->FindAlgIds(NULL, NULL, NULL, &tmpCEAlg))
         {
            // look through the recipientInfos while there are more to look
            // at AND while this instance is not marked applicable
            m_SnaccEnvelopedData.recipientInfos.SetCurrToFirst();
            RecipientInfo *pRI = m_SnaccEnvelopedData.recipientInfos.First();
            while ((pRI != NULL) && (!(pTmpInstLst->Get()->IsApplicable())))
            {
               if (pRI->keyEncryptionAlgorithm == NULL)
                  SME_THROW(SM_ENV_DATA_DEC_ERROR,
                        "recipientInfo->keyEncryptionAlgorithm missing",
                        NULL);
               // construct a temp alg to hold this recipient's key enc alg
               CSM_Alg tmpKEAlg(*(pRI->keyEncryptionAlgorithm));
               if (pTmpInstLst->Get()->FindAlgIds(NULL, NULL, &tmpKEAlg, 
                     NULL))
                  // this instance is applicable
                  pTmpInstLst->Get()->SetApplicable();
               // get the next recipientInfo
               pRI = m_SnaccEnvelopedData.recipientInfos.GoNext();
            }
         }
      } while ((pTmpInstLst = pTmpInstLst->Next()) != NULL);
   }

   SME_FINISH_CATCH
   // TBD, cleanup???
}

void CSM_DecryptData::PreProc(CSM_Buffer *pbufEnvelopedData,
                                    CSMIME *pCSMIME)
{
   SME_SETUP("CSM_DecryptData::PreProc buf csmime");
   if ((pbufEnvelopedData == NULL) || (pCSMIME == NULL))
      SME_THROW(SM_MISSING_PARAM, NULL, NULL);
   // ASN.1 decode the provided pbufEnvelopedData
   Decode(pbufEnvelopedData);
   // call preproc that does the real work
   SME(PreProc(pCSMIME));
   SME_FINISH_CATCH
}

//////////////////////////////////////////////////////////////////////////
// CSM_DecryptData::Decode
//////////////////////////////////////////////////////////////////////////
void CSM_DecryptData::Decode(CSM_Buffer *pbufEnvelopedData)
{
   SME_SETUP("CSM_DecryptData::Decode");

   if (pbufEnvelopedData)
      DECODE_BUF((&m_SnaccEnvelopedData), pbufEnvelopedData);

   SME_FINISH_CATCH
}

//////////////////////////////////////////////////////////////////////////
// CSM_DecryptData::TryThisInstance
//////////////////////////////////////////////////////////////////////////
// this function uses the provided instance, finds a matching recipient,
// and attempts to decrypt the EMEK resulting in the MEK...success returns
// the MEK
CSM_Buffer* CSM_DecryptData::TryThisInstance(CSM_CSInst *pInst, 
      CSM_CertificateChoiceLst *pOrigCerts)
{
   CSM_Alg *pOrigAlg;
   CSM_Buffer *pMEK = NULL;
   RecipientInfo *pSnaccRI;

   SME_SETUP("CSM_DecryptData::TryThisInstance");

   if ((pInst == NULL) || (pOrigCerts == NULL))
      SME_THROW(SM_MISSING_PARAM, NULL, NULL);

   // if we are here, pInst was marked applicable or UseThis,
   // see if there is a recipientInfo that matches what
   // this instance can support...
   m_SnaccEnvelopedData.recipientInfos.SetCurrToFirst();
   pSnaccRI = m_SnaccEnvelopedData.recipientInfos.First();
   do
   {
      CSM_Buffer *pOrigKey = NULL;

      // TBD, there are probably memory leaks in the loops below...
      // look at pOrigAlg and pOrigKey memory usage...
      if (pSnaccRI->keyEncryptionAlgorithm != NULL)
      {
         // does this instance support the current recipient info's alg?
         CSM_Alg tmpAlg(*(pSnaccRI->keyEncryptionAlgorithm));
         if (pInst->FindAlgIds(NULL, NULL, &tmpAlg, NULL))
         {
            // alg match, try to match issuer/serial number in recipient
            // with issuer/serial number in this instance...
            CSM_Recipient tmpRecip;
            tmpRecip.SetFromSNACCRecipientInfo(pSnaccRI);
            if (*(tmpRecip.GetIssuerAndSerial()) == 
                  *(pInst->GetIssuerAndSerialNumber()))
            {
               // yes, attempt to use this instance to extract the MEK
               // in order to call SMTI_ExtractMEK we need the
               // originator's public value, look for it in pOrigCerts
               // TBD, we will want to use pSnaccRI->originatorCert to
               // help identify what we need from pOrigCerts, but for
               // now, we'll match Algs
               // search through pOrigCerts...
               if (pOrigCerts)
               {
                  CSM_CertificateChoiceLst *pCertsTmp = pOrigCerts->First();
                  do
                  {
                     SME(pOrigAlg = pCertsTmp->Get()->GetPublicKeyAlg());
                     if (*(pOrigAlg->GetId()) == *(tmpAlg.GetId()))
                        // this originator cert's subject public key OID
                        // match this recipient's keyEncryptionAlgorithm
                        // OID, get the public key out
                        SME(pOrigKey = pCertsTmp->Get()->GetPublicKey());
                  } while (((pCertsTmp = pCertsTmp->Next()) != NULL) &&
                           (pOrigKey == NULL));
               }
               if ((pOrigKey == NULL) && (m_SnaccEnvelopedData.originatorInfo)
                     && (m_SnaccEnvelopedData.originatorInfo->certs))
               {
                  // look for a public key in m_SnaccEnvelopedData.
                  // originatorInfo->certs
                  m_SnaccEnvelopedData.originatorInfo->certs->
                           SetCurrToFirst();
                  CertificateChoices *pSnaccCert = m_SnaccEnvelopedData.
                        originatorInfo->certs->First();
                  do
                  {
                     if (pSnaccCert->choiceId == 
                           CertificateChoices::certificateCid)
                     {
                        CSM_CertificateChoice tmpCert(*pSnaccCert);
                        SME(pOrigAlg = tmpCert.GetPublicKeyAlg());
                        if (*(pOrigAlg->GetId()) == *(tmpAlg.GetId()))
                           SME(pOrigKey = tmpCert.GetPublicKey());
                     }
                  } while (((pSnaccCert = m_SnaccEnvelopedData.originatorInfo
                        ->certs->GoNext()) != NULL) && (pOrigKey == NULL));
               }

               // now we either have something in pOrigKey or we don't,
               // if we don't, then the algorithm being used may not
               // require originator info (e.g. RSA), so pass NULL,
               // if we do, then we are assuming (TBD?) that the match
               // of algorithm OIDs from either the provided pOrigCerts
               // or the originatorInfo->certs will be sufficient for
               // identifying the originator's public key...we will want
               // to look at the recipient info originatorCert field
               // in the future...also, there may be another, more
               // accurate way to find the correct originator info cert

               // get parameters
               CSM_Buffer *pParameters=NULL;
               if (pSnaccRI->keyEncryptionAlgorithm->parameters)
                 SM_EXTRACT_ANYBUF(pParameters, (pSnaccRI->
                     keyEncryptionAlgorithm->parameters));

               // get the EMEK
               CSM_Buffer bufEMEK((char*)pSnaccRI->encryptedKey, 
                     pSnaccRI->encryptedKey.Len());

               // get the UKM by searching for matching alg OID
               CSM_Buffer *pUKM = NULL;
               if ((m_SnaccEnvelopedData.originatorInfo) &&
                     (m_SnaccEnvelopedData.originatorInfo->ukms))
               {
                  m_SnaccEnvelopedData.originatorInfo->ukms->SetCurrToFirst();
                  UserKeyingMaterial *pTmpUkm = m_SnaccEnvelopedData.
                        originatorInfo->ukms->First();
                  do
                  {
                     CSM_Alg tmpUkmAlg(*(pTmpUkm->algorithm));
                     if (*(tmpUkmAlg.GetId()) == *(tmpAlg.GetId()))
                        if ((pUKM = new CSM_Buffer((char*)pTmpUkm->ukm,
                              pTmpUkm->ukm.Len())) == NULL)
                           SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
                  } while (((pTmpUkm = m_SnaccEnvelopedData.originatorInfo
                        ->ukms->GoNext()) != NULL) && (pUKM == NULL));
               }

               // create a buffer to receive the MEK
               if ((pMEK = new CSM_Buffer) == NULL)
                  SME_THROW(SM_MEMORY_ERROR, NULL, NULL);

               // decrypt the EMEK to get the MEK
               SME(pInst->GetTokenInterface()->SMTI_ExtractMEK(pOrigKey, 
                     pParameters, &bufEMEK, pUKM, pMEK));
               // TBD, need code in the higher level Decrypt that marks
               // the decrypted recipient with WasDecrypted
               // TBD, clean up parameters, pOrigKey, etc...
            }
         }
      }
      // continue looking through the recipient's until we have data
      // in the MEK buffer or until we run out of recipients
   } while (((pSnaccRI = m_SnaccEnvelopedData.recipientInfos.GoNext())
         != NULL) && (pMEK == NULL));

   SME_FINISH_CATCH

   return pMEK;
}

//////////////////////////////////////////////////////////////////////////
// CSM_DecryptData::Decrypt
//////////////////////////////////////////////////////////////////////////
void CSM_DecryptData::Decrypt(CSMIME *pCSMIME,
                                    CSM_CertificateChoiceLst *pOrigCerts,
                                    CSM_Buffer *pbufDecryptedContent)
{
   CSM_CSInstLst *pInstLstTmp;
   CSM_Buffer *pbufMEK = NULL;
   CSM_Buffer *pbufParameters=NULL;

   SME_SETUP("CSM_DecryptData::Decrypt");

   // check the incoming parameters
   if ((pCSMIME == NULL) || (pCSMIME->m_pCSInsts == NULL) || 
         (pbufDecryptedContent == NULL))
      SME_THROW(SM_MISSING_PARAM, NULL, NULL);
   
   // lock the CTIs
   SME(pCSMIME->InstanceLock(SM_INST_APPLICABLE | SM_INST_USE_THIS));

   // give priority to instances marked UseThis...search for an instance
   // marked UseThis
   for (pInstLstTmp = pCSMIME->m_pCSInsts->First(); pInstLstTmp && 
         (pbufMEK == NULL); pInstLstTmp = pInstLstTmp->Next())
      if (pInstLstTmp->Get()->IsThisUsed())
         SME(pbufMEK = TryThisInstance(pInstLstTmp->Get(), pOrigCerts));
   // search for an instance marked applicable
   for (pInstLstTmp = pCSMIME->m_pCSInsts->First(); pInstLstTmp &&
         (pbufMEK == NULL); pInstLstTmp = pInstLstTmp->Next())
      if (pInstLstTmp->Get()->IsApplicable())
         SME(pbufMEK = TryThisInstance(pInstLstTmp->Get(), pOrigCerts));

   // now, we either have an MEK or we don't.  If we don't then there was
   // no instance marked UseThis or Applicable that had everything
   // necessary to decrypt the EMEK
   if (pbufMEK == NULL)
      SME_THROW(SM_NO_SUPPORTING_INSTANCE, "couldn't decrypt EMEK", NULL);

   // Extract the content encryption parameters
   if (m_SnaccEnvelopedData.encryptedContentInfo->contentEncryptionAlgorithm->
       parameters)
       SM_EXTRACT_ANYBUF(pbufParameters, m_SnaccEnvelopedData.
         encryptedContentInfo->contentEncryptionAlgorithm->parameters);
   // create a buffer holding the encrypted content
   CSM_Buffer bufContent((char*)(*(m_SnaccEnvelopedData.
         encryptedContentInfo->encryptedContent)), m_SnaccEnvelopedData.
         encryptedContentInfo->encryptedContent->Len());
   // decrypt the content
   SME(pInstLstTmp->Get()->GetTokenInterface()->SMTI_Decrypt(pbufParameters,
         &bufContent, pbufMEK, pbufDecryptedContent));

   // unlock the CTI
   SME(pCSMIME->InstanceUnlock(SM_INST_APPLICABLE | SM_INST_USE_THIS));

   SME_FINISH
   SME_CATCH_SETUP
      // catch/cleanup logic as necessary
      // unlock the CTI if necessary
   SME_CATCH_FINISH
}

// EOF sm_Decrypt.cpp
