#ifndef NO_SCCS_ID
static char SccsId[ ] = "@(#) sm_Sign.cpp 1.20 3/25/98 09:56:08"; 
#endif

//
//  sm_Sign.cpp
//  These routines support the CSM_Sign class.

#include <stdlib.h>
#include <malloc.h>
#include <string.h>
#include "sm_api.h"

SM_RET_VAL sm_PutSignerInfo(CSM_CSInst *pCSInst, SignedData *lpSignData,
                                         SignerInfo *lpSNACCSignerInfo);


//
// RWC; TBD; DAVE???? 12/31/97???   attributes in Sign() operation? common to 
//   all CSInstances??? called CSM_MsgAttributes!  ADDED to CSM_SignMsg class.

//
// CSM_SignMsg::Sign
//  This function expects the following member variables to be loaded before 
//  execution:  
//      pCSM_MsgCertCrls (OPTIONAL), mp_UnauthAttrs (OPTIONAL), 
//      mp_AuthAttrs (OPTIONAL), CSM_CommonData::m_contentValue 
//      (for content to sign)
//  The result is stored in the member variable "CSM_CommonData::m_EncodedBlob".
//  It is expected that at least one session is valid for processing (as many
//  SignerInfos will be added to the resulting SignedData ASN.1 structure as
//  there are flagged sessions in the pCsmime structure).
SM_RET_VAL CSM_SignMsg::Sign(CSMIME *pCsmime)
{
    SignedData *lpSignedData=&m_SnaccSignedData;
    SM_RET_VAL status=0;
    CSM_CSInst *tmpCSInst;
    CSM_CSInstLst *ttt;
    AsnOcts      *tmpOcts = new AsnOcts;
    CSM_Buffer *pTmpStr;
    SignerInfo *tmpSNACCSignerInfo=NULL;
    CSM_Buffer tmpCStr;
    CSM_CertificateChoiceLst *tmpCertList=NULL;
    CertificateChoices *tmnSNACCCertChoices=NULL;
    CSM_Buffer *pDigest=NULL;
    CSM_HashDefLst *pHash=NULL;
    bool bAtLeastOneSigner;

    SME_SETUP("Sign");
    
    if (m_pMsgCertCrls && (tmpCertList=m_pMsgCertCrls->GetACs()) != NULL)
        lpSignedData->version.Set(3);
    else
        lpSignedData->version.Set(1);

    if (m_pContentValue && 
       (m_pContentValue->m_lType == CSM_Content::CONTENT_DATA_DATA || 
        m_pContentValue->m_lType == CSM_Content::CONTENT_DATA_ANY))
    {
        //signedData.content_info = *mCSM_contentValue.GetContent();  
        lpSignedData->encapContentInfo = new EncapsulatedContentInfo;
        lpSignedData->encapContentInfo->eContentType = 
            m_pContentValue->m_contentType;
                        // change SM_Content to class with SNACC OID

        SME(lpSignedData->encapContentInfo->eContent = 
            new AsnOcts((char *)m_pContentValue->m_content, 
            m_pContentValue->m_content.Length()));

        // Load Attribute certificates into SNACC structure.
        if (m_pMsgCertCrls && (tmpCertList=m_pMsgCertCrls->GetACs()) != NULL)
        {
              if (lpSignedData->certificates == NULL)
              {             // May be partially filled by Certificates above.
                if ((lpSignedData->certificates = new CertificateSet) == NULL)
                    SME_THROW(22, "SignedData->certificates (ACs) bad memory alloc", NULL);
              }
              for ( ; tmpCertList; tmpCertList=tmpCertList->Next())
              {               // For each cert buffer, load into SNACC cert list.
                  tmnSNACCCertChoices = lpSignedData->certificates->Append();
                  tmnSNACCCertChoices->choiceId = CertificateChoices::attrCertCid;
                  tmnSNACCCertChoices->certificate = new Certificate;
                  DECODE_BUF(tmnSNACCCertChoices->certificate, 
                        (*(tmpCertList->Get()->GetEncodedCert())));
              }
        }

        // for each session marked to sign this message, load a SignerInfo
        //  (BE SURE TO ONLY COMPUTE THE COMMON HASH code once for any Signers
        //   that happen to use the same hash computation.)
        bAtLeastOneSigner = false;
        for (ttt=pCsmime->m_pCSInsts->First(); ttt; ttt=ttt->Next())
        {
            tmpCSInst = ttt->Get();
            if (tmpCSInst->IsThisUsed())
            {
              // Process the content and produce a Hash value.
              pTmpStr = NULL;
              SignCalculateHash(tmpCSInst, lpSignedData, pHash, pTmpStr, pDigest);
              // RWC; TBD; do same for Unauthenticated Attributes.
              SME(sm_PutSignerInfo(tmpCSInst, lpSignedData, tmpSNACCSignerInfo));
              
              // NOW, process the content and produce a signature.
              if (pTmpStr != NULL)
              {
                  SME(tmpCSInst->GetTokenInterface()->SMTI_Sign(pTmpStr, 
                      &tmpCStr,                // returned result
                      pDigest));               // data digest (Hash of Content)
                  tmpSNACCSignerInfo->signature.Set((char *)tmpCStr, 
                      tmpCStr.Length());
                  bAtLeastOneSigner = true;
              }

              // RWC; TBD; hash and sign authenticated attribute signature.
              if (pTmpStr != NULL)
                  delete pTmpStr;
              if (pDigest != NULL)
                  delete pDigest;

            }       // if session requested to sign this message

        }           // for all sessions
        if (!bAtLeastOneSigner)
          SME_THROW(23, "At least one signer required.", NULL);

        if (m_pEncodedBlob)             // delete old definition.
            delete m_pEncodedBlob;
        m_pEncodedBlob = new CSM_Buffer;
        status = CSM_SignData::Sign(pCsmime, m_pMsgCertCrls, m_pEncodedBlob);
                    // The "m_pMsgCertCrls" variable contains optional certs 
                    //  and CRLs specified by the application, not those 
                    //  available from the logon instances.
        if (pHash)
            delete pHash;
    
    }
    else
    {
        SME_THROW(22, "SignedData->contentInfo->content bad content type", NULL);
        // report_error, bad content type; OK to have SignedData OID and ASN.1
        //  encoded content, but "type" must be OCTET or ANY.

    }

    SME_FINISH
    SME_CATCH_SETUP
        // local cleanup logic
        delete tmpOcts;
        if (m_pEncodedBlob)
            delete m_pEncodedBlob;
    SME_CATCH_FINISH

    delete tmpOcts;

    return(status);

}

SM_RET_VAL CSM_SignMsg::SignCalculateHash(CSM_CSInst *tmpCSInst, 
    SignedData *lpSignedData, CSM_HashDefLst *&pHash, 
    CSM_Buffer *&pContent, CSM_Buffer *&pHashResult)
{
    long status=0;
    CSM_TokenInterface *tmpTokenIF;
    CSM_HashDefLst *pTmpHash;
    CSM_HashDef *pHashItem;
    CSM_OID *pTmpOID;
    CSM_Buffer *pTmpBuf=NULL;

    SME_SETUP("CSM_SignMsg::SignCalculateHash");
              pHashResult = new CSM_Buffer;
              tmpTokenIF = tmpCSInst->GetTokenInterface();
              pTmpOID=tmpTokenIF->GetPrefDigest();
              if (pContent == NULL)
                 pContent = new CSM_Buffer;
              SME(ENCODE_BUF(lpSignedData->encapContentInfo, pContent));
              for (pTmpHash=pHash; pTmpHash && 
                   *pTmpHash->Get()->m_pOID != *tmpTokenIF->GetPrefDigest();
                   pTmpHash = pTmpHash->Next()); // Search for our Hash Algorithm.
                                                 //  Avoids recomputing!!!
              if (pTmpHash == NULL)     // NONE found, we need to compute Hash 
              {
                  SME(tmpTokenIF->SMTI_DigestData(pContent, pHashResult));
                  pHashItem = new CSM_HashDef(
                        tmpTokenIF->GetPrefDigest(), pHashResult);
                  if (pHash == NULL)    // First one in list.
                      pHash = new CSM_HashDefLst(pHashItem);
                  else
                      pHash->Append(pHashItem);
                  //delete pHashItem;
              }
              else
              {                     // Already Hashed.
                 pHashResult = new CSM_Buffer(*pTmpHash->Get()->m_pHash);
              }
              // If there are AuthAttributes AND not a "messageDigest" attribute
              //    add one after computing the hash of the content before the call to
              //    "sm_PutSignerInfo()".
              if (m_pAuthAttrs)
              {
                  pTmpBuf = NULL;
                  m_pAuthAttrs->GetMessageDigest(pTmpBuf);
                  if (pTmpBuf == NULL)        // Then load our newly generated Hash.
                  {
                     CSM_Attrib *pTmpAttrib=new CSM_Attrib(*pHashResult);
                     if (m_pAuthAttrs->m_pAttrs == NULL)
                         m_pAuthAttrs->m_pAttrs = new CSM_AttribLst(pTmpAttrib);
                     else
                         m_pAuthAttrs->m_pAttrs->Append(pTmpAttrib);
                  }
                  // ENCODE the authenticated attributes
              }

    SME_FINISH
    SME_CATCH_SETUP
        // local cleanup logic
        if (pContent)
        {
            delete pContent;
            pContent = NULL;
        }
    SME_CATCH_FINISH
    return status;
}

//
//
SM_RET_VAL CSM_SignMsg::sm_PutSignerInfo(CSM_CSInst *pCSInst, 
     SignedData *lpSignedData, SignerInfo *&lpSNACCSignerInfo)
{
    long status=0;
    CSM_OID oidDigest;      // temporary OID for each pCSInst signing alg.
    CSM_OID oidDigestEncryption; // tmp OID for each pCSInst signing alg
    //CertificateChoices *tmpSNACCCert;
    CSM_AlgLst *pdigestAlgID=NULL;
    CSM_AlgLst *pdigestEncryptionAlgID=NULL;
    //CSM_AlgLst *pkeyEncryptionAlgID=NULL;
    //CSM_AlgLst *pcontentEncryptionAlgID=NULL;
    CSM_AlgLst *tmpAlgLst;
    CSM_Alg *tmpAlg;
    CSM_IssuerAndSerialNumber *tmpIssSN=NULL;
    CertificateList *tmpSNACCCertList=NULL;
    CSM_CertificateChoiceLst *tmpCertList=NULL;
    CertificateChoices *tmnSNACCCertChoices=NULL;
    CSM_BufferLst *tmpCRLs=NULL;


  SME_SETUP("sm_PutSignerInfo");
  if ((tmpIssSN = pCSInst->GetIssuerAndSerialNumber()) != NULL)
                    // MUST HAVE valid Issuer and SN from cert.
  {
      lpSNACCSignerInfo = lpSignedData->signerInfos.Append();

      lpSNACCSignerInfo->version.Set(2);
      // process originator Issuer and SerialNumber from user cert (1st in list).
      lpSNACCSignerInfo->issuerAndSerialNumber = new IssuerAndSerialNumber;
      *lpSNACCSignerInfo->issuerAndSerialNumber =  
          *(IssuerAndSerialNumber *)*tmpIssSN;

      pCSInst->GetPreferredCSInstAlgs(&oidDigest, &oidDigestEncryption, 
      NULL, NULL);
      pCSInst->GetTokenInterface()->BTIGetAlgIDs(&pdigestAlgID, 
            &pdigestEncryptionAlgID, NULL,NULL);
          //&pkeyEncryptionAlgID, &pcontentEncryptionAlgID);
      // Locate our AlgIds for preferred algorithms
      for (tmpAlgLst=pdigestAlgID->First(); tmpAlgLst && 
         *tmpAlgLst->Get()->GetId() != oidDigest; tmpAlgLst=tmpAlgLst->Next());
      if (tmpAlgLst)        // if the preferred OID is found.
      {
        tmpAlg = tmpAlgLst->Get();
        lpSNACCSignerInfo->digestAlgorithm = new DigestAlgorithmIdentifier();
        *lpSNACCSignerInfo->digestAlgorithm = *tmpAlg->GetSNACCAlgId();
        // ALSO, load into the SignedData->digestAlgorithms
        lpSignedData->digestAlgorithms.AppendCopy(
          *lpSNACCSignerInfo->digestAlgorithm);
      }

      for (tmpAlgLst=pdigestEncryptionAlgID->First(); tmpAlgLst && 
          *tmpAlgLst->Get()->GetId() != oidDigestEncryption; 
          tmpAlgLst=tmpAlgLst->Next());
      if (tmpAlgLst)        // if the preferred OID is found.
      {
        tmpAlg = tmpAlgLst->Get();
        lpSNACCSignerInfo->signatureAlgorithm = new SignatureAlgorithmIdentifier();
        *lpSNACCSignerInfo->signatureAlgorithm = *tmpAlg->GetSNACCAlgId();
      }

         // RWC; TBD; Hash and sign these results, save appropriately.
         // RWC; TBD;m_pAuthAttrs->encodeSNACC();
         //lpSNACCSignerInfo->authenticatedAttributes = 
         //                 m_pAuthAttrs->GetSNACCAuthAttributes();
      // RWC; TBD; PERFORM SIGNING operation on SignerInfo component, when finished.
      //lpSNACCSignerInfo->signature.Set((const char *)lpsm_SignerInfo->signature.str, 
      //                                 (size_t)lpsm_SignerInfo->signature.lgth);

        if (m_bIncludeOrigCertsFlag)
        {
           //  Then all Buf_fileLst items can be loaded as files or bufs.
            //  ALSO, handle for decodes where temporary file created if large.
          if ((tmpCRLs=pCSInst->GetCRLs()) != NULL)
          {
            lpSignedData->crls = new CertificateRevocationLists;
            if (lpSignedData->crls)
            {
              for (tmpCRLs=tmpCRLs->First(); tmpCRLs; tmpCRLs=tmpCRLs->Next())
              {                   // For each CRL in list, add to SNACC structure
                  SME(tmpSNACCCertList = lpSignedData->crls->Append());
                  SME(DECODE_BUF(tmpSNACCCertList, *tmpCRLs->Get()));
                              // Assign each CRL buffer to the SNACC structure.
              }
            }
            else
                SME_THROW(22, "SignedData->crls bad memory alloc", NULL);
          }
          // Load certificates into SNACC structure.
          if ((tmpCertList=pCSInst->GetCertificates()) != NULL)
          {
              if (lpSignedData->certificates == NULL)
              {             // May be partially filled by Certificates above.
                if ((lpSignedData->certificates = new CertificateSet) == NULL)
                    SME_THROW(22, "SignedData->certificates (Certs) bad memory alloc", NULL);
              }
              for ( ; tmpCertList; tmpCertList=tmpCertList->Next())
              {               // For each cert buffer, load into SNACC cert list.
                  tmnSNACCCertChoices = lpSignedData->certificates->Append();
                  tmnSNACCCertChoices->choiceId = CertificateChoices::certificateCid;
                  tmnSNACCCertChoices->certificate = new Certificate;
                  DECODE_BUF(tmnSNACCCertChoices->certificate, 
                        (*(tmpCertList->Get()->GetEncodedCert())));
              }
          }
       }
  }

    SME_FINISH
    SME_CATCH_SETUP
        // local cleanup logic
    SME_CATCH_FINISH

  return(status);
}


// Sign
// This method assumes that the calling application has loaded all of the 
//  appropriate SNACC information, including SignerInfo and signatures, into
//  the SNACC "SignedData m_SnaccSignedData" member variable for encoding.
SM_RET_VAL CSM_SignData::Sign(
     CSMIME *pCSMIME,               // IN,logged-on Instance list
     CSM_MsgCertCrls *pMsgCertCrls, // IN,Originator(s) certs (May be NULL).
     CSM_Buffer *&pEncodedBlob)      // OUT, Resulting SignedData/ContentInfo.
{
    SM_RET_VAL status=0;

    SME_SETUP("Sign");
 
    if (pMsgCertCrls)       // Then fill in the appropriate SNACC element.
    {
        pMsgCertCrls->PutSNACCCerts(m_SnaccSignedData.certificates);
        //pMsgCertCrls->PutSNACCACs(m_SnaccSignedData.certificates);
    }
    SME(ENCODE_BUF(&m_SnaccSignedData, pEncodedBlob));

    SME_FINISH
    SME_CATCH_SETUP
        // local cleanup logic
    SME_CATCH_FINISH

    return(status);
}

// EOF sm_Sign.cpp
