#ifndef NO_SCCS_ID
static char SccsId[ ] = "@(#) sm_Attr.cpp 1.4 3/26/98 12:28:55"; 
#endif


// sm_Attr.cpp
// This support file handles the Attribute class functionality for the SMIME 
//  library.

#include "sm_api.h"


CSM_Attrib::CSM_Attrib(CSM_Buffer *pMessageDigest)
{   
   MessageDigest   snaccMD;

   m_lContentType = ATTR_Message_Digest;

    m_pMessageDigest = new CSM_Buffer(*pMessageDigest);
   
   m_poid = new CSM_OID(id_messageDigest);

    snaccMD.Set((const char *)*m_pMessageDigest,(size_t)m_pMessageDigest->Length());

    ENCODE_BUF((&snaccMD), m_pEncodedAttrib);
}

CSM_Attrib::CSM_Attrib(char *pSigningTime)
{   
   SigningTime      snaccST;

   snaccST.choiceId = SigningTime::generalTimeCid;

   m_lContentType = ATTR_Signing_Time;

    m_pSigningTime = strdup(pSigningTime);
   
    m_poid = new CSM_OID(id_signingTime);

    snaccST.generalTime = new GeneralizedTime((const char *)m_pSigningTime);

    ENCODE_BUF((&snaccST), m_pEncodedAttrib);
}

CSM_Attrib::CSM_Attrib(CSM_CounterSignature *pCounterSignature)
{   
   m_lContentType = ATTR_Counter_Signature;

    m_pCounterSignature = new CSM_CounterSignature(*pCounterSignature);
   
    m_poid = new CSM_OID(id_countersignature);

   ENCODE_BUF(m_pCounterSignature->GetCounterSignature()->GetSignerInfo(), m_pEncodedAttrib);
}

CSM_Attrib::CSM_Attrib(CSM_ReceiptRequest *pReceiptRequest)
{   
   ReceiptRequest   snaccRR;
   CSM_DNLst        *tmpDNLst;
    CSM_DN           *tmpDNLstInst;
    GeneralNames     *tmpNames;
   GeneralName      *tmpName;

   m_lContentType = ATTR_Receipt_Request;

    m_pReceiptRequest = new CSM_ReceiptRequest(*pReceiptRequest);
   
    m_poid = new CSM_OID("1.2.3.4451");//RWC; TBD;id_aa_receiptRequest);

    snaccRR.signedContentIdentifier.Set((const char *)m_pReceiptRequest->m_SignedContentIdentifier,
             m_pReceiptRequest->m_SignedContentIdentifier.Length());

   //TBD: Update receiptLst processing
    snaccRR.receiptsFrom = new ReceiptsFrom();
    snaccRR.receiptsFrom->choiceId = ReceiptsFrom::receiptListCid;
    snaccRR.receiptsFrom->receiptList = new ReceiptsFromSeqOf();

   for(tmpDNLst = m_pReceiptRequest->m_ReceiptsFrom.First();tmpDNLst;tmpDNLst = tmpDNLst->Next())
   {
      tmpDNLstInst = tmpDNLst->Get();

     tmpNames = snaccRR.receiptsFrom->receiptList->Append();
     tmpName = tmpNames->Append();

     m_pReceiptRequest->Load_GeneralName(tmpDNLstInst,tmpName);

    }

   if(m_pReceiptRequest->m_ReceiptsTo != NULL)
    {
     for(tmpDNLst = m_pReceiptRequest->m_ReceiptsTo->First();tmpDNLst;
        tmpDNLst = tmpDNLst->Next())
     {
        tmpDNLstInst = tmpDNLst->Get();

       tmpNames = snaccRR.receiptsTo.Append();
       tmpName = tmpNames->Append();

       m_pReceiptRequest->Load_GeneralName(tmpDNLstInst,tmpName);

     }
    }

    ENCODE_BUF((&snaccRR), m_pEncodedAttrib);
}

CSM_Attrib::CSM_Attrib(CSM_ContentHints *pContentHints)
{   
   ContentHints   snaccCH;

   m_lContentType = ATTR_Content_Hints;

   m_pContentHints = new CSM_ContentHints(*pContentHints);
   
   m_poid = new CSM_OID("1.2.3.4451");//RWC; TBD;id_aa_contentHints);

   // contentDescription will always be a UTF8 String but encoded as an
   // implicit octet string with a context specific tag.
   //
   snaccCH.contentDescription = new AsnOcts ( 
                               (char *) m_pContentHints->m_ContentDescription,
                               m_pContentHints->m_ContentDescription.Length());

   snaccCH.contentType = m_pContentHints->m_OID; 

   ENCODE_BUF((&snaccCH), m_pEncodedAttrib);
}

CSM_Attrib::CSM_Attrib(CSM_SecLbl *pSecLbl)
{   
   ESSSecurityLabel       snaccSL;
   CSM_SecCatLst       *tmpSecCatsLst;
   CSM_SecCat          *tmpSecCatsInst;
   SecurityCategory    *tmpSecurityCat;

   m_lContentType = ATTR_Security_Label;

   m_pSecurityLabel = new CSM_SecLbl(*pSecLbl);
   
   m_poid = new CSM_OID("1.2.3.4452");//RWC;TBD;id_aa_securityLabel);

   snaccSL.security_policy_identifier = new SecurityPolicyIdentifier;
   *snaccSL.security_policy_identifier = (*m_pSecurityLabel->m_PolicyId);

   snaccSL.security_classification = 
     new SecurityClassification(m_pSecurityLabel->m_lSecClass);

   snaccSL.privacy_mark = new ESSPrivacyMark;
    snaccSL.privacy_mark->choiceId = ESSPrivacyMark::pStringCid;
    snaccSL.privacy_mark->pString = new PrintableString((char *)m_pSecurityLabel->m_pPmark,
           m_pSecurityLabel->m_pPmark->Length());

    snaccSL.security_categories = new SecurityCategories;
   for(tmpSecCatsLst = m_pSecurityLabel->m_pSecCats->First();tmpSecCatsLst;
       tmpSecCatsLst = tmpSecCatsLst->Next())
   {
      tmpSecCatsInst = tmpSecCatsLst->Get();

     tmpSecurityCat = snaccSL.security_categories->Append();

     tmpSecurityCat->type = tmpSecCatsInst->m_Type;
      SM_ASSIGN_ANYBUF(&tmpSecCatsInst->m_Value, &tmpSecurityCat->value);
 
   }
}  

void CSM_MsgAttributes::GetMessageDigest(CSM_Buffer *&pMessageDigest)
{
  CSM_AttribLst *tmpAttribs;

  pMessageDigest = NULL;

  if (m_pAttrs)
  {
    for(tmpAttribs = m_pAttrs->First();tmpAttribs && !pMessageDigest;tmpAttribs = tmpAttribs->Next() )
    {
        if(tmpAttribs->Get()->m_lContentType == CSM_Attrib::ATTR_Message_Digest)
       {
          pMessageDigest = new CSM_Buffer(*tmpAttribs->Get()->m_pMessageDigest);
       }
    }
  }

}


/* TBD: Told press on and write Pseudo Code for testing

  CSM_Attrib(CSM_MlLst *&m_pMlLst)
{   
   MailListKeyId   snaccML;

   m_lContentType = ATTR_MlExpHist;

    m_pMlLst = new CSM_MlLst;
   
    m_poid = new CSM_OID(id_aa_mlExpandHistory);

    //snaccML.Set((const unsigned char *)*m_pMlLst,(size_t)*m_pMlLst->Length());

    //ENCODE_BUF((&snaccML), m_pEncodedAttrib);
}

CSM_Attrib(CSM_GeneralAsn *&m_pGeneralAsn)
{   
   //GeneralAsn   snaccGA;

   m_lContentType = ATTR_General_ASN;

    m_pGeneralAsn = new CSM_GeneralAsn;
   
    //m_poid = new CSM_OID(???);

    //snaccML.Set((const unsigned char *)*m_pMlLst,(size_t)*m_pMlLst->Length());

    //ENCODE_BUF((&snaccML), m_pEncodedAttrib);
}
*/



// EOF sm_Attr.cpp
