#ifndef NO_SCCS_ID
static char SccsId[ ] = "@(#) sm_mabRout.cpp 1.6 3/26/98 11:50:21"; 
#endif

//////////////////////////////////////////////////////////////////////////
//  sm_mabRout.cpp
//  This set of routines support the MSP test address book 
//  identifying the location and DN of binary certificates.
#include <malloc.h>
#ifdef WIN32
#include <conio.h>
#include <stdlib.h>
#endif
#include <ctype.h>
#include <string.h>
#include <stdio.h>

#include "sm_api.h"

static char MABGlobalFileNameStore[256]="";

#define MAB_MAX_ENTRY_LEN 2048
#define MAB_PATH_KW "PATH="
#define MAB_ENTRY_KW "ENTRY="
#define MAB_NULL_VALUE "MAB_NULL"
#define MAB_DEFAULT_AB_FN "./certs/mabRecips.dat"

//////////////////////////////////////////////////////////////////////////
// convert null terminated string to all upper case
void MABtoupper(char *pszDest, char *pszSrc)
{
   int n;
   for (n=0; n < (int)strlen(pszSrc); n++)
      pszDest[n] = toupper(pszSrc[n]);
   pszDest[n] = '\0';
}

//////////////////////////////////////////////////////////////////////////
//  MABInit
//  This routine initializes access to the address book data structures.
SM_RET_VAL MAB_AB_def::Init(char *pszFileName)
{
   FILE *fp = NULL;
   MAB_Entrydef *pNewEntry;
   char szLine[MAB_MAX_ENTRY_LEN];
   char *pchTmpBuf;
   char tmpBuf2[512];
   char *pszField;  // used in strtok-ing the input line

   SME_SETUP("MAB_AB_def::Init");

   if (pszFileName == NULL)
      SME_THROW(SM_MISSING_PARAM, "No mab file name provided", NULL);

   // the format of this file allows for comments, '#', the format of
   // the entries is as follows:
   // ENTRY=alias:DN:CertFN:PrivateKeyInfo:PrivateKeyOID
   //
   // fields that can be and are null must contain MAB-NULL

   // open the provided address book file
   if ((fp = fopen(pszFileName, "r")) == NULL)
      SME_THROW(SM_MAB_ERROR, "couldn't open mab file", NULL);

   // read the entires
   while (fgets(szLine, MAB_MAX_ENTRY_LEN, fp) != NULL)
   {
      if (szLine[0] != '#') // ignore comment lines
      {
         if (strncmp(szLine, MAB_PATH_KW, strlen(MAB_PATH_KW)) == 0)
         {
            // copy path
            strcpy(m_szGlobalPath, &szLine[(strlen(MAB_PATH_KW))]);
            // remove return
            if (m_szGlobalPath[strlen(m_szGlobalPath)-1] == '\n')
               m_szGlobalPath[strlen(m_szGlobalPath)-1] = '\0';
         }
         else if (strncmp(szLine, MAB_ENTRY_KW, strlen(MAB_ENTRY_KW)) 
               == 0)
         {
            // create new entry
            if ((pNewEntry = new MAB_Entrydef) == NULL)
               SME_THROW(SM_MEMORY_ERROR, NULL, NULL);

            // point at first field
            pchTmpBuf = &szLine[strlen(MAB_ENTRY_KW)]; 

            // get the alias field, mandatory
            if ((pszField = strtok(pchTmpBuf, ":")) == NULL)
               SME_THROW(SM_MAB_ERROR, "failed getting alias", NULL);
            if ((pNewEntry->m_pszAlias = strdup(pszField)) == NULL)
               SME_THROW(SM_MEMORY_ERROR, NULL, NULL);

            // get the cert subject dn field, mandatory
            if ((pszField = strtok(NULL, ":")) == NULL)
               SME_THROW(SM_MAB_ERROR, "failed getting subject dn", NULL);
            if ((pNewEntry->m_pszCertificateDN = strdup(pszField)) == NULL)
               SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
            // make all X.500 DNs upper case 
            // (case insensitive for compares)
#ifdef SM_CASE_INSENSITIVE
            MABtoupper(pNewEntry->m_pszCertificateDN, 
                  pNewEntry->m_pszCertificateDN);
#endif
            // get the cert file name, mandatory
            if ((pszField = strtok(NULL, ":")) == NULL)
               SME_THROW(SM_MAB_ERROR, "failed getting cert fn", NULL);
            // start with the global path
            strcpy(&tmpBuf2[0], m_szGlobalPath);
            // concatenate cert FN onto temp buffer
            strcat(&tmpBuf2[0], pszField);
            // store cert FN in buffer
            pNewEntry->m_pCertFile = new CSM_Buffer(tmpBuf2);

            // get the private key path, optional
            if ((pszField = strtok(NULL, ":")) != NULL)
            { // RWC;SME_THROW(SM_MAB_ERROR, "failed getting private key", NULL);
              // if private key is not null, store it in a buffer
              if (strcmp(pszField, MAB_NULL_VALUE) != 0)
               if ((pNewEntry->m_pPrivateInfo = new CSM_Buffer(pszField)) 
                     == NULL)
                  SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
            }

            // get the private key oid, optional
            if ((pszField = strtok(NULL, ":")) == NULL)
               SME_THROW(SM_MAB_ERROR, "failed getting private key oid", 
                     NULL);
            // strip off extra char if necessary
            if ((pszField[strlen(pszField)-1] == '\n') ||
                  (pszField[strlen(pszField)-1] == '\r'))
               pszField[strlen(pszField)-1] = '\0';
            // if private key is not null, store it in a buffer
            if (strcmp(pszField, MAB_NULL_VALUE) != 0)
               pNewEntry->m_pPrivateOID = new CSM_OID(pszField);

            // put the entry in the list
            if (m_pEntries == NULL)
            {
               // currently no entries, create the list
               if ((m_pEntries = new CSM_List<MAB_Entrydef>(pNewEntry)) 
                     == NULL)
                  SME_THROW(SM_MEMORY_ERROR, NULL, NULL);
            }
            else
            {
               // there are entries, append to the list
               m_pEntries->Append(pNewEntry);
            }
         } // end if MAB_ENTRY_KW handler
      } // end if comment check
   } // end of while loop to read mab file
   
   fclose(fp);

   SME_FINISH
   SME_CATCH_SETUP
      if (fp != NULL)
         fclose(fp);
      // TBD, more catch/cleanup logic as necessary
   SME_CATCH_FINISH

   return SM_NO_ERROR;
}

//////////////////////////////////////////////////////////////////////////
MAB_AB_def::MAB_AB_def()
{
   m_pEntries = NULL;
   Init(MAB_DEFAULT_AB_FN);
}

//////////////////////////////////////////////////////////////////////////
MAB_AB_def::MAB_AB_def(char *pszFileName)
{
   m_pEntries = NULL;
   Init(pszFileName);
}

//////////////////////////////////////////////////////////////////////////
SM_RET_VAL MAB_AB_def::FindCertDN(char *pszDN, CSM_Buffer *result)
{
   SM_RET_VAL lStatus = -1;
   MAB_Entrydef *pRec = (MAB_Entrydef *)NULL;

   if ((pRec = FindRecord(pszDN, MABTYPE_DN)) != (MAB_Entrydef *)NULL)
   {
      if (result && pRec->m_pCertFile)
      {
         *result = *pRec->m_pCertFile;
         lStatus = SM_NO_ERROR;
      }
   }
   return lStatus;
}

//////////////////////////////////////////////////////////////////////////
SM_RET_VAL MAB_AB_def::FindCertAlias(char *pszAlias, CSM_Buffer *result)
{
   SM_RET_VAL lStatus = -1;
   MAB_Entrydef *pRec = (MAB_Entrydef *)NULL;

   if ((pRec=FindRecord(pszAlias, MABTYPE_ALIAS)) != (MAB_Entrydef *)NULL)
   {
      if (result && pRec->m_pCertFile)
      {
         *result = *pRec->m_pCertFile;
         lStatus = SM_NO_ERROR;
      }
   }
   return lStatus;
}

//////////////////////////////////////////////////////////////////////////
SM_RET_VAL MAB_AB_def::FindIssuer(CSM_IssuerAndSerialNumber &csmIssuer, 
                            CSM_Buffer *result)
{
   SM_RET_VAL lStatus = -1;
   CSM_List<MAB_Entrydef> *pTmp = m_pEntries->First();

   if (pTmp != NULL)
   {
      do
      {
         if (csmIssuer == *((pTmp->Get())->GetIssuer()))
         {
            *result = *((pTmp->Get())->m_pCertFile);
            lStatus = SM_NO_ERROR;
            break;
         }
      } while ((pTmp = pTmp->Next()) != NULL);
   }
   return lStatus;
}

//////////////////////////////////////////////////////////////////////////
MAB_Entrydef *MAB_AB_def::FindRecord(char *pszID, long lType)
{
   MAB_Entrydef *result = (MAB_Entrydef *)NULL;
   long lStat = -1;
   char szUpperID[512];
   CSM_List<MAB_Entrydef> *pTmp = m_pEntries->First();

   if (pTmp != NULL)
   {
#ifdef SM_CASE_INSENSITIVE
      MABtoupper(szUpperID, pszID);
#else
      strcpy(szUpperID, pszID);
#endif
      do
      {
         if (lType == MABTYPE_DN)
            lStat = (long)strcmp((pTmp->Get())->m_pszCertificateDN, 
                  szUpperID);
         else if (lType == MABTYPE_ALIAS)
            lStat = (long)strcmp((pTmp->Get())->m_pszAlias, szUpperID);
         if (lStat == 0)
         {
            result = pTmp->Get();
            break;
         }
      } while ((pTmp = pTmp->Next()) != NULL);
   }
   return result;
}


//////////////////////////////////////////////////////////////////////////
SM_RET_VAL MAB_AB_def::FillCertPath(
    CSM_CertificateChoice *pCertToPath,
    CSM_MsgCertCrls *pMsgCertCrls)
{
   int status=0;
   char *section="";
   CSM_List<MAB_Entrydef> *pTmp;
   char *pTmpIssuerDN=(char *)calloc(1,4096);
   char *pTmpSubjectDN=(char *)calloc(1,4096);
   CSM_CertificateChoice *pTmpCert;
   CSM_CertificateChoice *pTmpCertChoice;
   CSM_DN *pSubjectDN;
   CSM_DN *pIssuerDN;

      pTmpCertChoice = pCertToPath;    // Assign first cert in list.
      pSubjectDN = pTmpCertChoice->GetSubject();
      pIssuerDN = pTmpCertChoice->GetIssuer();
#ifdef SM_CASE_INSENSITIVE
      MABtoupper(pIssuerDN->GetDNString(), pTmpIssuerDN);
      MABtoupper(pSubjectDN->GetDNString(), pTmpSubjectDN);
#else           // CASE Sensitive for NOW.
      strcpy(pTmpIssuerDN, pIssuerDN->GetDNString());
      strcpy(pTmpSubjectDN, pSubjectDN->GetDNString());
#endif
      while(pTmpCertChoice && strcmp(pTmpIssuerDN, pTmpSubjectDN) != 0)
      {                   // STOP at root PCA/CA; IssuerDN=SubjectDN.
        pTmp = this->m_pEntries->First();
        pTmpCert = NULL;        // FLAG that not found.
        do                // For each address book recipient entry.
        {
          if (strcmp(pTmpIssuerDN, pTmp->Get()->m_pszCertificateDN) == 0)
          {
            pTmpCert = new CSM_CertificateChoice(pTmp->Get()->m_pCertFile, 
                CSM_CertificateChoice::CERT_TYPE_CERT);
            pMsgCertCrls->AddCert(pTmpCert);

            status = SM_NO_ERROR;
            break;
          }
        }  while ((pTmp = pTmp->Next()) != NULL);

        // DO NOT DELETE pTmpCert, it is assigned to pMsgCertCrls through 
        //   the CSM_List class Append().
        if (pTmpCert != NULL)             // FOUND one.
        {
          pTmpCertChoice = pTmpCert;
          // DO NOT DELETE pSubjectDN and pIssuerDN since only pointer to 
          //  "pTmpCertChoice" components.
          pSubjectDN = pTmpCertChoice->GetSubject();
          pIssuerDN = pTmpCertChoice->GetIssuer();
#ifdef SM_CASE_INSENSITIVE
          MABtoupper(pIssuerDN->GetDNString(), pTmpIssuerDN);
          MABtoupper(pSubjectDN->GetDNString(), pTmpSubjectDN);
#else           // CASE Sensitive for NOW.
          strcpy(pTmpIssuerDN, pIssuerDN->GetDNString());
          strcpy(pTmpSubjectDN, pSubjectDN->GetDNString());
#endif
        }
        else
            pTmpCertChoice = NULL;
      }             // END While certs to process.

      free(pTmpIssuerDN);
      free(pTmpSubjectDN);

    return(status);
}


//////////////////////////////////////////////////////////////////////////
MAB_AB_def::~MAB_AB_def()
{
   if (m_pEntries != NULL)
      delete m_pEntries;
}

//////////////////////////////////////////////////////////////////////////
CSM_IssuerAndSerialNumber *MAB_Entrydef::GetIssuer()
{
   if (!m_pIssuer)
      m_pIssuer = new CSM_IssuerAndSerialNumber(m_pCertFile);
   return(m_pIssuer);
}

//////////////////////////////////////////////////////////////////////////
Certificate *MAB_Entrydef::GetCertificate()
{
   if (!m_pCertificate)
   {
      if (m_pCertFile)
         DECODE_BUF(m_pCertificate, m_pCertFile);
   }
   return(m_pCertificate);
}



//////////////////////////////////////////////////////////////////////////
MAB_Entrydef::~MAB_Entrydef()
{
   if (m_pCertificate)
      delete m_pCertificate;
   if (m_pIssuer)
      delete m_pIssuer;
   if (m_pszAlias)
      free(m_pszAlias);
   if (m_pszCertificateDN)
      free(m_pszCertificateDN);
   if (m_pCertFile)
      delete m_pCertFile;
   if (m_pPrivateInfo)
      delete m_pPrivateInfo;
   if (m_pPrivateOID)
      delete m_pPrivateOID;
}

// EOF sm_mabRout.cpp
