/* @(#) sm_util.h 1.19 3/30/98 16:22:33 */

//////////////////////////////////////////////////////////////////////////
// This header contains defintions, defines, and prototypes of the test
// code used to test the SFL and related CTI libraries...
//////////////////////////////////////////////////////////////////////////
#ifndef _SM_UTIL_H_
#define _SM_UTIL_H_

#include <string.h>
#include "sm_api.h"

//////////////////////////////////////////////////////////////////////////
// this file includes prototypes of test functions
#define SM_PATH_LEN 512 // size of arrays to hold paths & file names

#define DEFAULT_CFG_FILE "default.cfg"

#define CFG_KW_SZ      40  /* maximum length of the Keyword */
#define CFG_KW_VAL_SZ 512  /* maximum length of the Keyword Value */
#define CFG_SECT_SZ    40  /* maximum length of the Section label */

// maximum length of a line in a configuration file
#define CFG_MAX_LINE_LEN  CFG_KW_VAL_SZ + CFG_KW_SZ + 2

typedef char Keyword[CFG_KW_SZ];
typedef char KwValue[CFG_KW_VAL_SZ];
typedef char Section[CFG_SECT_SZ];

extern MAB_AB_def *p_GAddressBook;      // Declared in sm_Autohi.cpp

//////////////////////////////////////////////////////////////////////////
// PROTOTYPES
//////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////
// the functions in the prototype block below are utility testing functions
int SM_CfgDriver (char *keyword, char *destStr, char *filename);
char * SM_SetCfgFn(char *filename);
long SM_GetDfKwValue ( Keyword keyword, KwValue destStr, Section section);
long SM_GetDfCertIndex ( Keyword keyword, Section section);
long SM_GetKwValue (Keyword keyword, KwValue destStr, Section section);
long SM_GetEntireSection(char *pszSection, char **pszAllEntries);
SM_RET_VAL SM_Hex2BufferAlloc(char *pszHex, CSM_Buffer *&buffer);
SM_RET_VAL SM_Hex2Buffer(char *pszHex, CSM_Buffer *buffer);

//////////////////////////////////////////////////////////////////////////
// The SM_Auto*Function functions read the lists of tests to run from
// the specified config files and call the appropriate test drivers...
SM_RET_VAL SM_AutoHiFunction(
      char *lpszINConfigFile,/** IN, config file name **/
      CSMIME *pCsmime,         /** IN, valid Logins. **/
      char *lpszINConfigSection,/** IN, section identifying hilevel ops **/
      long output_flag,  /** IN, print flag options **/
      long verify_flag);  /** IN, verification flag options **/
SM_RET_VAL SM_AutoLoFunction(
      char *lpszProg,       // IN, Keyword for test
      char *lpszINConfigFile,/** IN, config file name **/
      CSMIME *pCsmime,         /** IN, valid Logins. **/
      char *lpszINConfigSection,/** IN, section identifying hilevel ops **/
      char *lpszOutFileName,    // IN, file (optional, if necessary)
      long output_flag,  /** IN, print flag options **/
      long verify_flag);  /** IN, verification flag options **/
SM_RET_VAL SM_AutoAlgFunction(
      char *lpszProg,
      char *lpszINConfigFile,/** IN, config file name **/
      char *lpszINConfigSection,/** IN, section identifying hilevel ops **/
      char *lpszOutputFile,
      long output_flag,  /** IN, print flag options **/
      long verify_flag); /** IN, verification flag options **/

//////////////////////////////////////////////////////////////////////////
// The CfgCheck* functions are the actual "test driver" functions
SM_RET_VAL SM_CfgCheckSignedData(char *lpszConfigFile, CSMIME *pCsmime,
      long output_flag,  /** IN, print flag options **/
      long verify_flag);  /** IN, verification flag options **/
SM_RET_VAL SM_CfgCheckDN(char *pszDnString, 
      long output_flag, long verify_flag);
SM_RET_VAL SM_CfgCheckIss(char *pszSerialNo, char *pszDnString, 
      long output_flag, long verify_flag);
SM_RET_VAL SM_CfgCheckEncrypt(char *pszCfgFile, CSMIME *pCSMIME,
                              long output_flag, long verify_flag);
SM_RET_VAL SM_CfgCheckDecrypt(char *pszCfgFile, CSMIME *pCSMIME,
                              long output_flag, long verify_flag);
SM_RET_VAL SM_CfgCheckVerify(char *pszCfgFile, CSMIME *pCsmime,
      long output_flag, long verify_flag);
long SM_CfgCert(Certificate *cert, Section  sectSrc);
SM_RET_VAL SM_CfgCheckContentInfo(char *lpszContentInfoFile, 
    char *lpszContentFile, char *lpszContentType, bool bEncodeFlag);

//////////////////////////////////////////////////////////////////////////
// the functions in the block of prototypes below are the test fill routines
// that are responsible for preparing objects for testing (ie filling them)
SM_RET_VAL SM_CfgSignedData(CSMIME *pCsmime, CSM_SignMsg *pSignedData, 
       Section section);
SM_RET_VAL SM_CfgFillEncryptMsg(CSMIME *pCSMIME, CSM_EncryptMsg *pEncryptMsg,
                               Section section);
SM_RET_VAL SM_CfgFillDecryptMsg(CSMIME *pCSMIME, CSM_DecryptMsg *pDecryptMsg,
                                Section section);

//////////////////////////////////////////////////////////////////////////
// misc test function prototypes
long SM_SnaccSetOfTest(
      char *lpszINConfigFile,/** IN, config file name **/
      CSMIME *pCsmime,   /** IN, valid Logins. **/
      long output_flag,  /** IN, print flag options **/
      long verify_flag); /** IN, verification flag options **/
long SM_SnaccCertEncode(
      char *lpszINConfigFile,/** IN, config file name **/
      CSMIME *pCsmime,   /** IN, valid Logins. **/
      long output_flag,  /** IN, print flag options **/
      long verify_flag); /** IN, verification flag options **/
SM_RET_VAL SM_CfgAliasCertFile(char *lpszAlias,CSM_Buffer *&pCertBuf);
SM_RET_VAL SM_CfgAliasCertDN(char *lpszDN, CSM_Buffer *&pCertBuf);
SM_RET_VAL SM_CfgCertsSection(char *lpszSection, 
    CSM_MsgCertCrls *pMsgCertsCrls);
void MABtoupper(char *pszDest, char *pszSrc);


//////////////////////////////////////////////////////////////////////////
// The functions in the block of prototypes below are debug print funcs
void SM_PrtSignData(CSM_SignMsg *pSignData);
void SM_PrtHex(unsigned char *buf, unsigned long len);

//////////////////////////////////////////////////////////////////////////
// SM_Free CTI Test Functions
#ifdef SM_FREE_USED
SM_RET_VAL SM_FreeEncryptTest(char *pszConfigFile, char *pszConfigSection,
                              long output_flag, long verify_flag);
SM_RET_VAL SM_FreeDecryptTest(char *pszConfigFile, char *pszConfigSection,
                              long output_flag, long verify_flag);
SM_RET_VAL SM_FreeGenerateEMEKTest(char *pszConfigFile, char *pszConfigSection,
                              long output_flag, long verify_flag);
SM_RET_VAL SM_FreeExtractMEKTest(char *pszConfigFile, char *pszConfigSection,
                              long output_flag, long verify_flag);
SM_RET_VAL SM_FreeRandomTest(char *pszConfigFile, char *pszConfigSection,
                              long output_flag, long verify_flag);
SM_RET_VAL SM_FreeSignTest(char *pszConfigFile, char *pszConfigSection,
                              long output_flag, long verify_flag);
SM_RET_VAL SM_FreeVerifyTest(char *pszConfigFile, char *pszConfigSection,
                              long output_flag, long verify_flag);
SM_RET_VAL SM_FreeDigestTest(char *pszConfigFile, char *pszConfigSection,
                              long output_flag, long verify_flag);
SM_RET_VAL SM_FreeGenerateDSAKeys(char *pszConfigFile, char *pszConfigSection,
                              long output_flag, long verify_flag);
SM_RET_VAL SM_FreeGenerateDHKeys(char *pszConfigFile, char *pszConfigSection,
                              long output_flag, long verify_flag);
#endif

// SM_Rsa Test Functions
#ifdef SM_RSA_USED
SM_RET_VAL SM_RsaEncryptTest(char *pszConfigFile, char *pszConfigSection,
                              long output_flag, long verify_flag);
SM_RET_VAL SM_RsaRandomTest(char *pszConfigFile, char *pszConfigSection,
                              long output_flag, long verify_flag);
SM_RET_VAL SM_RsaDigestTest(char *pszConfigFile, char *pszConfigSection,
                              long output_flag, long verify_flag);
#endif

#endif // _SM_UTIL_H_

// EOF sm_util.h
