#ifndef NO_SCCS_ID
static char SccsId[ ] = "@(#) sm_Autohif.cpp 1.13 3/30/98 08:47:41"; 
#endif

/* SM_auto_asnf.c */

#include <stdio.h>
#include <stdlib.h>
#include "sm_util.h"

/*** 
  SM_auto_asnFunction
  This routine handles the interpretation of configuration file entries for
  SMIME enc/dec functions and their execution.  A status is returned.
***/
SM_RET_VAL SM_AutoHiFunction(
      char *lpszINConfigFile,/** IN, config file name **/
      CSMIME *pCsmime,         /** IN, valid Logins. **/
      char *lpszINConfigSection,/** IN, section identifying hilevel ops **/
      long output_flag,  /** IN, print flag options **/
      long verify_flag)  /** IN, verification flag options **/
{
   long status=0;
   int testIndex=-1;        /** index of the test being executed in while().**/
   int sectionFound=0;
   FILE *fp;
   char buf[250];
   char lpszProg[80], lpszConfigFile[200], lpszConfigSection[80];
   char lpszOutFileName[80];
   short local_clear = 0;
   short depend_clear = 0;
   
   SME_SETUP("SM_AutoHiFunction");
   if ((fp=fopen(lpszINConfigFile, "r")) != NULL)
   {
      /** first, find our section. **/
      while (status == 0 && !sectionFound && fgets(buf, 80, fp) != NULL)
      {                  /** only perform "fgets()" if "sectionNotFound".**/
          if (buf[0] == '[' && strncmp(&buf[1], lpszINConfigSection, 
              strlen(lpszINConfigSection)) == 0)
          {
              sectionFound = 1;
          }            
      }

      /** next, Read each entry in order and execute until EOF or next 
          section. **/
      while (sectionFound && (status == 0  || status == 3) && 
             fgets(buf, 250, fp) != NULL && 
             buf[0] != '[')     /** next section name. **/
      {
         if (buf[strlen(buf)-1] == '\n')
               buf[strlen(buf)-1] = '\0';        /** terminate, no return. **/
         if (buf[0] != '#' && buf[0] != '\0')      /** comment line **/
         {
            lpszProg[0] = '\0';
            lpszConfigFile[0] = '\0';
            lpszConfigSection[0] = '\0';
            lpszOutFileName[0] = '\0';
            sscanf(buf, "%s %s %s %s", lpszProg, lpszConfigFile, 
            lpszConfigSection, lpszOutFileName);

            // FIRST, check for Lolevel, or Algorithm specific tests.
            if (strncmp(lpszProg, "LO_", 2) == 0)
            {
                status = SM_AutoLoFunction(lpszProg, lpszConfigFile, 
                     pCsmime, lpszConfigSection, lpszOutFileName,
                     output_flag, verify_flag);
            }
            else if (strncmp(lpszProg, "ALG_", 3) == 0 || 
                     strncmp(lpszProg, "FREE_", 5) == 0 ||
                     strncmp(lpszProg, "RSA_", 4) == 0)
            {
                status = SM_AutoAlgFunction(
                     lpszProg, lpszConfigFile, 
                     lpszConfigSection, lpszOutFileName,
                     output_flag, verify_flag);
            }

            /* process buf, label of hilevel function, and config file to 
               execute. */
            else if (strcmp(lpszProg, "SM_SnaccSetOfTest") == 0)
            {
               status = SM_SnaccSetOfTest(lpszConfigFile,  pCsmime, 
                   output_flag, verify_flag);
            }
            else if (strcmp(lpszProg, "SM_SnaccCertEncode") == 0)
            {
               status = SM_SnaccCertEncode(lpszConfigFile,  pCsmime, 
                   output_flag, verify_flag);
            }
            else if (strcmp(lpszProg, "CSM_SignMsg::Sign") == 0)
            {
               status = SM_CfgCheckSignedData(lpszConfigFile,  pCsmime, 
                   output_flag, verify_flag);
            }
            else if (strcmp(lpszProg, "CSM_VerifyMsg::Verify") == 0)
            {
               status = SM_CfgCheckVerify(lpszConfigFile,  pCsmime, 
                   output_flag, verify_flag);
            }
            else if (strcmp(lpszProg, "sm_ContentInfoEncode") == 0)
            {
               status = SM_CfgCheckContentInfo(lpszConfigFile, 
                   lpszConfigSection, lpszOutFileName, true);
            }
            else if (strcmp(lpszProg, "sm_ContentInfoDecode") == 0)
            {
               status = SM_CfgCheckContentInfo(lpszConfigFile, 
                   lpszConfigSection, lpszOutFileName, false);
            }
            else if (strcmp(lpszProg, "CSM_EncryptMsg::Encrypt") == 0)
            {
               status = SM_CfgCheckEncrypt(lpszConfigFile, pCsmime,
                     output_flag, verify_flag);
            }
            else if (strcmp(lpszProg, "CSM_DecryptMsg::Decrypt") == 0)
            {
               status = SM_CfgCheckDecrypt(lpszConfigFile, pCsmime,
                     output_flag, verify_flag);
            }
            else if (strcmp(lpszProg, "SM_TestMAB") == 0)
            {
               //status = TestMAB();
               printf("NOT Currently Supported.\n");
            }
            else if (strcmp(lpszProg, "Generate_Cert") == 0)
            {
                Certificate SNACCCert;
                CSM_Buffer *blob=NULL;
                char *ptr;
                SM_SetCfgFn(lpszConfigFile);
                status = SM_CfgCert(&SNACCCert, lpszConfigSection);
                ENCODE_BUF(&SNACCCert, blob);
                if ((ptr=strrchr(lpszConfigFile, '.')) != NULL)
                    strcpy(ptr, ".out");
                else
                    strcat(lpszConfigFile, ".out");
                blob->ConvertMemoryToFile(lpszConfigFile);//"./certCfg.out");
            }
            else
                printf("NO PROCESSING PERFORMED: %s ####################.\n",
                     buf);

            /* is status (return of cfg4_check* function SM_NO_ERROR? */
            if (status == SM_NO_ERROR || status == 3)
               printf("SUCCESS ON");
            else
               printf("UNSUCCESSFUL ON");
            printf(" %ld:%s %s %s %s ######################\n",
                  status, lpszINConfigSection, lpszProg, lpszConfigFile, 
                  lpszConfigSection);
         } /* condition for comment line */
      } /* go through all entries */
      fclose(fp);
      if (status == 3)
        status = 0;       /** return successful, not "3" to caller. **/
   }
   else
   {
      printf("SM_AutoHiFunction: could not open testConfig File=%s.\n", 
          lpszConfigFile);
      status = -1;
   }

   SME_FINISH_CATCH

   return(status);
}

/*** EOF SM_autohif.c ***/
