#ifndef NO_SCCS_ID
static char SccsId[ ] = "@(#) sm_CertTest.cpp 1.3 2/12/98 11:20:08"; 
#endif

/***
  sm_CertTest.cpp
  This main program will read the binary certificate files from the specified
  directory (CERT_DIRECTORY) and re-encode the certificate.
  ***/

#ifdef BOB
#include <stdlib.h>
#include <math.h>
#include "mosaic.h"   /** MSP include header **/
#include "mosaic4.h"   /** MSP include header **/
#include "cryptint.h"
#include "proto.h"
#include "proto4.h"
#include "util4.h"
long mabu_get_nextFile(char *fileName);
long mabu_get_firstFile(char *mab_directory, char *fileName);
long mabu_get_dnAlias(char *fileName, char *lpszDn, char *lpszAlias, 
                      char *lpszVer, char *lpszId, long *cert_index);
long mabu_set_Entry(char *fileName, char *lpszDn, char *lpszAlias, 
                    char *lpszVer, char *lpszId, long cert_index);
long dn2str(Dn4 *dn,        /** IN, DN struct input. **/
            char *lpszDn);   /** OUT, string result. **/
long mabu_write(char *fileName);

long GLOBAL_LAST_COUNT=0;
char GLOBAL_Directory[1024];

short debug; /* referenced by cfg4_crl, cfg4_cert, cfg4_exts, cfg4_genSubs */
#endif





#ifdef WIN32
#include <windows.h>
#else
#include <dirent.h>
#endif

#include "sm_api.h"
#define CERT_DIRECTORY "./certs"
char GLOBAL_Directory[1024];
long mabu_get_firstFile(char *mab_directory, char *fileName);
long mabu_get_nextFile(char *fileName);
long mabu_processCert(char *fileName);



// This function tests the ASN.1 DER encoding of certificates.
//  It will read a directory of certificates and re-encode each
//  for easy comparison with the original.  The directory is
//  hardcoded as "./certs"; the resulting duplicate files are named
//  "file.ext2".
long SM_SnaccCertEncode(
      char *lpszINConfigFile,/** IN, config file name **/
      CSMIME *pCsmime,   /** IN, valid Logins. **/
      long output_flag,  /** IN, print flag options **/
      long verify_flag) /** IN, verification flag options **/
{
    long status=0;
    char *mab_file=NULL;
    char *mab_directory=NULL;
    char fileName[1024];
    long cert_index=0;

    mab_directory = CERT_DIRECTORY;
        if ((status=mabu_get_firstFile(mab_directory, fileName)) == 0)
        {
            if (strcmp(fileName, ".") != 0 &&
                strcmp(fileName, "..") != 0)
              status = mabu_processCert(fileName);
            while (status == 0 && mabu_get_nextFile(fileName) == 0)
            {
                if (strcmp(fileName, ".") != 0 &&
                    strcmp(fileName, "..") != 0)
                {
                    status = mabu_processCert(fileName);
                }

                printf("mabu_processCert: successful load %s.\n", fileName);
            }
        }
        else
            printf("mab_update: bad mabu_get_firstFile() call.\n");
    
    return(status);
}

#ifdef WIN32
HANDLE handle;              /*** GLOBAL for easy API call. ***/
WIN32_FIND_DATA result;
#else
DIR *unixHandle;
#endif
long mabu_get_firstFile(char *mab_directory, char *fileName)
{
    int status=-1;
    char buf[1024];

    strcpy(GLOBAL_Directory, mab_directory);
    sprintf(buf, "%s/%s", GLOBAL_Directory, "*.crt");
#ifdef WIN32
    {
        if ((handle=FindFirstFile(buf, &result)) != INVALID_HANDLE_VALUE)
        {
            strcpy(fileName, result.cFileName);
            status = 0;
        }
        else
            printf("mab_update: bad return from FindFirstFile.\n");
    }
#else
    {
        struct dirent *dirp = NULL;

        printf("opening unix dir: %s\n", mab_directory);

        unixHandle = opendir(mab_directory);
        if (unixHandle == NULL)
        {
           printf("mab_update: bad return from opendir()\n");
           return(2);
        }
        while ((dirp=readdir(unixHandle)) && dirp != NULL && 
               dirp->d_name[0] == '.');
        if (dirp == NULL)
        {
           printf("mab_update: bad return from readdir()\n");
           return(2);
        }
        sprintf(fileName, "%s/%s", GLOBAL_Directory, dirp->d_name);
        status = 0;
/*
        printf("mab_get_firstFile: unix commands not written yet (opendir(),readdir()).\n");
*/
    }
#endif
    return(status);
}



long mabu_get_nextFile(char *fileName)
{
    int status=-1;

#ifdef WIN32
    {
        if (FindNextFile(handle, &result))
        {
          if (result.cFileName[0] != '.')
          {
            strcpy(fileName, GLOBAL_Directory);
            strcat(fileName, "/");
            strcat(fileName, result.cFileName);
          }
          else
              strcpy(fileName, result.cFileName);   /** flag "." **/
          status = 0;
        }
        else
        {
            if (GetLastError() == ERROR_NO_MORE_FILES)
                status = 2;     /** flag indicating end of list. **/
            else
              printf("mab_update: bad return from FindFirstFile.\n");
            FindClose(handle);
            handle = 0;
        }
    }
#else
    {
        struct dirent *dirp;

        if ((dirp = readdir(unixHandle)) != NULL)
        {
          if (dirp->d_name[0] != '.')
          {
            strcpy(fileName, GLOBAL_Directory);
            strcat(fileName, "/");
            strcat(fileName, dirp->d_name);
          }
          else
              strcpy(fileName, dirp->d_name);   /** flag "." **/
          status = 0;
        }
        else
        {
          status = 2;     /** flag indicating end of list. **/
          closedir(unixHandle);
        }
/*
        printf("mabu_get_nextFile: unix commands not written yet (opendir(),readdir()).\n");
*/
    }
#endif
    return(status);
}



long mabu_processCert(char *fileName)
{
    long status=0;
    Certificate snaccCert;
    CSM_Buffer *tmpBuf=new CSM_Buffer(fileName);
    CSM_Buffer *tmpBuf2;
    char *fileName2=strdup(fileName);

    DECODE_BUF(&snaccCert, tmpBuf); // decode certificate.
    ENCODE_BUF(&snaccCert, tmpBuf2);
    strcat(fileName2, "2");
    tmpBuf2->ConvertMemoryToFile(fileName2);

    return(status);
}



/*** EOF sm_CertTest.cpp ***/
