#ifndef NO_SCCS_ID
static char SccsId[ ] = "@(#) sm_CfgCert.cpp 1.6 3/16/98 16:02:40"; 
#endif


//
// sm_CfgCert.cpp

#include <stdlib.h>
#include <malloc.h>
#include <string.h>
#include "sm_util.h"

long str2dn(char *dn_str, Name  *&nameP);
long str2dn_RDN_OID(AttributeTypeAndDistinguishedValue *pRDN, char *dn_label_data);

/*
 * FUNCTION: SM_CfgCert()
 * 
 * PURPOSE: to read the following keywords from the specified section of
 *          the current configuration file.
 *
 * KEYWORDS:     
 *    Name                   Value
 *    -------------------------------------------
 *    version                integer
 *    serial_no              integer in hex format
 *    issuer_dn              string representing a DN. 
 *    val_not_before         string representing a UTC Date.
 *    val_not_after          string representing a UTC Date.
 *    subject_dn             string representing a DN.
 *    issuer_id              string representing a BIT STRING.
 *    subj_id                string representing a BIT STRING.
 *    exts                   string of integers seperated by spaces and 
 *                             terminated with a -1.
 *    cert_sig_alg_id        string rep. cert. signature alg. OID
 *    cert_sig_dsaparams     string rep. section for DSA Parameters
 *    subj_pubkey_alg_id     string rep. subject public key alg. OID
 *    subj_pubkey_dsaparams  string rep. section for DSA Parameters
 *    kea_id                 string rep. subject public KEA ID parameters
 *    public_key             string rep. hex values of public key
 *    scert_alg_id           string rep. issuer's sig. alg. OID
 *    scert_dsaparams        string rep. section for DSA parameters
 *    sig_r_value            string of hex values representing R value
 *    sig_s_value            string of hex values representing S value
 *
 *    note: subj_pubkey_dsaparams and key_id should not be present at the
 *          same time.  kea_id is the keyword to use for parameters in a 
 *          KEA certificate and subj_pubkey_dsaparams has parameters for
 *          a DSA certificate.
 */
long SM_CfgCert(Certificate *cert, Section  sectSrc)
{
   int        *extensions = NULL;
   KwValue     kw_value,kw_value2;
   CSM_Buffer Pubkey;
   CSM_Buffer tmpBuf, tmpBuf2;
   char       *section;

   if (sectSrc == NULL)
   {
      if (SM_GetKwValue("startingCertSection", kw_value, NULL) == 0)
      {
         section = (char *) calloc(1, strlen(kw_value));
         strcpy(section, kw_value);
      }
      else
      {
         {
            printf("ERROR:\n");
            printf("Section is NULL and keyword 'startingCertSection' does\n");
            printf("not exist in Global section of the config. file.\n\n");
         }
         return 1;
      }
   }
   else 
      section = sectSrc;


   cert->certificateToSign = new CertificateToSign;
   cert->certificateToSign->version = new Version;
   if (SM_GetKwValue("version", kw_value, section) == 0)
      *cert->certificateToSign->version = atoi(kw_value);
   else
      *cert->certificateToSign->version = 3;

   if (SM_GetKwValue("serial_no", kw_value, section) == 0)
   {
       SM_Hex2Buffer (kw_value, &tmpBuf);
       cert->certificateToSign->serialNumber.Set((char *)tmpBuf, 
           tmpBuf.Length());
   }

   
   /* If keyword does not exist set Algorithm ID ERROR
    */
   cert->certificateToSign->signature = new AlgorithmIdentifier;
   if (SM_GetKwValue("cert_sig_alg_id", kw_value, section) == 0)
      cert->certificateToSign->signature->algorithm = CSM_OID(kw_value);
   else
       printf("cfg_cert: MISSING signature->algorithm.\n");

   /*
    * Load in DSA Parameters for cert_sig.  If the keyword does not
    * exist set  to NULL.
    */
   if (SM_GetKwValue("cert_sig_dsaparams", kw_value, section) == 0)
   {
      cert->certificateToSign->signature->parameters = new AsnAny;
      //RWC; TBD;cfg_dsaParams( cert->cert_sig.dss_params, kw_value );
      tmpBuf = CSM_Buffer("bob test params", strlen("bob test params"));
      SM_ASSIGN_ANYBUF(&tmpBuf, cert->certificateToSign->signature->parameters);
   }

   if (SM_GetKwValue("issuer_dn", kw_value, section) == 0)
        str2dn(kw_value, cert->certificateToSign->issuer);
   else
      cert->certificateToSign->issuer = NULL;

   if (SM_GetKwValue("val_not_before", kw_value, section) == 0)
   {
      cert->certificateToSign->validity = new Validity;
      cert->certificateToSign->validity->notBefore = new Time;
      cert->certificateToSign->validity->notBefore->choiceId = 
          Time::generalizedTimeCid;
      cert->certificateToSign->validity->notBefore->generalizedTime = 
          new GeneralizedTime(kw_value);
   }

   if (SM_GetKwValue("val_not_after", kw_value, section) == 0)
   {
      if (cert->certificateToSign->validity == NULL)
        cert->certificateToSign->validity = new Validity;
      cert->certificateToSign->validity->notAfter = new Time;
      cert->certificateToSign->validity->notAfter->choiceId = 
          Time::generalizedTimeCid;
      cert->certificateToSign->validity->notAfter->generalizedTime = 
          new GeneralizedTime(kw_value);
   }

   if (SM_GetKwValue("subject_dn", kw_value, section) == 0)
       str2dn(kw_value, cert->certificateToSign->subject);
   else
      cert->certificateToSign->subject = NULL;

   /* If subj_key_alg_id keyword does not exist fill the Subject Public Key 
    * Algorithm OID with DSAWithSHA-1
    */
   if (SM_GetKwValue("subj_pubkey_alg_id", kw_value, section) == 0)
   {
      cert->certificateToSign->subjectPublicKeyInfo = new SubjectPublicKeyInfo;
      cert->certificateToSign->subjectPublicKeyInfo->algorithm = new AlgorithmIdentifier;
      cert->certificateToSign->subjectPublicKeyInfo->algorithm->algorithm = 
          CSM_OID(kw_value);
   }
   else
      printf("cfg_cert:  no Subject Public Key Algorithm listed "
        "(subj_pubkey_alg_id=)\n");

   if (SM_GetKwValue("EncodedParams", kw_value, section) == 0)
   {
        // RWC; TBD;cfg_dsaParams(dsaParams, kw_value);
       if (cert->certificateToSign->subjectPublicKeyInfo == NULL)
           printf("cfg_cert: subjectPublicKeyInfo params must have OID.\n");
       else
           cert->certificateToSign->subjectPublicKeyInfo->algorithm->parameters
             = new AsnAny;
       //tmpBuf = CSM_Buffer("bob test PublicKeyInfo params", 
       //    strlen("bob test PublicKeyInfo params"));
       if (strchr(kw_value, '/') == NULL)
            SM_Hex2Buffer (kw_value, &tmpBuf);
       else         // FILE NAME specified, read from file.
       {
            tmpBuf.SetFileName(kw_value);
            tmpBuf.ConvertFileToMemory();
       }
       SM_ASSIGN_ANYBUF(&tmpBuf, 
          cert->certificateToSign->subjectPublicKeyInfo->algorithm->parameters);
   }

   if (SM_GetKwValue("kea_id", kw_value, section) == 0)
   {
       SM_Hex2Buffer (kw_value, &tmpBuf);
       if (cert->certificateToSign->subjectPublicKeyInfo == NULL)
           printf("cfg_cert: subjectPublicKeyInfo params must have OID.\n");
       else
           cert->certificateToSign->subjectPublicKeyInfo->subjectPublicKey.Set(
              kw_value, strlen(kw_value));
   }

   if (SM_GetKwValue("PublicKey", kw_value, section) == 0)
   {
       if (strchr(kw_value, '/') == NULL)
            SM_Hex2Buffer (kw_value, &tmpBuf);
       else         // FILE NAME specified, read from file.
       {
            tmpBuf.SetFileName(kw_value);
            tmpBuf.ConvertFileToMemory();
       }
   }
   else
   {
       strcpy(kw_value, "dummy_pubkey"); /** dummy params **/
       tmpBuf = CSM_Buffer(kw_value, strlen(kw_value));
       printf("cfg_cert: no public_key= keyword, defaulted=|%s|.\n",kw_value);
   }
   if (cert->certificateToSign->subjectPublicKeyInfo == NULL)
           printf("cfg_cert: subjectPublicKeyInfo params must have OID.\n");
   else
           cert->certificateToSign->subjectPublicKeyInfo->subjectPublicKey.Set(
             (char *)tmpBuf, tmpBuf.Length()*8);
            //CSM_Buffer Buffer;
            //SM_AsnBits2Buffer(&cert->certificateToSign->subjectPublicKeyInfo->
            //     subjectPublicKey, &Buffer);
   /* set issuer id */
   if (SM_GetKwValue("issuer_id", kw_value, section) == 0)
   {
       cert->certificateToSign->issuerUniqueIdentifier = new UniqueIdentifier;
       SM_Hex2Buffer (kw_value, &tmpBuf);
       cert->certificateToSign->issuerUniqueIdentifier->Set((char *)tmpBuf, 
          tmpBuf.Length());
   }

   /* set subject id */
   if (SM_GetKwValue("subj_id", kw_value, section) == 0)
   {
       cert->certificateToSign->subjectUniqueIdentifier = new UniqueIdentifier;
       SM_Hex2Buffer (kw_value, &tmpBuf);
       cert->certificateToSign->subjectUniqueIdentifier->Set((char *)tmpBuf, 
          tmpBuf.Length()*8);
   }
   

#ifdef BOB
   if (SM_GetKwValue("exts", kw_value, section) == 0)
   {
      extensions = atoia(kw_value);

      /* this call will eventually be replaced by a call to a cfg4_() func.*/
      error = cfg4_crtExts(cert, extensions);
      free(extensions);
   }

   if (error != 0)
   {
      printf("!!ERROR!!  Loading extensions\n");
      printf("Aborting fill..\n");
      return(error);
   }
#endif
   /* Issuer Signature Algorithm 
    *
    * If the scert_alg_id keyword does not exist then set Alogorithm ID 
    * to DSAWithSHA-1\n 
    */
   if (SM_GetKwValue("scert_alg_id", kw_value, section) != 0)
   {
       strcpy(kw_value, "1.2.3.4"); /** dummy OID **/
       printf("cfg_cert: no scert_alg_id= keyword, defaulted=|%s|.\n",kw_value);
   }
   
   cert->algorithmIndentifier = new AlgorithmIdentifier;
   cert->algorithmIndentifier->algorithm = CSM_OID(kw_value);
   

   if (SM_GetKwValue("scert_dsaparams", kw_value, section) == 0)
   {
         SM_Hex2Buffer (kw_value, &tmpBuf);
       if (cert->algorithmIndentifier == NULL)
           printf("cfg_cert: no scert_alg_id= keyword.\n");
       else
       {
         cert->algorithmIndentifier->parameters = new AsnAny;
         SM_ASSIGN_ANYBUF(&tmpBuf, cert->algorithmIndentifier->parameters);
       }
   }
#ifdef NOT_USED
   else
   {
       char tmpChar[2];
       strcpy(kw_value, "dummy_params"); /** dummy params **/
       tmpBuf.Open(SM_FOPEN_WRITE);
       tmpChar[0] = (char)3;          // ASN.1 encoding, PrintableString.
       tmpChar[0] = (char)strlen(kw_value);; // ASN.1 Length < 128.
       tmpBuf.Write(tmpChar, 2);
       tmpBuf.Write(kw_value, strlen(kw_value));
       tmpBuf.Close();

       tmpBuf = CSM_Buffer(kw_value, strlen(kw_value));
       printf("cfg_cert: no dsaparams= keyword, defaulted=|%s|.\n",kw_value);
   }
#endif
   
   

   /* Signature Value */
   if (SM_GetKwValue("sig_r_value", kw_value, section) == 0 &&
       SM_GetKwValue("sig_s_value", kw_value2, section) == 0)
   {
       SM_Hex2Buffer (kw_value, &tmpBuf);
       SM_Hex2Buffer (kw_value2, &tmpBuf2);
       tmpBuf.Open(SM_FOPEN_APPEND);
       tmpBuf.Write((char *)tmpBuf2, tmpBuf2.Length());
       tmpBuf.Close();
   }
   else
   {
       strcpy(kw_value, "dummy_signature                         "); /** dummy params **/
       tmpBuf = CSM_Buffer(kw_value, strlen(kw_value));
       printf("cfg_cert: no sig_X_value= keyword, defaulted=|%s|.\n",kw_value);
   }
   cert->signatureValue.Set((char *)tmpBuf, tmpBuf.Length()*8);
   


   return(0);
}

//////////////////////////////////////////////////////////////////////////
// This function converts a string to a snacc DN structure
long str2dn(char *dn_str, Name  *&nameP)
{
   RelativeDistinguishedName/*RDNSequence4*/ *rdn;        
                                  /* ASN.c Relative DN pointer */
   AttributeTypeAndDistinguishedValue *pTmpRDN;
   PrintableString tmpPS;
   // TeletexString *att_teleP; // not used, see comment below
   PrintableString *att_valP;

   long dn_str_len = 0;
   char *workDn;         /* var used to work on the DN */
   char *dn_ptr;
   char *dn_field;
   char dn_label_data[10];
   char *dn_label;
   int  last_oid_component = 0;
   int  j = 0;
   int  test;

   if (nameP == NULL)
       nameP = new Name;
   /**** Choose the rDNSequence choice even though there is only ****/
   /**** one choice. ****/

   nameP->choiceId = Name::rDNSequenceCid;
   nameP->rDNSequence = new RDNSequence;


   dn_str_len = strlen(dn_str);
   workDn = (char *) calloc(1, (dn_str_len + 1));
   strcpy(workDn, dn_str); 

   dn_ptr = strtok(workDn,"@");
   while (dn_ptr != NULL)
   {
      dn_field = dn_ptr;
      dn_label = &dn_label_data[0];
      memset(dn_label, 0, 10);
      while (*dn_field != '=')
      {
         *dn_label++ =  *dn_field++;
      }
      dn_field++;  /* skip the '=' sign */

      rdn = nameP->rDNSequence->Append(); // Create present RDN grouping.
      pTmpRDN = rdn->Append();          // Create single RDN attribute component.
      str2dn_RDN_OID(pTmpRDN, dn_label_data);

      // commented out the code that produces nodes as Teletex at level 4
      // and below because it causes problems in CSM_DN::GetDNString
      // this needs to be resolved...TBD
      // for now, just encode each node as a printable string
      //if ( j == 4 )
      //{
      //   att_teleP = new TeletexString((const char *)dn_field, 
      //       strlen(dn_field));
      //   ENCODE_ANY(att_teleP, &pTmpRDN->value);
      //}
      //else
      //{
         att_valP = new PrintableString((const char *)dn_field, strlen(dn_field));
         ENCODE_ANY(att_valP, &pTmpRDN->value);
      //}


      j++;

      test = abs(workDn - dn_ptr);
      test += strlen(dn_ptr);
      if ( abs(workDn - dn_ptr) + strlen(dn_ptr) != (unsigned long)dn_str_len) 
         dn_ptr = strtok(dn_ptr + strlen(dn_ptr) + 1,"@");
      else
         dn_ptr = NULL;

    }  /* END FOR LOOP */

    free(workDn);
    return(0);
}

long str2dn_RDN_OID(AttributeTypeAndDistinguishedValue *pRDN, char *dn_label_data)
{
          if (strcmp(dn_label_data, "C") == 0)
          {
              pRDN->type = id_at_countryName;

          }
          else if (strcmp(dn_label_data, "CN") == 0)
          {
              pRDN->type = id_at_commonName;
          }
          else if (strcmp(dn_label_data, "SN") == 0)
          {
              pRDN->type = id_at_surname;
          }
          else if (strcmp(dn_label_data, "L") == 0)
          {
              pRDN->type = id_at_localityName;
          }
          else if (strcmp(dn_label_data, "O") == 0)
          {
              pRDN->type = id_at_organizationName;
          }
          else if (strcmp(dn_label_data, "OU") == 0)
          {
              pRDN->type = id_at_organizationalUnitName;
          }
          else
          {
              printf("Unsupported RDN character for DN translation: |%s|..\n", 
                  dn_label_data);
          }

    return(0);
}


#ifdef BOB
/*
 * FUNCTION: cfg4_crtExts()
 *
 * PURPOSE: To call the appropriate cfg4 function which will read the
 *           the keywords for the requested extensions (in the extensions
 *           integer array).  The section for the extensions are hard-coded
 *           within the function call.
 *
 * NOTE: The following numbers correspond to the specified extensions.
 * 
 *   Number  Extension           Section         Cfg. Function            
 *   ------------------------------------------------------------------------
 *   1       Auth. Key Id.       AuthKeyIdExt    cfg4_authKeyIdExt()
 *   2       Key Usage           KeyUsageExt     cfg4_keyUsageExt()
 *   3       Private Key Usage   PrvKeyUsageExt  cfg4_prvKeyUsageExt()
 *   4       Cert. Policy        CertPolicyExt   cfg4_certPolicyExt()
 *   5       Policy Mapping      PolicyMapExt    cfg4_policyMapExt()
 *   6       Subj. Alt. Name     SubjAltNmExt    cfg4_subjAltNmExt()
 *   7       Iss. Alt. Name      IssAltNmExt     cfg4_issAltNmExt()
 *   8       Subj. Dir. Att.     SubjDirAttExt   cfg4_subjDirAttExt()
 *   9       Name Constraints    NameConsExt     cfg4_nameConsExt()
 *   10      Policy Constraints  PolConsExt      cfg4_polConsExt()
 *   11      CRL Dist. Points    CrlDistPtsExt   cfg4_crlDistPtsExt()
 *   12      Subj. Key ID        SubjKeyIDExt    cfg4_subjKeyIdExt()
 *   13      Basic Constraints   BasicConsExt    cfg4_basicConsExt()
 *   14      Unknown Extension   NA              NA
 */
long cfg4_crtExts(Cert4 *cert, int *extensions)
{
   int   count = 0;
   int   stat = 0;
   long  error = 0;
   Exts *exts_list;

   while ( !error &&  extensions[count] > 0)
   {
      if (count == 0)
      {
    cert->exts = exts_list = calloc (1, sizeof (Exts));
    exts_list->next = NULL;
      }
      else
      {
         exts_list->next = calloc (1, sizeof (Exts));
    exts_list = exts_list->next;
    exts_list->next = NULL;
      }
      switch (extensions[count])
      {
       case 1:
     if (debug)
        printf("Filling the Authority Identifier Extension!!\n");
     error = cfg4_authKeyIdExt(&(exts_list->extn_val.auth_key_id), 
                &(exts_list->critical),
                "AuthKeyIdExt");
     exts_list->extn_id = AUTH_KEY_TYPE;
     break;

       case 2:
     /* Fill in the key usage extension */
     if (debug)
        printf("Filling the Key Usage Extension!!\n");
     error = cfg4_keyUsageExt( &(exts_list->extn_val.key_use),
                &(exts_list->critical),
                "KeyUsageExt");
     exts_list->extn_id = KEY_USAGE_TYPE;
     break;
 
       case 3:
     if (debug)
        printf("Filling the Private Key Usage extension!!\n"); 
          error = cfg4_privKeyUseExt ( &(exts_list->extn_val.priv_key_use),
                                       &(exts_list->critical),
                                       "PrvKeyUsageExt");
          exts_list->extn_id = PRIV_KEY_USE_TYPE;
     break;

       case 4:
     if (debug)
       printf("Filling the Certificate Policy Extension!!\n");
     error = cfg4_certPolicyExt( &(exts_list->extn_val.pol_infos),
                  &(exts_list->critical),
                  "CertPolicyExt");
     exts_list->extn_id = CERT_POL_TYPE;
     break;

       case 5:
     if (debug)
        printf("Filling the Policy Mapping extension!!\n"); 
          error = cfg4_policyMapExt( &(exts_list->extn_val.pol_maps_syn),
                                     &(exts_list->critical),
                                     "PolicyMapExt");
     exts_list->extn_id = POL_MAP_TYPE;
     break;

       case 6:
     if (debug)
        printf("Filling the Subject Alternate Names extension\n");
        error = cfg4_subjAltNmExt( &(exts_list->extn_val.subj_alt_name),
               &(exts_list->critical),
                 "SubjAltNmExt");
     exts_list->extn_id = SUBJ_ALT_NAME_TYPE;
     break;

       case 7:
     if (debug)
        printf("Filling the Issuer Alternate Name Extension!!\n");
     error = cfg4_issAltNmExt( &(exts_list->extn_val.issuer_alt_name),
             &(exts_list->critical),
             "IssAltNmExt");
     exts_list->extn_id = ISUR_ALT_NAME_TYPE;
     break;
       
       case 8:
     if (debug)
        printf("Filling the Subject Directory Attributes Extension!!\n");
          error = cfg4_subjDirAttExt( &(exts_list->extn_val.subj_dir_atts),
                                      &(exts_list->critical),
                                      "SubjDirAttExt");
     exts_list->extn_id = SUBJ_DIR_ATT_TYPE;
     break;

       case 9:
     if (debug)
        printf("Filling the Name Constraints Extension!! \n");

          error = cfg4_nameConsExt ( &(exts_list->extn_val.name_cons),
                                     &(exts_list->critical),
                                     "NameConsExt");
     exts_list->extn_id = NAME_CONST_TYPE;
     break;

       case 10:
     if (debug)
        printf("Filling the Policy Constrainst extension!!\n");
     
     error = cfg4_polConsExt ( &(exts_list->extn_val.pol_cons_syn),
                &(exts_list->critical),
                "PolConsExt");
     exts_list->extn_id = POL_CON_TYPE;
     break;

       case 11:
     /* Fill in the CRL Distribution Points */
     printf ("Encoding with the CRL Distrubution points extension\n") ;
     exts_list->critical = CRITICAL;
     exts_list->extn_id = CRL_DISTRO_TYPE;

     /* exts_list->extn_val.dist_pts = calloc (1, sizeof (Dist_pts));*/
    stat = (int)cfg4_crlDistPtsExt( &(exts_list->extn_val.dist_pts), 
                                         &(exts_list->critical),
                                         "CrlDistPtsExt");
    if (stat != NO_ERROR)
    {
       return (-1);
    }
    break;

      case 12:
    if (debug)
       printf("Filling Subject Key ID Extension!!\n");
    error = cfg4_subjKeyIdExt( &(exts_list->extn_val.subj_key_id),
                &(exts_list->critical),
                                    "SubjKeyIDExt");
    exts_list->extn_id = SUBJ_KEY_ID_TYPE;
    break;

      case 13:
    if (debug)
       printf("Filling the Basic Constraints Extension!!\n");
    error = cfg4_basicConsExt( &(exts_list->extn_val.basic_cons),
                                    &(exts_list->critical),
                                    "BasicConsExt");
    exts_list->extn_id = BASIC_CONST_TYPE;
    break;

       case 14:
    /* Process unknown extensions */
    exts_list->critical = NON_CRITICAL;
    exts_list->extn_id = UNKN_TYPE;
    exts_list->extn_val.unkn_extn = calloc (1, sizeof (Unkn_extn));
    fill4_unkn_extn (exts_list->extn_val.unkn_extn);
    break;
       
       default:
    printf ("Encoding with no extensions\n") ;
    break;
      } /* End Switch */

      count ++;
   } /* End While */
   return (error);
}


/*
 * FUNCTION: cfg4_dsaParams()
 * 
 * PURPOSE: to read the following keywords from the specified section of
 *          the current configuration file.
 *
 * KEYWORDS:     
 *    Name                   Value
 *    ---------------------------------------------------------------
 *    p                      string of hex data rep. DSA P parameter
 *    q                      string of hex data rep. DSA Q parameter
 *    g                      string of hex data rep. DSA G parameter
 */
long cfg_dsaParams(dsaParams, section)
   Params     *dsaParams;
   Section     section;
{
   KwValue     kw_value;
   Str_struct *p;
   long        error = 0;

   if (SM_GetKwValue("p", kw_value, section) == 0)
   {
      SM_Hex2Buffer (kw_value, &dsaParams->p);
   }
   else
      error = 1;

   if (SM_GetKwValue("q", kw_value, section) == 0)
   {
      p = &dsaParams->q;
      hex2strStruct(kw_value, &p);
   }
   else
      error = 1;

   if (SM_GetKwValue("g", kw_value, section) == 0)
   {
      p = &dsaParams->g;
      hex2strStruct(kw_value, &p);
   }
   else
      error = 1;

   return (error);
}
#endif

// EOF sm_CfgCert.cpp
