#ifndef NO_SCCS_ID
static char SccsId[ ] = "@(#) sm_CfgDriver.cpp 1.2 1/13/98 15:51:27"; 
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
//#include "c_sm_api.h"
#include "sm_util.h"

/*
 * FUNCTION: SM_CfgDriver()
 *
 * AUTHOR: Pierce Leonberger/VDA
 *
 * PURPOSE: To dynamically set local "char *" variables from a
 *          configuration file.  
 * 
 * FORMAT: the configuration file <filename> must have the following
 *         format:
 *                <keyword>=<value>
 * FORMAT RULES:
 * 1) no spaces between <keyword> and "=", or between "=" and <value>, or
 *    after <value>.
 * 2) any line that has a "#" as the first character will be considered
 *    a comment and ignored.
 * 3) it is assumed that the configuration file <filename> is in the
 *    current directory.  If you want to specify a path you must include
 *    it as part of <filename> when you call cfg_driver().
 *
 * NOTES:
 *     currently <destStr> must be allocated or declared prior to calling
 *     cfg_driver().
 */
 
int SM_CfgDriver (char *keyword, char *destStr, char *filename)
{
   FILE *fp;
   char *strPtr = NULL;
   char line[256];
   int  found = 0;

   /* check if keyword or destStr is null */
   if (! keyword || ! destStr)
   {
      printf("cfg_driver: check parameters passed to cfg_driver()!!\n");
      return (-1);
   }

   /* let's try and open the configuration file */
   if ( (fp = fopen(filename, "r")) == NULL)
   {
      printf("cfg_driver: unable to open file: %s\n", filename);
      return (-1);
   }

   while (! feof(fp) && ! found)
   {
      fgets(line, 256, fp);

      if (! feof(fp) && line[0] != '#' )
      {
         strPtr = strtok(line, "=");
         if (strPtr != NULL && strcmp(strPtr, keyword) == 0)
         {
            strPtr = strtok(NULL, "=");
            if (strPtr != NULL)
               found = 1;
         }
      }
   }
   fclose(fp);
   if (found)
   {
      strncpy(destStr, strPtr, strlen(strPtr) - 1);
      destStr[strlen(strPtr) - 1] = '\0';
   }
   return 0;
}

/*
 * FUNCTION: SM_SetCfgFn()
 * 
 * AUTHOR: Pierce Leonberger/VDA
 *
 * PURPOSE:
 *           If filename != NULL the cfg_file will be set the filename and
 *           a pointer to a character string containing the value of the 
 *           old config file.  
 *
 *           If the filename == NULL then the func. will return the 
 *           a pointer to a character string containg the value of the current
 *           config file.  It is the responsibility of the calling function
 *           to free this area of memory.
 *
 *           If the user passes in a filename that can not be opened for
 *           reading the cfg_file_set flag will be reset to 0, the file-
 *           name will not change, and a NULL character pointer will be
 *           returned to indicate an error.  The following illustrates this
 *           condition -- which means that the specified filename could not be 
 *           opened for reading.
 *
 *           if ( SetCfgFn("test.cfg") == NULL)
 *              printf("error opening test.cfg file\n");
 */
char * SM_SetCfgFn(char *filename)
{
   FILE        *fp;
   static      cfg_file_set = 0;
   static char cfg_file[256]; /* current config file */
   static char oldCfgFn[256]; /* old configuration file */
   char        szCopyFn[256];
  
   if ((filename != NULL) && /* if a new config. file is given */
       (filename[0] != '\0'))
   {
      if ( (fp = fopen(filename, "r")) != NULL)   /* check if its readable */
      {
         cfg_file_set = 1;
         /* at this point, filename can potentially be pointing at oldCfgFn
            so we make a copy of filename into szCopyFn before overwriting
            oldCfgFn (which might affect filename) */
         strcpy(szCopyFn, filename);
         strcpy(oldCfgFn, cfg_file);
         strcpy(cfg_file, szCopyFn);
         fclose(fp);
      }
      else
         cfg_file_set = 0;
   }

   /*
    * Return current config file if filename is NULL.  Return old config
    * file if filename was not null and readable.  Else there was
    * an error so return NULL.
    */
   if (filename == NULL)
      return strdup(cfg_file);
   else if (cfg_file_set)   
      return oldCfgFn; 
   else
      return (char *) NULL;
}

/*
 * FUNCTION: SM_GetDfKwValue()
 *
 * AUTHOR: Pierce Leonberger/VDA
 *
 * PURPOSE: same as GetKwValue() except you can supply a default value
 *          to use if the Keyword is not found.  Also, if the user passes
 *          defStr in as NULL this function will attempt to open 
 *          DEFAULT_CFG_FILE and read the keywords from there.  The keywords
 *          and section name must be indentical.
 * 
 * RETURN VALUES:
 *                error return codes 
 *                ------------------
 *                -2 section not found and defualt value not used 
 *                -1 parameters missing.
 *
 *                succesfull return codes
 *                -----------------------
 *                 0 success.
 *                 1 keyword was not found in original or default config.
 */
long SM_GetDfKwValue ( Keyword keyword, KwValue destStr, Section section)
{
   long error = 1;
   char *orgCfgFn = NULL;

   /* try to read keyword value from default config file first.
    */
   if ( (orgCfgFn = SM_SetCfgFn(DEFAULT_CFG_FILE)) != NULL)
   {
      error = SM_GetKwValue(keyword, destStr, NULL);
      SM_SetCfgFn(orgCfgFn);
   }

   /* if the keyword value was not found in the default config file
    * try to read it from the current config file.
    */
   if (error == 1)
   {
      error = SM_GetKwValue(keyword, destStr, section);
   }

   return (error);
}

/* FUNCTION: SM_GetDfCertIndex()
   PURPOSE:  Pass a section and entry to this function to obtain the
             certificate index for a card.  This function first checks
             to see if the keyword that it obtains from the section/entry
             is numeric.  If so, it returns the numeric value.  If not,
             then it attempts to read that keyword from the default.cfg
             and returns the default numeric value.  This allows the default
             config file to specificy the location of the DefaultDSAOrg,
             DefaultKEAOrg, DefaultDSAInd, DefaultKEAInd, and 
             DefaultROKEAOrg certificates */
/* note: this function returns -1 for failure or the cert index */
#define NATOL(value) (((atol(value) == 0) && (value[0] != '0')) ? -1 : atol(value))
long SM_GetDfCertIndex ( Keyword keyword, Section section)
{
   KwValue kw_value;
   KwValue kw_default;
   long lRet = -1; /* init value is error */
   long lCertIndex = -1;

   if (SM_GetKwValue(keyword, kw_value, section) == SM_NO_ERROR)
      if ((lCertIndex = NATOL(kw_value)) == -1)
         if (SM_GetDfKwValue(kw_value, kw_default, NULL) == SM_NO_ERROR)
            lCertIndex = NATOL(kw_default);
   if (lCertIndex != -1)
      lRet = lCertIndex;
   return lRet;
}

/*
 * FUNCTION: SM_GetKwValue()
 *
 * AUTHOR: Pierce Leonberger/VDA
 * 
 * PURPOSE: get the value of the keyword from the configuration file (set
 *          priviously by the SetCfgFn() function) in the specified
 *          section.  If the section is not provided (a NULL is passed in
 *          it's place) it will look for keywords only and will abort it's
 *          search when the keyword is found or a section is encountered
 *          in the configuration file.
 *
 * FORMAT RULES:
 * 1) no spaces between <keyword> and "=", or between "=" and <value>, or
 *    after <value>.  Any white space or comments entered after <value> will 
 *    be stripped.
 *    
 * 2) any line that has a "#" or ";" as the first character will be considered
 *    a comment and ignored.  
 * 
 * 3) Comments can be added on the same line as the keyword value by placing
 *    one of the comment indicators (';' or '#') and entering the comments
 *    to the right of the indicator.
 *
 * NOTE:
 *    It is assumed that the configuration file <filename> is in the
 *    current directory.  If you want to specify a path you must include
 *    it as part of <filename> when you call SetCfgFn().
 *
 * RETURN VALUES: 
 *                error condition return codes
 *                ----------------------------
 *                -2 section was specified and not found
 *                -1 parameters missing
 *  
 *                successful return codes
 *                -----------------------
 *                 0 success
 *                 1 keyword not found
 */
long SM_GetKwValue (Keyword keyword, KwValue destStr, Section section)
{
    
   Keyword tmp_kw;
   char *filename = SM_SetCfgFn(NULL);
   FILE *fp;
   char *sect_with_brackets = NULL;
   char *strPtr = NULL;
   char *p;                    /* used for stripping comments from a line */
   char line[CFG_MAX_LINE_LEN];
   int  found = 0;
   int  error = 0; /* default error code to successful */

   if (! filename )
   {
      printf("SM_GetKwValue: configuration file is not set!!\n");
      free(filename);
      return(-1);
   }

   /* check if keyword or destStr is null */
   if (! keyword || ! destStr )
   {
      printf("SM_GetKwValue: keyword or destination string not present!!\n");
      free(filename);
      return (-1);
   }

   /* let's try and open the configuration file */
   if ( (fp = fopen(filename, "r")) == NULL)
   {
      printf("SM_GetKwValue(): unable to open file: %s\n", filename);
      return (-1);
   }
 
   /* if requested find the appropriate section first */
   if (section != NULL)
   {
      sect_with_brackets = (char *) calloc(1, strlen(section) + 3);
      sprintf(sect_with_brackets,"[%s]", section);

      found = 0;
      while (! feof(fp) && ! found)
      {
         fgets(line, CFG_MAX_LINE_LEN, fp);
         if (! feof(fp) && line[0] == '[')
         {
            if (strstr(line, sect_with_brackets) != NULL)
               found = 1;
         }
      }
      if (!found)
         error = -2;       /* section not found error */
   }

   /* If the section was not specified or it was specified and found check
    * for the keyword.
    */
   if (section == NULL || found)
   {
      found = 0;
      sprintf(tmp_kw,"%s=", keyword);
      while (! feof(fp) && ! found)
      {
    fgets(line, CFG_MAX_LINE_LEN, fp);

    if (!feof(fp) && line[0] != '#' && line[0] != ';' && line[0] != '[')
    {
            if ((strPtr = strstr(line, tmp_kw)) == line)
               found = 1;
    }
         else if (line[0] == '[') /* stop when next section has been reached */
            break;
      }
      if (found)
      {
         strPtr = strchr(strPtr, '=') + 1;

         /* If there is not a comment on the then starting stripping
          * whitespace from the end of the keyword value.  If a comment is
          * present then the strchr() will return a non-null pointer and
          * the white and comments will be stripping of starting at the
          * beginning of the comment.
          */
         if ((p  = strchr(strPtr, '#')) == NULL &&
             (p  = strchr(strPtr, ';')) == NULL)
            p = strPtr + (strlen(strPtr) - 1);   /* minus 1 cuz of newline */

         /* strip whitespace and any comments that may exist */
         while ( *p == ' ' || *p == '\t' || *p == '\n' || *p == '#' || 
                 *p == ';') 
               *p-- = '\0';

    strncpy(destStr, strPtr, strlen(strPtr));
    destStr[strlen(strPtr)] = '\0';
      }
      else
      {
    error = 1; /* keyword not found error */
      }
   }
   fclose(fp);
    
   /* clean up */
   if (sect_with_brackets != NULL)
      free(sect_with_brackets);

   free(filename);

   return error;
}

/* FUNCTION: SM_GetEntireSection()
   PURPOSE:  Pass a section to this function and it returns the
             entire section in the pszAllEntries parameter.  The
             lines are tokenized with the '|' character.  Memory
             for the pszAllEntries parameter is allocated here and
             must be freed by the caller. */
/* note: this function returns -1 for failure or, upon success,
   it returns the number of entries read in the section */
long SM_GetEntireSection(char *pszSection, char **pszAllEntries)
{
   long lRet = -1;
   char *pszFileName = NULL; 
   FILE *fp = NULL;
   char szBuffer[256];
   char szSearchFor[256];
   char *pszReplace = NULL;
   char *pszInsert = NULL;

   if ((pszSection == NULL) || (pszAllEntries == NULL))
      return (lRet);
   sprintf(szSearchFor, "[%s]", pszSection);

   /* this double pointer should NOT have memory associated
      with it upon entry to this function */
   *pszAllEntries = NULL;

   /* find out what the current config file name is */
   if ((pszFileName = SM_SetCfgFn(NULL)) != NULL)
   {
      /* open the current config file name */
      if ((fp = fopen(pszFileName, SM_FOPEN_READ)) != NULL)
      {
         /* scan for the section */
         while (fgets(szBuffer, sizeof(szBuffer), fp) != NULL)
         {
            if (strstr(&szBuffer[0], &szSearchFor[0]) == &szBuffer[0])
            {
               /* found it, start reading & allocating 
                  until next section found */
               while ((fgets(szBuffer, sizeof(szBuffer), fp) != NULL) &&
                     (szBuffer[0] != '['))
               {
                  /* if its important, use it... (i.e. not NULL char, not
                     new line, not return, not space, not tab, not comment */
                  if (strpbrk(szBuffer, "\n\r \t#;") != &szBuffer[0])
                  {
                     if (*pszAllEntries == NULL)
                     {
                        *pszAllEntries = strdup(szBuffer);
                        lRet = 1;
                     }
                     else
                     {
                        pszReplace = *pszAllEntries; /* save current */
                        /* get enough memory for new stuff and old stuff */
                        *pszAllEntries = (char *)calloc(1, 
                              strlen(pszReplace) + strlen(szBuffer) + 2);
                        /* put the old and new stuff, tokenized, into 
                           the new memory */
                        sprintf(*pszAllEntries, "%s|%s", pszReplace,
                              szBuffer);
                        /* free old memory */
                        free(pszReplace);
                        ++lRet;
                     }
                  }
               } /* end of while scan for section entries */
               break;
            }
         } /* end of while scan for section */
         fclose(fp);
      }
      else
         printf("ERR SM_GetEntireSection couldn't open cfg %s\n",
               pszFileName);
      free(pszFileName);
   }
   else
      printf("ERR SM_GetEntireSection no current cfg fn\n");

   return lRet;
}
