#ifndef NO_SCCS_ID
static char SccsId[ ] = "@(#) sm_CfgEnvelopedData.cpp 1.8 3/27/98 18:36:11"; 
#endif

//////////////////////////////////////////////////////////////////////////
// sm_CfgEnvelopedData.cpp

#include <stdio.h>
#include "sm_util.h"

//////////////////////////////////////////////////////////////////////////
SM_RET_VAL SM_CfgFillEncryptMsg(CSMIME *pCSMIME, CSM_EncryptMsg *pEncryptMsg,
                               Section section)
{
   SM_RET_VAL lRet = SM_NO_ERROR;
   char szIndexedKW[80];
   KwValue kwValue, kwContentFN, kwContentOID, kwContentEncryptionOID;
   CSM_CSInstLst *ptmpInstLst;
   int n = 1;
   char szErrorMsg[256];
   CSM_DN *ptmpDN;
   CSM_Buffer *pbufContent, *pbufCert;
   char *pszTmpDNString;

   SME_SETUP("SM_CfgFillEncryptMsg");

   if ((pCSMIME == NULL) || (pCSMIME->m_pCSInsts == NULL)
         || (pEncryptMsg == NULL))
      SME_THROW(SM_MISSING_PARAM, NULL, NULL);

   // find all the instances indicated by DN's in the LoginN entries
   // and mark them UseThis
   pCSMIME->ClearFlag(SM_INST_ALL);
   sprintf(szIndexedKW, "Login%d", n++);
   while (true)
   {
      if (SM_GetKwValue(szIndexedKW, kwValue, section) != SM_NO_ERROR)
         break;
      for (ptmpInstLst = pCSMIME->m_pCSInsts->First(); ptmpInstLst;
            ptmpInstLst = ptmpInstLst->Next())
      {
         if (strcmp(kwValue, ptmpInstLst->Get()->GetID()) == 0)
         {              // CSInstance ID was simply ID name.
            break; 
         }
         else           // TRY DN as CSInstance ID
         {
           ptmpDN = ptmpInstLst->Get()->GetSubjectDN();
           if ((pszTmpDNString = ptmpDN->GetDNString()) == NULL)
            SME_THROW(SM_UNKNOWN_ERROR, "couldn't get dn string", NULL);
           if (strcmp(kwValue, pszTmpDNString) == 0)
           {
              free(pszTmpDNString);
              break;
           }
           free(pszTmpDNString);
         }
      }
      if (ptmpInstLst)
         ptmpInstLst->Get()->SetUseThis();
      sprintf(szIndexedKW, "Login%d", n++);
   }

   // read the content
   if ((lRet = SM_GetKwValue("ContentFN", kwContentFN, section)) 
         != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "ContentFN keyword missing", NULL);
   pbufContent = new CSM_Buffer(kwContentFN);

   // convert to memory if requested
   if ((lRet = SM_GetKwValue("ContentInBuffer", kwValue, section))
         != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "ContentInBuffer keyword missing", NULL);
   if (kwValue[0] == '1')
      pbufContent->ConvertFileToMemory();
   // set the encapsulated content
   SME(pEncryptMsg->SetEncapContent(pbufContent, 
         CSM_Content::CONTENT_DATA_DATA));
   delete(pbufContent);

   // get OIDs
   if ((lRet = SM_GetKwValue("ContentOID", kwContentOID, section))
         != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "ContentOID keyword missing", NULL);
   if ((lRet = SM_GetKwValue("ContentEncryptionOID", 
         kwContentEncryptionOID, section)) != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "ContentEncryptionOID keyword missing", 
            NULL);

   // set OIDs in EncryptMsg
   CSM_OID oidContent(kwContentOID);
   SME(pEncryptMsg->setContentType(oidContent));
   CSM_OID *poidContentEncryption = new CSM_OID(kwContentEncryptionOID);
   pEncryptMsg->SetContentEncryptOID(poidContentEncryption);

   // get and set include orig certs flag
   if ((lRet = SM_GetKwValue("IncludeOrigCertsFlag", kwValue, section))
         != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "IncludeOrigCertsFlag keyword missing", 
            NULL);
   if (kwValue[0] == '1')
      pEncryptMsg->SetIncludeOrigCertsFlag(true);
   // default is false

   // currently not including any other certs in m_pMsgCrtCrls

   // setup recipients
   // get the location of the address book
   if ((lRet = SM_GetKwValue("RecipientAddressBook", kwValue, section))
         != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "RecipientAddressBook keyword missing", 
            NULL);
   MAB_AB_def mab(kwValue); // create and initialize address book
   // go through all recipients
   n = 1;
   while (true)
   {
      sprintf(szIndexedKW, "Recipient%d", n++);
      // get recipient
      if (SM_GetKwValue(szIndexedKW, kwValue, section) != SM_NO_ERROR)
         break;
      // if path/file name, read directly from there
      if (strchr(kwValue, '/') != NULL)
      {
         pbufCert = new CSM_Buffer(kwValue);
         pbufCert->ConvertFileToMemory();
      }
      else // else, treat as alias
      {
         pbufCert = new CSM_Buffer;
         if ((lRet = mab.FindCertAlias(kwValue, pbufCert)) != SM_NO_ERROR)
         {
            sprintf(szErrorMsg, "couldn't find cert alias %s", kwValue);
            SME_THROW(SM_UNKNOWN_ERROR, szErrorMsg, NULL);
         }
      }
      // if recipients list already exists, append new recipient based
      // on cert buffer
      if (pEncryptMsg->m_pRecipients)
         pEncryptMsg->m_pRecipients->Append(new CSM_Recipient(pbufCert));
      else // else, create the recipient list with cert buffer recipient
         pEncryptMsg->m_pRecipients = new 
               CSM_RecipientLst(new CSM_Recipient(pbufCert));
      delete (pbufCert);
   }

   SME_FINISH
   SME_CATCH_SETUP
      // catch/cleanup logic as necessary
   SME_CATCH_FINISH

   return lRet;
}

//////////////////////////////////////////////////////////////////////////
SM_RET_VAL SM_CfgFillDecryptMsg(CSMIME *pCSMIME, CSM_DecryptMsg *pDecryptMsg,
                               Section section)
{
   SM_RET_VAL lRet = SM_UNKNOWN_ERROR;
   KwValue kwValue, kwEncodedFN;
   CSM_Buffer *pbufBlob;

   SME_SETUP("SM_CfgFillDecryptMsg");

   if ((pCSMIME == NULL) || (pCSMIME->m_pCSInsts == NULL)
         || (pDecryptMsg == NULL))
      SME_THROW(SM_MISSING_PARAM, NULL, NULL);

   // set the encoded blob
   if ((lRet = SM_GetKwValue("EncodedFN", kwEncodedFN, section))
         != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "EncodedFN kw missing", NULL);
   pbufBlob = new CSM_Buffer(kwEncodedFN);

   // convert to memory if requested
   if ((lRet = SM_GetKwValue("EncodedFileInBuffer", kwValue, section))
            != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "EncodedFileInBuffer keyword missing", 
            NULL);
   if (kwValue[0] == '1')
      pbufBlob->ConvertFileToMemory();

   // Pre Process the encoded blob
   SME(pDecryptMsg->PreProc(pbufBlob, pCSMIME));

   SME_FINISH
   SME_CATCH_SETUP
      // catch/cleanup logic as necessary
   SME_CATCH_FINISH

   return lRet;
}

// EOF sm_CfgEnvelopedData.cpp

