#ifndef NO_SCCS_ID
static char SccsId[ ] = "@(#) sm_CfgFree.cpp 1.13 3/13/98 18:00:42"; 
#endif

// sm_CfgFree.cpp
#include "sm_util.h"

#ifdef SM_FREE_USED
#include "sm_free.h"
#include "sm_free_asn.h"

SM_RET_VAL SM_FreeEncryptTest(char *pszConfigFile, char *pszConfigSection,
                              long output_flag, long verify_flag)
{
   KwValue kwValue;
   SM_RET_VAL lRet = SM_NO_ERROR;
   char *pszOrigCfgFile = SM_SetCfgFn(pszConfigFile); /* set config file */
   bool bInFile = true;
   KwValue kwPlainDataFile, kwEncryptedDataFile, kwParametersFile, kwMEKFile;
   CSM_Free freeCTI;
   CSM_Buffer bufferData, bufferEncryptedData, bufferParameters, bufferMEK;

   SME_SETUP("SM_FreeEncryptTest");

   if (output_flag == 1)
      printf("SM_FreeEncryptTest: reading config entries\n");
   if ((lRet = SM_GetKwValue("EncryptInBuffer", kwValue, 
         pszConfigSection)) != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "EncryptInBuffer keyword missing", NULL);
   if (kwValue[0] == '1')
      bInFile = false;

   if ((lRet = SM_GetKwValue("PlainData", kwPlainDataFile, 
         pszConfigSection)) != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "PlainData keyword missing", NULL);

   if ((lRet = SM_GetKwValue("EncryptedData", kwEncryptedDataFile, 
                  pszConfigSection)) != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "EncryptedData keyword missing", NULL);

   if ((lRet = SM_GetKwValue("ParametersFile", kwParametersFile, 
                  pszConfigSection)) != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "ParametersFile keyword missing", NULL);

   if ((lRet = SM_GetKwValue("MEKFile", kwMEKFile,
                     pszConfigSection)) != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "MEKFile keyword missing", NULL);

   // now we're ready for the actual test
   // setup data to file
   bufferData.SetFileName(kwPlainDataFile);
   if (bInFile)
      // setup encrypted data to file
      bufferEncryptedData.SetFileName(kwEncryptedDataFile);
   else
      // if data should be in memory, copy it in from file
      bufferData.ConvertFileToMemory();

   if (output_flag == 1)
      printf("SM_FreeEncryptTest:  calling SMTI_Encrypt\n");
   SME(lRet = freeCTI.
         SMTI_Encrypt(&bufferData, &bufferEncryptedData, 
               &bufferParameters, &bufferMEK));

   if (output_flag > -1)
      printf("SM_FreeEncryptTest success!\n");
   if (!bInFile)
      bufferEncryptedData.
            ConvertMemoryToFile(kwEncryptedDataFile);
   bufferParameters.ConvertMemoryToFile(kwParametersFile);
   // write MEK to file
   bufferMEK.ConvertMemoryToFile(kwMEKFile);

   SME_FINISH_CATCH

   // TBD?  What do we need to cleanup?
   SM_SetCfgFn(pszOrigCfgFile);
   return lRet;
}

SM_RET_VAL SM_FreeDecryptTest(char *pszConfigFile, char *pszConfigSection,
                              long output_flag, long verify_flag)
{
   KwValue kwValue;
   SM_RET_VAL lRet = SM_NO_ERROR;
   char *pszOrigCfgFile = SM_SetCfgFn(pszConfigFile); /* set config file */
   bool bInFile = true;
   KwValue kwPlainDataFile, kwEncryptedDataFile, kwParametersFile, 
         kwOriginalPlainData, kwMEKFile;
   CSM_Free freeCTI;
   CSM_Buffer bufferData, bufferEncryptedData, bufferParameters, bufferMEK;

   SME_SETUP("SM_FreeDecryptTest");

   if (output_flag == 1)
      printf("SM_FreeDecryptTest: reading config entries\n");
   if ((lRet = SM_GetKwValue("DecryptInBuffer", kwValue, 
         pszConfigSection)) != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "DecryptInBuffer keyword missing", NULL);
   if (kwValue[0] == '1')
      bInFile = false;

   if ((lRet = SM_GetKwValue("PlainData", kwPlainDataFile, 
         pszConfigSection)) != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "PlainData keyword missing", NULL);

   if ((lRet = SM_GetKwValue("EncryptedData", kwEncryptedDataFile, 
         pszConfigSection)) != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "EncryptedData keyword missing", NULL);

   if ((lRet = SM_GetKwValue("ParametersFile", kwParametersFile, 
         pszConfigSection)) != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "ParametersFile keyword missing", NULL);
   bufferParameters.SetFileName(kwParametersFile);

   if ((lRet = SM_GetKwValue("MEKFile", kwMEKFile, 
         pszConfigSection)) != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "MEKFile keyword missing", NULL);

   if (SM_GetKwValue("OriginalPlainData", kwOriginalPlainData, 
            pszConfigSection) != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "OriginalPlainData keyword missing", NULL);

   // now we're ready for the actual test
   // load mek
   bufferMEK.SetFileName(kwMEKFile);
   bufferMEK.ConvertFileToMemory();
   // setup data to file
   bufferEncryptedData.SetFileName(kwEncryptedDataFile);
   if (bInFile)
      // setup plain data to file
      bufferData.SetFileName(kwPlainDataFile);
   else
      // if data should be in memory, copy it in from file
      bufferEncryptedData.ConvertFileToMemory();

   if (output_flag == 1)
      printf("SM_FreeDecryptTest:  calling SMTI_Decrypt\n");
   SME(lRet = freeCTI.
         SMTI_Decrypt(&bufferParameters, &bufferEncryptedData, &bufferMEK,
               &bufferData));
   if (output_flag > -1)
      printf("SM_FreeDecryptTest success!\n");
   if (!bInFile)
      bufferData.ConvertMemoryToFile(kwPlainDataFile);
   if (verify_flag)
      printf("WARNING:  Verify not implemented yet\n");

   SME_FINISH_CATCH

   // TBD?  What do we need to cleanup?
   SM_SetCfgFn(pszOrigCfgFile);
   return lRet;
}

SM_RET_VAL SM_FreeGenerateEMEKTest(char *pszConfigFile, char *pszConfigSection,
                              long output_flag, long verify_flag)
{
   KwValue kwValue;
   SM_RET_VAL lRet = SM_NO_ERROR;
   char *pszOrigCfgFile = SM_SetCfgFn(pszConfigFile); /* set config file */
   CSM_Buffer bufferY, bufferEMEK, bufferMEK, bufferParams, bufferUKM;
   CSM_Free freeCTI;

   SME_SETUP("SM_FreeGenerateEMEKTest");

   if (output_flag == 1)
      printf("SM_FreeGenerateEMEKTest: reading config entries\n");
   if ((lRet = SM_GetKwValue("EMEKFile", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "EMEKFile keyword missing", NULL);
   bufferEMEK.SetFileName(kwValue);

   // get and load MEK
   if ((lRet = SM_GetKwValue("MEKFile", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "MEKFile keyword missing", NULL);
   bufferMEK.SetFileName(kwValue);
   bufferMEK.ConvertFileToMemory();

   // get and load recipient's Y
   if ((lRet = SM_GetKwValue("YB", kwValue, pszConfigSection)) 
         != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "YB keyword missing", NULL);
   SM_Hex2Buffer(kwValue, &bufferY);

   // get and load param P
   if ((lRet = SM_GetKwValue("ParamP", kwValue, pszConfigSection)) 
         != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "ParamP keyword missing", NULL);
   SM_Hex2Buffer(kwValue, &freeCTI.m_ParamP);

   // get and load param G
   if ((lRet = SM_GetKwValue("ParamG", kwValue, 
         pszConfigSection)) != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "ParamG keyword missing", NULL);
   SM_Hex2Buffer(kwValue, &freeCTI.m_ParamG);

   // get and load X
   if ((lRet = SM_GetKwValue("XA", kwValue,
         pszConfigSection)) != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "XA keyword missing", NULL);
   SM_Hex2Buffer(kwValue, &freeCTI.m_X);

   if ((lRet = SM_GetKwValue("ParametersFile", kwValue,
         pszConfigSection)) != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "ParametersFile keyword missing", NULL);
   bufferParams.SetFileName(kwValue);

   if ((lRet = SM_GetKwValue("UKMFile", kwValue,
         pszConfigSection)) != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "UKMFile keyword missing", NULL);
   bufferUKM.SetFileName(kwValue);

   if (output_flag == 1)
      printf("SM_FreeGenerateEMEKTest: calling SMTI_GenerateEMEK\n");
   SME(lRet = freeCTI.SMTI_GenerateEMEK(&bufferY, &bufferParams, 
         &bufferMEK, &bufferEMEK, &bufferUKM));
   if (output_flag > -1)
      printf("SM_FreeGenerateEMEKTest success!\n");
   // TBD, do emek in buffer?

   SME_FINISH_CATCH

   // TBD?  What do we need to cleanup?
   SM_SetCfgFn(pszOrigCfgFile);
   return lRet;
}

SM_RET_VAL SM_FreeRandomTest(char *pszConfigFile, char *pszConfigSection,
                              long output_flag, long verify_flag)
{
   KwValue kwValue;
   SM_RET_VAL lRet = SM_NO_ERROR;
   char *pszOrigCfgFile = SM_SetCfgFn(pszConfigFile); /* set config file */
   bool bInFile = true;
   char szResultFile[SM_PATH_LEN];
   SM_SIZE_T lLength;

   SME_SETUP("SM_FreeRandomTest");

   if ((lRet = SM_GetKwValue("TestInBuffer", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "TestInBuffer kw missing", NULL);

   if (kwValue[0] == '1')
      bInFile = false;
   if ((lRet = SM_GetKwValue("ResultFile", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "ResultInFile kw missing", NULL);
   strcpy(szResultFile, kwValue);
   if ((lRet = SM_GetKwValue("Length", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "Length kw missing", NULL);
   lLength = atol(kwValue);
   // now we're ready for the actual test
   CSM_Free freeCTI;
   CSM_Buffer buffer;
   if (bInFile)
      buffer.SetFileName(szResultFile);
   SME(lRet = freeCTI.
         SMTI_Random(NULL, &buffer, lLength));
   if (!bInFile)
      buffer.ConvertMemoryToFile(szResultFile);

   SME_FINISH_CATCH

   // TBD?  What do we need to cleanup?
   SM_SetCfgFn(pszOrigCfgFile);
   return lRet;
}
            
SM_RET_VAL SM_FreeDigestTest(char *pszConfigFile, char *pszConfigSection,
                              long output_flag, long verify_flag)
{
   KwValue kwValue;
   SM_RET_VAL lRet = SM_NO_ERROR;
   char *pszOrigCfgFile = SM_SetCfgFn(pszConfigFile); /* set config file */
   bool bInFile = true;
   char szDigestFile[SM_PATH_LEN];
   CSM_Buffer bufferData;
   CSM_Buffer bufferDigest;
   CSM_OID oidMd5(id_md5);

   SME_SETUP("SM_FreeDigestTest");

   if ((lRet = SM_GetKwValue("TestInBuffer", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "TestInBuffer kw missing", NULL);
   if (kwValue[0] == '1')
      bInFile = false;
   if ((lRet = SM_GetKwValue("Data", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "Data kw missing", NULL);
   bufferData.SetFileName(kwValue);
   if (!bInFile)
      bufferData.ConvertFileToMemory();
   if ((lRet = SM_GetKwValue("Digest", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "Digest kw missing", NULL);
   strcpy(szDigestFile, kwValue);
   if (bInFile)
      bufferDigest.SetFileName(szDigestFile);
   if ((lRet = SM_GetKwValue("SHA1", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "SHA1 kw missing", NULL);
   CSM_Free freeCTI;
   if (kwValue[0] != '1')
      freeCTI.BTISetPreferredCSInstAlgs(&oidMd5, NULL, NULL, NULL);
   // now we're ready for the actual test
   SME(lRet = freeCTI.
         SMTI_DigestData(&bufferData, &bufferDigest));
   if (!bInFile)
      bufferDigest.ConvertMemoryToFile(szDigestFile);

   SME_FINISH_CATCH

   // TBD?  What do we need to cleanup?
   SM_SetCfgFn(pszOrigCfgFile);
   return lRet;
}
            
SM_RET_VAL SM_FreeSignTest(char *pszConfigFile, char *pszConfigSection,
                              long output_flag, long verify_flag)
{
   KwValue kwValue;
   SM_RET_VAL lRet = SM_NO_ERROR;
   char *pszOrigCfgFile = SM_SetCfgFn(pszConfigFile); /* set config file */
   bool bOutputInFile = true;
   char szDigestFile[SM_PATH_LEN];
   char szSignatureFile[SM_PATH_LEN];
   CSM_Buffer bufferData, bufferSignature, bufferDigest, bufferTemp;
   byte *pbyte;

   SME_SETUP("SM_FreeSignTest");

   CSM_Free freeCTI;

   if ((lRet = SM_GetKwValue("P", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "P kw missing", NULL);
   bufferTemp.SetFileName(kwValue);
   pbyte = (byte *)(bufferTemp.AccessAll());
   freeCTI.m_DSAP.BERDecode(pbyte);

   if ((lRet = SM_GetKwValue("Q", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "Q kw missing", NULL);
   bufferTemp.SetFileName(kwValue);
   pbyte = (byte *)(bufferTemp.AccessAll());
   freeCTI.m_DSAQ.BERDecode(pbyte);

   if ((lRet = SM_GetKwValue("G", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "G kw missing", NULL);
   bufferTemp.SetFileName(kwValue);
   pbyte = (byte *)(bufferTemp.AccessAll());
   freeCTI.m_DSAG.BERDecode(pbyte);

   if ((lRet = SM_GetKwValue("Y", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "Y kw missing", NULL);
   bufferTemp.SetFileName(kwValue);
   pbyte = (byte *)(bufferTemp.AccessAll());
   freeCTI.m_DSAY.BERDecode(pbyte);

   if ((lRet = SM_GetKwValue("X", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "X kw missing", NULL);
   bufferTemp.SetFileName(kwValue);
   pbyte = (byte *)(bufferTemp.AccessAll());
   freeCTI.m_DSAX.BERDecode(pbyte);

   if ((lRet = SM_GetKwValue("Data", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "Data kw missing", NULL);
   bufferData.SetFileName(kwValue);
   
   if ((lRet = SM_GetKwValue("DataInBuffer", kwValue, pszConfigSection)) 
         != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "Data kw missing", NULL);
   if (kwValue[0] == '1')
      bufferData.ConvertFileToMemory();

   if ((lRet = SM_GetKwValue("OutputInBuffer", kwValue, pszConfigSection))
         != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "OutputInBuffer kw missing", NULL);
   if (kwValue[0] == '1')
      bOutputInFile = false;

   if ((lRet = SM_GetKwValue("Signature", kwValue, pszConfigSection))
         != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "Signature kw missing", NULL);
   strcpy(szSignatureFile, kwValue);
   
   if ((lRet = SM_GetKwValue("Digest", kwValue, pszConfigSection))
         != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "Digest kw missing", NULL);
   strcpy(szDigestFile, kwValue);

   if (bOutputInFile)
   {
      bufferSignature.SetFileName(szSignatureFile);
      bufferDigest.SetFileName(szDigestFile);
   }
   
   // now we're ready for the actual test
   SME(lRet = freeCTI.SMTI_Sign(&bufferData, &bufferSignature,
            &bufferDigest));
   if (!bOutputInFile)
   {
      bufferSignature.ConvertMemoryToFile(szSignatureFile);
      bufferDigest.ConvertMemoryToFile(szDigestFile);
   }
   if (output_flag >= 0)
      printf("SM_FreeSignTest: signature generated!\n");

   SME_FINISH_CATCH

   // TBD?  What do we need to cleanup?
   SM_SetCfgFn(pszOrigCfgFile);
   return lRet;
}

SM_RET_VAL SM_FreeExtractMEKTest(char *pszConfigFile, char *pszConfigSection,
                              long output_flag, long verify_flag)
{
   KwValue kwValue;
   SM_RET_VAL lRet = SM_NO_ERROR;
   char *pszOrigCfgFile = SM_SetCfgFn(pszConfigFile); /* set config file */
   bool bInputsInFile = true;
   CSM_Buffer bufferOriginator, bufferEMEK, bufferUKM, bufferParameters, 
            bufferMEK;
   KwValue kwMEKFile;
   bool bMEKInFile = true;

   SME_SETUP("SM_FreeExtractMEKTest");

   CSM_Free freeCTI;

   if ((lRet = SM_GetKwValue("InputsInFile", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "InputsInFile kw missing", NULL);
   if (kwValue[0] == '0')
      bInputsInFile = false;

   // get originator's Y and store it in memory buffer
   if ((lRet = SM_GetKwValue("YA", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "YA kw missing", NULL);
   SM_Hex2Buffer(kwValue, &bufferOriginator);

   // get recipient's X and store it in freeCTI
   if ((lRet = SM_GetKwValue("XB", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "XB kw missing", NULL);
   SM_Hex2Buffer(kwValue, &freeCTI.m_X);
   
   if ((lRet = SM_GetKwValue("ParametersFile", kwValue, pszConfigSection))
         != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "ParametersFile kw missing", NULL);
   bufferParameters.SetFileName(kwValue);

   if ((lRet = SM_GetKwValue("EMEKFile", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "EMEKFile kw missing", NULL);
   bufferEMEK.SetFileName(kwValue);

   if ((lRet = SM_GetKwValue("UKMFile", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "UKMFile kw missing", NULL);
   bufferUKM.SetFileName(kwValue);

   if ((lRet = SM_GetKwValue("MEKInBuffer", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "MEKInBuffer kw missing", NULL);
   if (kwValue[0] == '1')
      bMEKInFile = false;

   if ((lRet = SM_GetKwValue("MEKFile", kwMEKFile, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "MEKFile kw missing", NULL);
   if (bMEKInFile)
      bufferMEK.SetFileName(kwMEKFile);

   // convert to memory if requested
   if (!bInputsInFile)
   {
      bufferEMEK.ConvertFileToMemory();
      bufferParameters.ConvertFileToMemory();
      bufferUKM.ConvertFileToMemory();
   }
   
   // now we're ready for the actual test
   SME(lRet = freeCTI.SMTI_ExtractMEK(&bufferOriginator, &bufferParameters,
         &bufferEMEK, &bufferUKM, &bufferMEK));
   if (!bMEKInFile)
      bufferMEK.ConvertMemoryToFile(kwMEKFile);
   if (output_flag >= 0)
      printf("SM_FreeExtractMEKTest: MEK generated!\n");

   SME_FINISH_CATCH

   // TBD?  What do we need to cleanup?
   SM_SetCfgFn(pszOrigCfgFile);
   return lRet;
}

SM_RET_VAL SM_FreeVerifyTest(char *pszConfigFile, char *pszConfigSection,
                              long output_flag, long verify_flag)
{
   KwValue kwValue;
   SM_RET_VAL lRet = SM_NO_ERROR;
   char *pszOrigCfgFile = SM_SetCfgFn(pszConfigFile); /* set config file */
   bool bInputsInFile = true;
   CSM_Buffer bufferSignerKey, bufferData, bufferSignature, bufferTemp;
   CSM_OID oidSha1(id_sha1);
   CSM_OID oidDsa(id_dsa);
   CSM_Alg algDigest(oidSha1);
   CSM_Alg algSignature(oidDsa);
   byte *pbyte;

   SME_SETUP("SM_FreeVerifyTest");

   CSM_Free freeCTI;

   if ((lRet = SM_GetKwValue("InputsInFile", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "InputsInFile kw missing", NULL);
   if (kwValue[0] == '1')
      bInputsInFile = false;

   // get Y and store it in memory buffer
   if ((lRet = SM_GetKwValue("Y", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "Y kw missing", NULL);
   bufferSignerKey.SetFileName(kwValue);
   bufferSignerKey.ConvertFileToMemory();

   // get p, q, and g and store them in the class
   if ((lRet = SM_GetKwValue("P", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "P kw missing", NULL);
   bufferTemp.SetFileName(kwValue);
   pbyte = (byte *)(bufferTemp.AccessAll());
   freeCTI.m_DSAP.BERDecode(pbyte);

   if ((lRet = SM_GetKwValue("Q", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "Q kw missing", NULL);
   bufferTemp.SetFileName(kwValue);
   pbyte = (byte *)(bufferTemp.AccessAll());
   freeCTI.m_DSAQ.BERDecode(pbyte);
   
   if ((lRet = SM_GetKwValue("G", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "G kw missing", NULL);
   bufferTemp.SetFileName(kwValue);
   pbyte = (byte *)(bufferTemp.AccessAll());
   freeCTI.m_DSAG.BERDecode(pbyte);

   if ((lRet = SM_GetKwValue("Data", kwValue, pszConfigSection))
         != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "Data kw missing", NULL);
   bufferData.SetFileName(kwValue);

   if ((lRet = SM_GetKwValue("Signature", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "Signature kw missing", NULL);
   bufferSignature.SetFileName(kwValue);

   // convert to memory if requested
   if (!bInputsInFile)
   {
      bufferData.ConvertFileToMemory();
      bufferSignature.ConvertFileToMemory();
   }
   
   // now we're ready for the actual test
   // TBD, pass in ASN.1 encoded CSM_AlgID
   SME(lRet = freeCTI.SMTI_Verify(&bufferSignerKey, &algDigest,
         &algSignature, &bufferData, &bufferSignature));
   if (lRet != SM_NO_ERROR)
      SME_THROW(lRet, "Signature verification failed", NULL);
   if (output_flag >= 0)
      printf("SM_FreeVerifyTest: Signature Verified!\n");

   SME_FINISH_CATCH

   // TBD?  What do we need to cleanup?
   SM_SetCfgFn(pszOrigCfgFile);
   return lRet;
}

SM_RET_VAL SM_FreeGenerateDSAKeys(char *pszConfigFile, char *pszConfigSection,
                              long output_flag, long verify_flag)
{
   KwValue kwValue;
   SM_RET_VAL lRet = SM_NO_ERROR;
   char *pszOrigCfgFile = SM_SetCfgFn(pszConfigFile); /* set config file */
   bool bInputsInFile = true;
   int nKeyBits;
   bool bGenParams = false;
   CSM_Buffer bufferP, bufferQ, bufferG;
   CSM_Buffer bufferY, bufferX;
   Integer *p, *q, *g;
   DSAPrivateKey *x;
   KwValue kwP, kwQ, kwG, kwParamsFile;
   CSM_Buffer bufferParams;
   byte b[2048];
   unsigned int nLen;
   bool bReadParameters = false;

   SME_SETUP("SM_FreeGenerateDSAKeys");

   CSM_Free freeCTI;

   if ((lRet = SM_GetKwValue("PrivateKeyFile", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "PrivateKeyFile kw missing", NULL);
   bufferX.SetFileName(kwValue);

   if ((lRet = SM_GetKwValue("PublicKeyFile", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "PublicKeyFile kw missing", NULL);
   bufferY.SetFileName(kwValue);

   if ((lRet = SM_GetKwValue("ReadParameters", kwValue, pszConfigSection))
            != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "ReadParameters kw missing", NULL);
   if (kwValue[0] == '1')
      bReadParameters = true;

   // get p, q, and g and store them in the class
   if ((lRet = SM_GetKwValue("p", kwP, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "p kw missing", NULL);
   if (strchr(kwP, '/') == NULL)
      SM_Hex2Buffer(kwP, &bufferP);
   else
      bGenParams = true;
   if ((lRet = SM_GetKwValue("q", kwQ, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "q kw missing", NULL);
   if (strchr(kwQ, '/') == NULL)
      SM_Hex2Buffer(kwQ, &bufferQ);
   else
      bGenParams = true;
   if ((lRet = SM_GetKwValue("g", kwG, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "g kw missing", NULL);
   if (strchr(kwG, '/') == NULL)
      SM_Hex2Buffer(kwG, &bufferG);
   else
      bGenParams = true;

   if ((!bGenParams) || (bReadParameters))
   {
      if (bReadParameters)
      {
         p = new Integer;
         bufferP.SetFileName(kwP);
         bufferP.ConvertFileToMemory();
         byte *pbyte = (byte *)(bufferP.AccessAll());
         p->BERDecode(pbyte);
         q = new Integer;
         bufferQ.SetFileName(kwQ);
         bufferQ.ConvertFileToMemory();
         pbyte = (byte *)(bufferQ.AccessAll());
         q->BERDecode(pbyte);
         g = new Integer;
         bufferG.SetFileName(kwG);
         bufferG.ConvertFileToMemory();
         pbyte = (byte *)(bufferG.AccessAll());
         g->BERDecode(pbyte);
      }
      else
      {
         // convert parameters to crypto++ integers
         p = new Integer((const unsigned char*)bufferP, bufferP.Length());
         q = new Integer((const unsigned char*)bufferQ, bufferQ.Length());
         g = new Integer((const unsigned char*)bufferG, bufferG.Length());
      }

      // now we're ready to generate the key pair
      x = new DSAPrivateKey(*(freeCTI.m_pRng), *p, *q, *g);
   }
   else
   {
      if ((lRet = SM_GetKwValue("keybits", kwValue, pszConfigSection))
            != SM_NO_ERROR)
         SME_THROW(SM_UNKNOWN_ERROR, "keybits kw missing", NULL);
      nKeyBits = atoi(kwValue);

      x = new DSAPrivateKey(*(freeCTI.m_pRng), nKeyBits);
   }

   // TBD, had to make DSAPrivateKey.x public to do this
   nLen = x->x.DEREncode(&b[0]);
   bufferX.Open(SM_FOPEN_WRITE);
   bufferX.Write((char *)(&b[0]), nLen);
   bufferX.Close();
   if (output_flag >= 0)
      printf("SM_FreeGenerateDSAKeys: X Length = %d\n", nLen);

   // TBD, had to make DSAPublicKey.y public to do this
   nLen = x->y.DEREncode(&b[0]);
   bufferY.Open(SM_FOPEN_WRITE);
   bufferY.Write((char *)(&b[0]), nLen);
   bufferY.Close();
   if (output_flag >= 0)
      printf("SM_FreeGenerateDSAKeys: Y Length = %d\n", nLen);

   // write params if necessary
   if ((bGenParams) && (!bReadParameters))
   {
      // write P to file
      nLen = x->p.DEREncode(&b[0]);
      bufferP.SetFileName(kwP);
      bufferP.Open(SM_FOPEN_WRITE);
      bufferP.Write((char *)(&b[0]), nLen);
      bufferP.Close();
      // write Q to file
      nLen = x->q.DEREncode(&b[0]);
      bufferQ.SetFileName(kwQ);
      bufferQ.Open(SM_FOPEN_WRITE);
      bufferQ.Write((char *)(&b[0]), nLen);
      bufferQ.Close();
      // write G to file
      nLen = x->g.DEREncode(&b[0]);
      bufferG.SetFileName(kwG);
      bufferG.Open(SM_FOPEN_WRITE);
      bufferG.Write((char *)(&b[0]), nLen);
      bufferG.Close();
      if (output_flag >= 0)
         printf("SM_FreeGenerateDSAKeys: Wrote P,Q, and G\n");
   }

   // encode parameters into file
   if (SM_GetKwValue("EncodedParamsFile", kwParamsFile, 
         pszConfigSection) == SM_NO_ERROR)
   {
      // parameters requested to be encoded into file
      FREE_DSAParameters snaccParams;
      nLen = x->p.Encode(&b[0], 128);
      snaccParams.p.Set((const char *)(&b[0]), nLen);
      nLen = x->q.Encode(&b[0], 20);
      snaccParams.q.Set((const char *)(&b[0]), nLen);
      nLen = x->g.Encode(&b[0], 128);
      snaccParams.g.Set((const char *)(&b[0]), nLen);
      ENCODE_BUF_NO_ALLOC((&snaccParams), (&bufferParams));
      bufferParams.ConvertMemoryToFile(kwParamsFile);
      if (output_flag == 1)
         printf("SM_FreeGenerateDSAKeys: params in %s\n", kwParamsFile);
   }

   SME_FINISH_CATCH

   // TBD?  What do we need to cleanup?
   SM_SetCfgFn(pszOrigCfgFile);
   return lRet;
}

SM_RET_VAL SM_FreeGenerateDHKeys(char *pszConfigFile, char *pszConfigSection,
                              long output_flag, long verify_flag)
{
   KwValue kwValue;
   SM_RET_VAL lRet = SM_NO_ERROR;
   char *pszOrigCfgFile = SM_SetCfgFn(pszConfigFile); /* set config file */
   bool bInputsInFile = true;
   int nKeyBits;
   bool bGenParams = false;
   CSM_Buffer bufferP, bufferG;
   CSM_Buffer bufferY, bufferX;
   Integer *p, *g;
   DH *pDH;
   KwValue kwP, kwG, kwParamsFile;
   CSM_Buffer bufferParams;
   byte b[2048];
   unsigned int nLen;
   Integer y;

   SME_SETUP("SM_FreeGenerateDHKeys");

   CSM_Free freeCTI;

   if ((lRet = SM_GetKwValue("PrivateKeyFile", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "PrivateKeyFile kw missing", NULL);
   bufferX.SetFileName(kwValue);

   if ((lRet = SM_GetKwValue("PublicKeyFile", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "PublicKeyFile kw missing", NULL);
   bufferY.SetFileName(kwValue);

   // get p, and g and store them in the class
   if ((lRet = SM_GetKwValue("p", kwP, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "p kw missing", NULL);
   if (strchr(kwP, '/') == NULL)
      SM_Hex2Buffer(kwP, &bufferP);
   else
      bGenParams = true;
   if ((lRet = SM_GetKwValue("g", kwG, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "g kw missing", NULL);
   if (strchr(kwG, '/') == NULL)
      SM_Hex2Buffer(kwG, &bufferG);
   else
      bGenParams = true;

   // setup and create the DH class
   if (!bGenParams)
   {
      // convert parameters to crypto++ integers
      p = new Integer((const unsigned char*)bufferP, bufferP.Length());
      g = new Integer((const unsigned char*)bufferG, bufferG.Length());

      pDH = new DH(*p, *g); // create based on preset params
   }
   else
   {
      if ((lRet = SM_GetKwValue("keybits", kwValue, pszConfigSection))
            != SM_NO_ERROR)
         SME_THROW(SM_UNKNOWN_ERROR, "keybits kw missing", NULL);
      nKeyBits = atoi(kwValue);

      pDH = new DH(*(freeCTI.m_pRng), nKeyBits);
   }

   // generate the key pair
   pDH->Setup(*(freeCTI.m_pRng), &b[0]);

   // decode the y value from b into y
   y.Decode(&b[0], pDH->OutputLength());
   // DEREncode the public value from Y back into b, then write it
   nLen = y.DEREncode(&b[0]);
   bufferY.Open(SM_FOPEN_WRITE);
   bufferY.Write((char *)(&b[0]), nLen);
   bufferY.Close();
   if (output_flag >= 0)
      printf("SM_FreeGenerateDHKeys: Y Length = %d\n", nLen);

   // TBD, had to make DH.x public to do this
   nLen = pDH->x.DEREncode(&b[0]);
   bufferX.Open(SM_FOPEN_WRITE);
   bufferX.Write((char *)(&b[0]), nLen);
   bufferX.Close();
   if (output_flag >= 0)
      printf("SM_FreeGenerateDHKeys: X Length = %d\n", nLen);

   // write params if necessary
   if (bGenParams)
   {
      // write P to file
      nLen = pDH->Prime().DEREncode(&b[0]);
      bufferP.SetFileName(kwP);
      bufferP.Open(SM_FOPEN_WRITE);
      bufferP.Write((char *)(&b[0]), nLen);
      bufferP.Close();
      // write G to file
      nLen = pDH->Generator().DEREncode(&b[0]);
      bufferG.SetFileName(kwG);
      bufferG.Open(SM_FOPEN_WRITE);
      bufferG.Write((char *)(&b[0]), nLen);
      bufferG.Close();
      if (output_flag >= 0)
         printf("SM_FreeGenerateDHKeys: Wrote P and G\n");
   }

   // encode parameters into file
   if (SM_GetKwValue("EncodedParamsFile", kwParamsFile, 
         pszConfigSection) == SM_NO_ERROR)
   {
      // parameters requested to be encoded into file
      DHParameters snaccParams;
      nLen = pDH->Prime().Encode(&b[0], 128);
      snaccParams.p.Set((const char *)(&b[0]), nLen);
      nLen = pDH->Generator().Encode(&b[0], 128);
      snaccParams.g.Set((const char *)(&b[0]), nLen);
      ENCODE_BUF_NO_ALLOC((&snaccParams), (&bufferParams));
      bufferParams.ConvertMemoryToFile(kwParamsFile);
      if (output_flag == 1)
         printf("SM_FreeGenerateDHKeys: params in %s\n", kwParamsFile);
   }

   SME_FINISH_CATCH

   // TBD?  What do we need to cleanup?
   SM_SetCfgFn(pszOrigCfgFile);
   return lRet;
}

#endif // ifdef SM_FREE_USED

// EOF sm_CfgFree.cpp

