#ifndef NO_SCCS_ID
static char SccsId[ ] = "@(#) sm_CfgRsa.cpp 1.3 3/30/98 17:09:01"; 
#endif
// sm_CfgRsa.cpp
#include "sm_util.h"

#ifdef SM_RSA_USED
#include "sm_rsa.h"
#include "sm_rsa_asn.h"

SM_RET_VAL SM_RsaEncryptTest(char *pszConfigFile, char *pszConfigSection,
                              long output_flag, long verify_flag)
{
   KwValue kwValue;
   SM_RET_VAL lRet = SM_NO_ERROR;
   char *pszOrigCfgFile = SM_SetCfgFn(pszConfigFile); /* set config file */
   bool bInFile = true;
   KwValue kwPlainDataFile, kwEncryptedDataFile, kwParametersFile, kwMEKFile;
   CSM_Rsa rsaCTI;
   CSM_Buffer bufferData, bufferEncryptedData, bufferParameters, bufferMEK;

   SME_SETUP("SM_RsaEncryptTest");

   if (output_flag == 1)
      printf("SM_RsaEncryptTest: reading config entries\n");
   if ((lRet = SM_GetKwValue("EncryptInBuffer", kwValue, 
         pszConfigSection)) != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "EncryptInBuffer keyword missing", NULL);
   if (kwValue[0] == '1')
      bInFile = false;

   if ((lRet = SM_GetKwValue("PlainData", kwPlainDataFile, 
         pszConfigSection)) != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "PlainData keyword missing", NULL);

   if ((lRet = SM_GetKwValue("EncryptedData", kwEncryptedDataFile, 
                  pszConfigSection)) != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "EncryptedData keyword missing", NULL);

   if ((lRet = SM_GetKwValue("ParametersFile", kwParametersFile, 
                  pszConfigSection)) != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "ParametersFile keyword missing", NULL);

   if ((lRet = SM_GetKwValue("MEKFile", kwMEKFile,
                     pszConfigSection)) != SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "MEKFile keyword missing", NULL);

   // now we're ready for the actual test
   // setup data to file
   bufferData.SetFileName(kwPlainDataFile);
   if (bInFile)
      // setup encrypted data to file
      bufferEncryptedData.SetFileName(kwEncryptedDataFile);
   else
      // if data should be in memory, copy it in from file
      bufferData.ConvertFileToMemory();

   if (output_flag == 1)
      printf("SM_RsaEncryptTest:  calling SMTI_Encrypt\n");
   SME(lRet = rsaCTI.
         SMTI_Encrypt(&bufferData, &bufferEncryptedData, 
               &bufferParameters, &bufferMEK));

   if (output_flag > -1)
      printf("SM_RsaEncryptTest success!\n");
   if (!bInFile)
      bufferEncryptedData.
            ConvertMemoryToFile(kwEncryptedDataFile);
   bufferParameters.ConvertMemoryToFile(kwParametersFile);
   // write MEK to file
   bufferMEK.ConvertMemoryToFile(kwMEKFile);

   SME_FINISH_CATCH

   // TBD?  What do we need to cleanup?
   SM_SetCfgFn(pszOrigCfgFile);
   return lRet;
}


SM_RET_VAL SM_RsaRandomTest(char *pszConfigFile, char *pszConfigSection,
                              long output_flag, long verify_flag)
{
   KwValue kwValue;
   SM_RET_VAL lRet = SM_NO_ERROR;
   char *pszOrigCfgFile = SM_SetCfgFn(pszConfigFile); /* set config file */
   bool bInFile = true;
   char szResultFile[SM_PATH_LEN];
   SM_SIZE_T lLength;

   SME_SETUP("SM_RsaRandomTest");

   if ((lRet = SM_GetKwValue("TestInBuffer", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "TestInBuffer kw missing", NULL);

   if (kwValue[0] == '1')
      bInFile = false;
   if ((lRet = SM_GetKwValue("ResultFile", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "ResultInFile kw missing", NULL);
   strcpy(szResultFile, kwValue);
   if ((lRet = SM_GetKwValue("Length", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "Length kw missing", NULL);
   lLength = atol(kwValue);
   // now we're ready for the actual test
   CSM_Rsa rsaCTI;
   CSM_Buffer buffer;
   if (bInFile)
      buffer.SetFileName(szResultFile);
   SME(lRet = rsaCTI.
         SMTI_Random(NULL, &buffer, lLength));
   if (!bInFile)
      buffer.ConvertMemoryToFile(szResultFile);

   SME_FINISH_CATCH

   // TBD?  What do we need to cleanup?
   SM_SetCfgFn(pszOrigCfgFile);
   return lRet;
}
            
SM_RET_VAL SM_RsaDigestTest(char *pszConfigFile, char *pszConfigSection,
                              long output_flag, long verify_flag)
{
   KwValue kwValue;
   SM_RET_VAL lRet = SM_NO_ERROR;
   char *pszOrigCfgFile = SM_SetCfgFn(pszConfigFile); /* set config file */
   bool bInFile = true;
   char szDigestFile[SM_PATH_LEN];
   CSM_Buffer bufferData;
   CSM_Buffer bufferDigest;
   CSM_OID oidMd5(bsafe_id_md5);

   SME_SETUP("SM_RsaDigestTest");

   if ((lRet = SM_GetKwValue("TestInBuffer", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "TestInBuffer kw missing", NULL);
   if (kwValue[0] == '1')
      bInFile = false;
   if ((lRet = SM_GetKwValue("Data", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "Data kw missing", NULL);
   bufferData.SetFileName(kwValue);
   if (!bInFile)
      bufferData.ConvertFileToMemory();
   if ((lRet = SM_GetKwValue("Digest", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "Digest kw missing", NULL);
   strcpy(szDigestFile, kwValue);
   if (bInFile)
      bufferDigest.SetFileName(szDigestFile);
   if ((lRet = SM_GetKwValue("SHA1", kwValue, pszConfigSection)) !=
         SM_NO_ERROR)
      SME_THROW(SM_UNKNOWN_ERROR, "SHA1 kw missing", NULL);
   CSM_Rsa rsaCTI;
   if (kwValue[0] != '1')
      rsaCTI.BTISetPreferredCSInstAlgs(&oidMd5, NULL, NULL, NULL);
   // now we're ready for the actual test
   SME(lRet = rsaCTI.
         SMTI_DigestData(&bufferData, &bufferDigest));
   if (!bInFile)
      bufferDigest.ConvertMemoryToFile(szDigestFile);

   SME_FINISH_CATCH

   // TBD?  What do we need to cleanup?
   SM_SetCfgFn(pszOrigCfgFile);
   return lRet;
}
            

#endif // ifdef SM_RSA_USED

// EOF sm_CfgRsa.cpp

