#ifndef NO_SCCS_ID
static char SccsId[ ] = "@(#) sm_CfgSupport.cpp 1.3 3/27/98 18:38:30"; 
#endif

//
//
// sm_CfgSupport.cpp
// This set of support functions handle the configuration file read operations.
// This includes reading the certificate and CRL lists and assigning the
// certificate path issuers.
//

#include "sm_util.h"

SM_RET_VAL SM_CfgCertsSection(char *lpszSection, CSM_MsgCertCrls *pMsgCertsCrls)
{
    SM_RET_VAL status=0;
    KwValue kwValue;
    Section lpszlSection;
    CSM_CertificateChoice *pTmpCert;
    CSM_Buffer *pTmpBuf;

    strcpy(lpszlSection, lpszSection);   // get copy of parameter.
    if (pMsgCertsCrls != NULL)
    {
       while (strlen(lpszSection) && !strstr(kwValue, "NULL"))
       {
         if (SM_GetKwValue("CertificateFILE", kwValue, 
              lpszSection) == SM_NO_ERROR && !strstr(kwValue, "NULL"))
         {
             pTmpBuf = NULL;
             status = SM_CfgAliasCertFile(kwValue, pTmpBuf);
                                        // RESOLVE alias if necessary.
             pTmpCert = new CSM_CertificateChoice(pTmpBuf, 
                 CSM_CertificateChoice::CERT_TYPE_CERT);
         }
         else if (SM_GetKwValue("CertificateBUFFER", kwValue, 
              lpszSection) == SM_NO_ERROR && !strstr(kwValue, "NULL"))
         {
             pTmpBuf = NULL;
             if ((status = SM_CfgAliasCertFile(kwValue, pTmpBuf)) == 0)
                                        // RESOLVE alias if necessary.
             {
               if (pTmpBuf)
                 pTmpBuf->ConvertFileToMemory();
               pTmpCert = new CSM_CertificateChoice(pTmpBuf, 
                   CSM_CertificateChoice::CERT_TYPE_CERT);
             }
         }
         else if (SM_GetKwValue("CertificateDN", kwValue, 
              lpszSection) == SM_NO_ERROR && !strstr(kwValue, "NULL"))
         {
             pTmpBuf = NULL;
             status = SM_CfgAliasCertDN(kwValue, pTmpBuf);
                                        // RESOLVE alias if necessary.
             pTmpCert = new CSM_CertificateChoice(pTmpBuf, 
                 CSM_CertificateChoice::CERT_TYPE_CERT);
         }

         if (pTmpCert)
            pMsgCertsCrls->AddCert(pTmpCert);
         
         // CHECK to see if we are requested to fill in the cert path.
         if (SM_GetKwValue("CertificatePath", kwValue, 
              lpszSection) == SM_NO_ERROR && !strstr(kwValue, "NULL"))
         {                      // then fill in the entire certificate path.
             // Assume that "pTmpBuf" has been filled, then fill remainder of 
             //   Certificate path.
             if (strcmp(kwValue, "FILL") == 0 || strcmp(kwValue, "fill") == 0)
             {
                 status = p_GAddressBook->FillCertPath(pTmpCert, pMsgCertsCrls);
            }
         }



         if (pTmpBuf)
             delete pTmpBuf;
         if (SM_GetKwValue("Next", kwValue, lpszSection) == SM_NO_ERROR)
             strcpy(lpszlSection, kwValue);

       }        // END while cert sections to be input.

    }

    return(status);
}


// This function returns the file name corresponding to the indicated alias
// name.  It is expected to be in the address book.  The indicated name may 
// already be a file name (as indicated by a "/" OR "\" in the name) so it will 
// not be translated.
SM_RET_VAL SM_CfgAliasCertFile(char *lpszAlias, CSM_Buffer *&pCertBuf)
{
   int status=0;
   char *section="";
   bool alloced=false;


   /** next, process the potential presence of an alias.  This is indicated
       by the lack of a "/" for directory indication.  If a file is in the
       default directory, it can be specified by "./file". **/
   if (strchr(lpszAlias, '/') == NULL && strchr(lpszAlias, '\\') == NULL)
   {
        if (pCertBuf == NULL)
        {
           pCertBuf = new CSM_Buffer;
           alloced = true;
        }
        if ((status=p_GAddressBook->FindCertAlias(lpszAlias, pCertBuf)) != 0)
        {
            if (alloced)
              delete pCertBuf;
            status = -1;
            printf("SM_CfgAliasCertFile: bad FindCertAlias on %s.\n", lpszAlias);
        }
   }
   else
   {
     if (pCertBuf == NULL)
         pCertBuf = new CSM_Buffer(lpszAlias);  // use file name directly.
     else
         pCertBuf->SetFileName(lpszAlias);
   }

   return(status);
}


// This function returns the file name corresponding to the indicated DN
// name.  It is expected to be in the address book.  
SM_RET_VAL SM_CfgAliasCertDN(char *lpszDN, CSM_Buffer *&pCertBuf)
{
   int status=0;
   char *section="";
   bool alloced=false;


        if (pCertBuf == NULL)
        {
           pCertBuf = new CSM_Buffer;
           alloced = true;
        }
        if ((status=p_GAddressBook->FindCertDN(lpszDN, pCertBuf)) != 0)
        {
            if (alloced)
              delete pCertBuf;
            status = -1;
            printf("SM_CfgAliasCertDN: bad FindCertAlias on %s.\n", lpszDN);
        }

   return(status);
}
   





// EOF sm_CfgSupport.cpp
